!/*
!* ZZ-HIFU-K version 1.00,  11 Feb. 2013
!*
!* Copyright (c) 2013 the University of Tokyo, Japan. 
!*
!* Research and Development
!*  of the Next-Generation Integrated Simulation of Living Matter
!*
!* Department of Mechanical Engineering, School of Engineering, the University of Tokyo
!*
!* Principal developer: Yoichiro Matsumoto (Professor, the University of Tokyo)
!*
!* Signatured by Kohei Okita (okita.kohei@nihon-u.ac.jp)
!*/

      module physical_properties
      real*8, parameter :: p_inf = 0.1d+6 ! Ambient pressure [Pa]
      real*8, parameter :: T_inf = 308.15
      real*8, dimension(0:255) :: Cs0, Rho0, Gamma, rGamma, A1, Coef_Rho, Beta, Cp, Kappa, Mu, Lambda
      integer, dimension(0:255) :: AirID
      end

! Adipose tissue
#define _ADIPOSE_ \
      Cs0(ID) = 1450.d0 ; Rho0(ID) = 950.0d0 ; Gamma(ID) = 10.0d0+1.d0 ; Beta(ID) = 2.1d-4 ; Cp(ID) = 3100.d0 ; Kappa(ID) = 0.27d0  ; Mu(ID) = 29.d0/0.22d0*1.e-3 ;

! Blood (whole)
#define _BLOOD_ \
      Cs0(ID) = 1584.d0 ; Rho0(ID) = 1060.d0 ; Gamma(ID) =  6.1d0+1.d0 ; Beta(ID) = 2.1d-4 ; Cp(ID) = 3720.d0 ; Kappa(ID) = 0.510d0 ; Mu(ID) = 20.d0/0.22d0*1.e-3 ;

! Brain
#define _BRAIN_ \
      Cs0(ID) = 1560.d0 ; Rho0(ID) = 1040.d0 ; Gamma(ID) =  7.1d0+1.d0 ; Beta(ID) = 2.1d-4 ; Cp(ID) = 3640.d0 ; Kappa(ID) = 0.540d0 ; Mu(ID) = 60.d0/0.22d0*1.e-3 ;

! Breast
#define _BREAST_ \
      Cs0(ID) = 1510.d0 ; Rho0(ID) = 1020.d0 ; Gamma(ID) =  0.0d0+1.d0 ; Beta(ID) = 2.1d-4 ; Cp(ID) =         ; Kappa(ID) = 0.499d0 ; Mu(ID) = 75.d0/0.22d0*1.e-3 ;

! Cortical bone
#define _BONE_ \
      Cs0(ID) = 3635.d0 ; Rho0(ID) = 1920.d0 ; Gamma(ID) =  0.0d0+1.d0 ; Beta(ID) = 0.0d0  ; Cp(ID) = 1300.d0 ; Kappa(ID) = 0.790d0 ; Mu(ID) =  0.d0/0.22d0*1.e-3 ;

! Liver
#define _LIVER_ \
      Cs0(ID) = 1595.d0 ; Rho0(ID) = 1060.d0 ; Gamma(ID) =  6.6d0+1.d0 ; Beta(ID) = 2.1d-4 ; Cp(ID) = 3600.d0 ; Kappa(ID) = 0.524d0 ; Mu(ID) = 50.d0/0.22d0*1.e-3 ;

! Muscle-skeletal
#define _MUSCLE_ \
      Cs0(ID) = 1580.d0 ; Rho0(ID) = 1050.d0 ; Gamma(ID) =  6.6d0+1.d0 ; Beta(ID) = 2.1d-4 ; Cp(ID) = 3465.d0 ; Kappa(ID) = 0.498d0 ; Mu(ID) = 74.d0/0.22d0*1.e-3 ;

! Skin
#define _SKIN_ \
      Cs0(ID) = 1615.d0 ; Rho0(ID) = 1090.d0 ; Gamma(ID) =  7.9d0+1.d0 ; Beta(ID) = 2.1d-4 ; Cp(ID) = 3430.d0 ; Kappa(ID) = 0.335d0 ; Mu(ID) =  0.d0/0.22d0*1.e-3 ;

! fatty
#define _FATTY_ \
      Cs0(ID) = 1465.d0 ; Rho0(ID) = 985.0d0 ; Gamma(ID) =  8.5d0+1.d0 ; Beta(ID) = 2.1d-4 ; Cp(ID) = 3000.d0 ; Kappa(ID) = 0.350d0 ; Mu(ID) = 40.d0/0.22d0*1.e-3 ;

! non-fatty
#define _NON_FATTY_ \
      Cs0(ID) = 1575.d0 ; Rho0(ID) = 1055.d0 ; Gamma(ID) =  7.0d0+1.d0 ; Beta(ID) = 2.1d-4 ; Cp(ID) = 3550.d0 ; Kappa(ID) = 0.525d0 ; Mu(ID) = 60.d0/0.22d0*1.e-3 ;

! water@20C
#define _WATER_ \
      Cs0(ID) = 1482.d0 ; Rho0(ID) = 998.2d0 ; Gamma(ID) = 4.96d0+1.d0 ; Beta(ID) = 2.1d-4 ; Cp(ID) = 4182.d0 ; Kappa(ID) = 0.600d0 ; Mu(ID) = 0.22d0/0.22d0*1.e-3 ;

! air
#define _AIR_ \
                          Rho0(ID) = 1.184d0 ; Gamma(ID) = 1.4d0 ; Beta(ID) = 1.d0/(273.15d0+25.d0) ; Cp(ID) = 1007.d0 ; Kappa(ID) = 0.026d0 ; Mu(ID) = 0.0182d0*1.e-3 ; \
      Cs0(ID) = DSQRT( Gamma(ID) * p_inf / Rho0(ID) ) ; AirID(ID) = 1 ;

! Acoustic Lens
#define _ACOUSTIC_LENS_ \
      Cs0(ID) =  960.d0 ; Rho0(ID) = 1380.d0 ; Gamma(ID) =  0.0d0+1.d0 ; Beta(ID) = 0.0d0  ; Cp(ID) = 4182.d0 ; Kappa(ID) = 0.600d0 ; Mu(ID) =  0.d0/0.22d0*1.e-3 ;

! Acrylic
#define _ACRYLIC_ \
      Cs0(ID) = 2750.d0 ; Rho0(ID) = 1190.d0 ; Gamma(ID) =  0.0d0+1.d0 ; Beta(ID) = 0.0d0  ; Cp(ID) = 4182.d0 ; Kappa(ID) = 0.600d0 ; Mu(ID) =  0.d0/0.22d0*1.e-3 ;

#define _HARD_BONE_PLATE_ \
      Cs0(ID) = 4325.97d0 ; Rho0(ID) = 1810.d0 ; Gamma(ID) =  0.0d0+1.d0 ; Beta(ID) = 0.0d0  ; Cp(ID) = 1300.d0 ; Kappa(ID) = 0.790d0 ; Mu(ID) =  0.d0/0.22d0*1.e-3 ;

#define _SOFT_BONE_PLATE_ \
      Cs0(ID) = 2717.39d0 ; Rho0(ID) = 1380.d0 ; Gamma(ID) =  0.0d0+1.d0 ; Beta(ID) = 0.0d0  ; Cp(ID) = 1300.d0 ; Kappa(ID) = 0.790d0 ; Mu(ID) =  0.d0/0.22d0*1.e-3 ;

! Piezocomposite
#define _PIEZO_COMPOSITE_ \
      Cs0(ID) = 3550.0d0 ; Rho0(ID) = 3098.6d0 ; Gamma(ID) =  0.0d0+1.d0 ; Beta(ID) = 0.0d0  ; Cp(ID) = 4182.d0 ; Kappa(ID) = 0.600d0 ; Mu(ID) =  0.d0/0.22d0*1.e-3 ;
! Plate
#define _ACRYLIC_PLATE_ \
      Cs0(ID) = 2660.d0 ; Rho0(ID) = 1203.d0 ; Gamma(ID) =  0.0d0+1.d0 ; Beta(ID) = 0.0d0  ; Cp(ID) = 4182.d0 ; Kappa(ID) = 0.600d0 ; Mu(ID) =  0.d0/0.22d0*1.e-3 ;

      subroutine set_physical_properties
      use physical_properties
      implicit none

      integer :: ID

      do ID = 0, 255
         _WATER_
         AirID(ID) = 0
      enddo

      ID =   0 ; _WATER_
      ID =   2 ; _ACRYLIC_PLATE_
      ID = 255 ; _PIEZO_COMPOSITE_
!#define _BONE_PLATE_
#ifdef _BONE_PLATE_
! Bone
      ID =   1 ; _SOFT_BONE_PLATE_
#endif
      ID =  15 ; _NON_FATTY_
      ID =  18 ; _AIR_
      ID =  20 ; _AIR_
      ID =  21 ; _AIR_
      ID =  23 ; _WATER_
      ID =  24 ; _MUSCLE_
      ID =  25 ; _NON_FATTY_
      ID =  26 ; _LIVER_
      ID =  27 ; _AIR_
      ID =  29 ; _FATTY_
      ID =  31 ; _AIR_
      ID =  32 ; _NON_FATTY_
      ID =  33 ; _AIR_
      ID =  39 ; _AIR_
      ID =  45 ; _BONE_
      ID =  48 ; _ADIPOSE_
      ID =  49 ; _MUSCLE_
      ID =  51 ; _SKIN_
      ID =  55 ; _MUSCLE_
      ID =  80 ; _BRAIN_
      ID =  84 ; _NON_FATTY_
      ID =  85 ; _AIR_
      ID =  94 ; _NON_FATTY_

      do ID = 0, 255
         rGamma(ID) = 1.d0 / Gamma(ID) ;
         A1(ID) = Rho0(ID) * Cs0(ID)**2 / Gamma(ID) - p_inf ;
         Coef_Rho(ID) = Rho0(ID) / ( p_inf + A1(ID) )**rGamma(ID) ;
         Lambda(ID) = 2.d0 * Mu(ID) ;
      enddo
      return
      end

