!/*
!* ZZ-HIFU-K version 1.00,  11 Feb. 2013
!*
!* Copyright (c) 2013 the University of Tokyo, Japan. 
!*
!* Research and Development
!*  of the Next-Generation Integrated Simulation of Living Matter
!*
!* Department of Mechanical Engineering, School of Engineering, the University of Tokyo
!*
!* Principal developer: Yoichiro Matsumoto (Professor, the University of Tokyo)
!*
!* Signatured by Kohei Okita (okita.kohei@nihon-u.ac.jp)
!*/

! Use MPI
c#define _USE_MPI_
!
! Use buffer send recieve
c#define _BUFFER_SENDRECEIVE_
c#define BUFFER_J
c#define BUFFER_K
!
c#define _TEST_
#ifdef _TEST_
! Check results for sample input
c#define _TEST_CHECK_DATA_
! The size and division are obtained from command argument
c#define _ARGUMENT_
#endif
!
! Number of computational grid
#ifndef _NX_
#define _NX_ 1400
#endif
#ifndef _NY_
#define _NY_ 1200
#endif
#ifndef _NZ_
#define _NZ_ 1200
#endif
! Number of grid for PML
#ifndef _NPML_
#define _NPML_ 20
#endif
! Grid resolution dx = dy = dz
#ifndef _PITCH_
#define _PITCH_ .1d-3
#endif
! Division of numerical domain
! NPROCS must be _IDIM_ * _JDIM_ * _KDIM_
#ifndef _IDIM_
#define _IDIM_ 8
#endif
#ifndef _JDIM_
#define _JDIM_ 2
#endif
#ifndef _KDIM_
#define _KDIM_ 4
#endif
! Total time step
#ifndef _NSTEPMAX_
#define _NSTEPMAX_ 10000
#endif
#ifndef _CHECK_IN_
#define _CHECK_IN_  9000
#endif
! Data save
#ifndef _DATA3D_SAVE_
#define _DATA3D_SAVE_ 500
#endif
#ifndef _DATA2D_SAVE_
#define _DATA2D_SAVE_ 20000
#endif

c#define _RESTART_FILE_NAME_ '("restart-",i3.3,".bin")'
c#define _RESTART_
c#define _RECALCULATION_TEMP_

c#define _HearingMode_
c#define _RecordPointsTable_ "Vcenter.table"
c#define _RecordPatPiezo_

c#define _RecordPatFocalPoint_

c#define _PhaseDelayTable_ "PhaseDelay.table"

! Position of Focal Point
#ifndef _FocalPointX_
#define _FocalPointX_ 0.d-3
#endif
#ifndef _FocalPointY_
#define _FocalPointY_ 0.d-3
#endif
#ifndef _FocalPointZ_
#define _FocalPointZ_ 0.d-3
#endif

! Using Tait eq. for the equation of state of water
#define _EOS_TAIT_

! CFL number
#ifndef _CFL_
#define _CFL_ 0.1d0
#endif

! Temperature
c#define _TEMPERATURE_
! Time scale for Temperature
#ifndef _DTAU_
#define _DTAU_ 1d+6
#endif

! Denaturation
c#define _DENATURATION_
! Increase of acoustic impedance due to denaturation
#ifndef _DENATURATION_CS_
#define _DENATURATION_CS_ 1.d0
#endif

! Order of FDM 4 or 6
#ifndef _FDM_ORDER_
c#define _FDM_ORDER_ 4
#define _FDM_ORDER_ 6
c#define _FDM_ORDER_ 8
#endif
! Unroll
#define _UNROLL_FDM_

! Viscosity term is solved implicitly
c#define _IMPLICIT_VIS_ 3

c#define _PIEZO_DATA_LIST_ '("./PiezoList/",i3.3,".list")'
c#define _PARTS_DATA_LIST_ '("./PartsList/",i3.3,".list")'
#ifdef _TEST_
c#define _OBJECT_DATA_ '("../Objects/Empty/",i3.3,"-ev.sph")'
#else
c#define _OBJECT_DATA_ '("../Objects/Liver/",i3.3,"-body_m.sph")'
#define _OBJECT_DATA_ '("../Objects/Empty/",i3.3,"-ev.sph")'
#endif
c#define _VOLUME_OBJECT_

! Pressure constant in Tissu with Air properties
#define _CONST_PRESSURE_AIR_

! Using array of constants for not binary but mixture model
#define _ARRAY_CONST_

! No heat conduction in air
c#define _ZERO_HEATCONDUCTION_AIR_
#ifdef _ZERO_HEATCONDUCTION_AIR_
#ifndef _ARRAY_CONST_
#define _ARRAY_CONST_
#endif
#endif

! Boundary Condition
#ifdef _TEST_
#define _PML_X0_
#define _PML_X1_
#define _PML_Y0_
#define _PML_Y1_
#define _PML_Z0_
#define _PML_Z1_
#else
#define _PML_X0_
#define _PML_X1_
c#define _PML_Y0_
#define _PML_Y1_
#define _PML_Z0_
#define _PML_Z1_
!
!#define _MIRROR_X0_
!#define _MIRROR_X1_
#define _MIRROR_Y0_
!#define _MIRROR_Y1_
!#define _MIRROR_Z0_
!#define _MIRROR_Z1_
#endif

#ifdef _INTEL_
#define STIME(cpu0) call clockx(cpu0)
#define ETIME(cpu1, cpu0) call clockx(cpu1) ; cpu1 = (cpu1 - cpu0)*1.d-6
#else
#define STIME(cpu0) call gettod(cpu0)
#define ETIME(cpu1, cpu0) call gettod(cpu1) ; cpu1 = (cpu1 - cpu0)*1.d-6
c#define STIME(cpu0) call cpu_time(cpu0)
c#define ETIME(cpu1, cpu0) call cpu_time(cpu1) ; cpu1 = cpu1 - cpu0
#endif

! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      module Grid
! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      implicit none
!
      real*8, parameter :: dx = (_PITCH_), rdx = 1.d0/dx
      real*8, parameter :: dy = (_PITCH_), rdy = 1.d0/dy
      real*8, parameter :: dz = (_PITCH_), rdz = 1.d0/dz
!
! <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
      end module Grid
! <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      module FDM
! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      implicit none
!
      integer*4, parameter :: nfdm = (_FDM_ORDER_)                  ! 2     ! 4     ! 6
      integer*4, parameter :: ms = -nfdm/2 + 1, me =  nfdm + ms - 1 !  0, 1 ! -1, 2 ! -2, 3
      integer*4, parameter :: ns = -nfdm/2    , ne =  nfdm + ns - 1 ! -1, 0 ! -2, 1 ! -3, 2
!
      real*8, dimension(ms:me) :: d0n, d1n
      real*8, dimension(ns:ne) :: d0m, d1m
!
      contains
!
! ----------------------------------------------------------------------
      subroutine FDM_Init
! ----------------------------------------------------------------------
      use Grid , only : dx
!
      implicit none
!
      real*8 :: xc
      real*8, dimension(ms:me) :: xmk
      integer*4 :: m
!
      do m = ms, me
         xmk(m) = ( dble(m) - .5d0 ) * dx
      enddo
!
      xc = 0.d0
!
      call FDM_Operator(xc, nfdm, xmk, d0n, d1n)
!
      call FDM_Operator(xc, nfdm, xmk, d0m, d1m)
!
      return
      end subroutine FDM_Init

! ----------------------------------------------------------------------
      subroutine FDM_Operator(xc,n,xk,d0k,d1k)
! ----------------------------------------------------------------------
      implicit none

      real*8, intent(in) :: xc
      integer*4, intent(in) :: n
      real*8, dimension(n), intent(in) :: xk
      real*8, dimension(n), intent(out) :: d0k, d1k

      real*8 :: xck, xd
      real*8 :: pi0, pi1, pis, pik, sum0, sum1
      integer*4 :: npt
      integer*4 :: m, l1, k

      PI0=1.D0
      DO M=1,n
        PI0=PI0*(xc-xk(M))
      ENDDO

      PI1=0.D0
      DO L1=1,n
        PIS=1.D0
        DO M=1,n
          IF(M.NE.L1) PIS=PIS*(xc-xk(M))
        ENDDO
        PI1=PI1+PIS
      ENDDO

      SUM0=0.D0
      SUM1=0.D0
      NPT=n
      DO K=1,n
        IF(K.NE.NPT) THEN
          xcK=xk(K)
          PIK=1.D0
          DO M=1,n
            IF(M.NE.K) PIK=PIK*(xcK-xk(M))
          ENDDO
          XD=xc-xcK
          D0k(K)=PI0/XD/PIK
          D1k(K)=(XD*PI1-PI0)/XD**2/PIK
          SUM0=SUM0+D0k(K)
          SUM1=SUM1+D1k(K)
        ENDIF
      ENDDO

      D0k(NPT)=1.D0-SUM0
      D1k(NPT)=    -SUM1

      return
      end subroutine FDM_Operator

! <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
      end module FDM
! <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      module Array_Index
! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      implicit none 
!
#ifdef _TEST_
      integer*4 :: nx = (_NX_), ny = (_NY_), nz = (_NZ_)
      integer*4 :: ista_G = 1, iend_G = (_NX_)
      integer*4 :: jsta_G = 1, jend_G = (_NY_)
      integer*4 :: ksta_G = 1, kend_G = (_NZ_)
#else
      integer*4, parameter :: nx = (_NX_), ny = (_NY_), nz = (_NZ_)
      integer*4, parameter :: ista_G = 1, iend_G = nx
      integer*4, parameter :: jsta_G = 1, jend_G = ny
      integer*4, parameter :: ksta_G = 1, kend_G = nz
#endif /* _TEST_ */
!
      integer*4 :: istam , iendm 
      integer*4 :: istan , iendn 
      integer*4 :: ista1m, iend1m
      integer*4 :: ista1n, iend1n
      integer*4 :: ista2m, iend2m
      integer*4 :: ista2n, iend2n
!
      integer*4 :: jstam , jendm 
      integer*4 :: jstan , jendn 
      integer*4 :: jsta1m, jend1m
      integer*4 :: jsta1n, jend1n
      integer*4 :: jsta2m, jend2m
      integer*4 :: jsta2n, jend2n
!
      integer*4 :: kstam , kendm 
      integer*4 :: kstan , kendn 
      integer*4 :: ksta1m, kend1m
      integer*4 :: ksta1n, kend1n
      integer*4 :: ksta2m, kend2m
      integer*4 :: ksta2n, kend2n
!
      contains
!
#ifdef _ARGUMENT_
! ----------------------------------------------------------------------
      subroutine Array_Global_Index_Init(i, j, k)
! ----------------------------------------------------------------------
      integer*4, intent(in) :: i, j, k
!
      nx = i ; ny = j ; nz = k
      ista_G = 1 ; iend_G = i
      jsta_G = 1 ; jend_G = j
      ksta_G = 1 ; kend_G = k
!
      return
      end subroutine Array_Global_Index_Init
#endif /* _ARGUMENT_ */
!
! ----------------------------------------------------------------------
      subroutine Array_Index_Init(ista, iend, jsta, jend, ksta, kend)
! ----------------------------------------------------------------------
      use FDM , only : ms, me, ns, ne
!
      implicit none
!
      integer*4, intent(in) :: ista, iend, jsta, jend, ksta, kend
!
! Mid point
      istam  = ista        ; iendm  = iend
      jstam  = jsta        ; jendm  = jend
      kstam  = ksta        ; kendm  = kend
! Node point 
      istan  = istam  - 1  ; iendn  = iendm
      jstan  = jstam  - 1  ; jendn  = jendm
      kstan  = kstam  - 1  ; kendn  = kendm
! Mid point for 1st Difference at Node point
      ista1m = istan  + ms ; iend1m = iendn  + me
      jsta1m = jstan  + ms ; jend1m = jendn  + me
      ksta1m = kstan  + ms ; kend1m = kendn  + me
! Node point for 1st Difference at Mid point
      ista1n = istam  + ns ; iend1n = iendm  + ne
      jsta1n = jstam  + ns ; jend1n = jendm  + ne
      ksta1n = kstam  + ns ; kend1n = kendm  + ne
! Mid point for 2nd Difference at Mid point
      ista2m = ista1n + ms ; iend2m = iend1n + me
      jsta2m = jsta1n + ms ; jend2m = jend1n + me
      ksta2m = ksta1n + ms ; kend2m = kend1n + me
! Node point for 2nd Difference at Node point
! Plus 1 for MPI
      ista2n = ista1m + ns ; iend2n = iend1m + ne + 1
      jsta2n = jsta1m + ns ; jend2n = jend1m + ne + 1
      ksta2n = ksta1m + ns ; kend2n = kend1m + ne + 1
!      ista2n = ista1m + ns ; iend2n = iend1m + ne
!      jsta2n = jsta1m + ns ; jend2n = jend1m + ne
!      ksta2n = ksta1m + ns ; kend2n = kend1m + ne
!
      return
      end subroutine Array_Index_Init
!
! <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
      end module Array_Index
! <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      module MyMPI
! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      implicit none
!
      integer*4 :: myrank
      integer*4 :: ista, iend, jsta, jend, ksta, kend
      integer*4, dimension(3) :: cart_dims, mycoords
!
#ifdef _USE_MPI_
      integer*4, private :: iup, idown, jup, jdown, kup, kdown
      integer*4, private :: nprocs
      integer*4, private :: comm_cart
!
      real*8, dimension(:,:,:), allocatable, private ::
     $  p0_sbx0, p0_sbx1, p0_rbx0, p0_rbx1
     $, p0_sby0, p0_sby1, p0_rby0, p0_rby1
     $, p0_sbz0, p0_sbz1, p0_rbz0, p0_rbz1
      integer*4, private ::
     $  p0_sizex10, p0_sizex01
     $, p0_sizey10, p0_sizey01
     $, p0_sizez10, p0_sizez01
!
      real*8, dimension(:,:,:), allocatable, private ::
     $  u_sbx0, u_sbx1, u_rbx0, u_rbx1
     $, u_sby0, u_sby1, u_rby0, u_rby1
     $, u_sbz0, u_sbz1, u_rbz0, u_rbz1
      integer*4, private ::
     $  u_sizex10, u_sizex01
     $, u_sizey10, u_sizey01
     $, u_sizez10, u_sizez01
!
      real*8, dimension(:,:,:), allocatable, private ::
     $  v_sbx0, v_sbx1, v_rbx0, v_rbx1
     $, v_sby0, v_sby1, v_rby0, v_rby1
     $, v_sbz0, v_sbz1, v_rbz0, v_rbz1
      integer*4, private ::
     $  v_sizex10, v_sizex01
     $, v_sizey10, v_sizey01
     $, v_sizez10, v_sizez01
!
      real*8, dimension(:,:,:), allocatable, private ::
     $  w_sbx0, w_sbx1, w_rbx0, w_rbx1
     $, w_sby0, w_sby1, w_rby0, w_rby1
     $, w_sbz0, w_sbz1, w_rbz0, w_rbz1
      integer*4, private ::
     $  w_sizex10, w_sizex01
     $, w_sizey10, w_sizey01
     $, w_sizez10, w_sizez01
!
      real*8, dimension(:,:,:), allocatable, private ::
     $  T_sbx0, T_sbx1, T_rbx0, T_rbx1
     $, T_sby0, T_sby1, T_rby0, T_rby1
     $, T_sbz0, T_sbz1, T_rbz0, T_rbz1
      integer*4, private ::
     $  T_sizex10, T_sizex01
     $, T_sizey10, T_sizey01
     $, T_sizez10, T_sizez01
!
#endif /* _USE_MPI_ */
!
      contains
!
! ----------------------------------------------------------------------
#ifdef _ARGUMENT_
      subroutine MyMPI_init_comm_cart(l, m, n)
#else
      subroutine MyMPI_init_comm_cart
#endif /* _ARGUMENT_ */
! ----------------------------------------------------------------------
      use Array_Index , only : nx, ny, nz
!
      implicit none
!
#ifdef _USE_MPI_
      include 'mpif.h'
#endif
!
#ifdef _ARGUMENT_
      integer*4, intent(in) :: l, m, n
#endif /* _ARGUMENT_ */
!
#ifdef _USE_MPI_
      integer*4, parameter :: ndims = 3
      integer*4 :: ierr
      logical, dimension(3) :: periods
      logical :: reorder
!
#ifdef _ARGUMENT_
      cart_dims(1) = l
      cart_dims(2) = m
      cart_dims(3) = n
#else
      cart_dims(1) = _IDIM_
      cart_dims(2) = _JDIM_
      cart_dims(3) = _KDIM_
#endif /* _ARGUMENT_ */
!
      periods(1) = .false.
      periods(2) = .false.
      periods(3) = .false.
!
      reorder    = .false.
!
      call MPI_INIT(ierr)
      call MPI_COMM_SIZE(MPI_COMM_WORLD, nprocs, ierr)
      call MPI_COMM_RANK(MPI_COMM_WORLD, myrank, ierr)
!
      if( cart_dims(1)*cart_dims(2)*cart_dims(3) .ne. nprocs ) then
         if( myrank .eq. 0 ) then
            write(*,*) 'Invalid number of PE'
            write(*,*) 'Please check partitioning pattern'
            write(*,*) '                 or number of  PE'
         end if
         call MPI_FINALIZE(ierr)
         stop
      end if
!
      call MPI_CART_CREATE(MPI_COMM_WORLD, ndims, cart_dims, periods,
     $                     reorder, comm_cart, ierr)
!
      call MPI_CART_GET(comm_cart, ndims, cart_dims, periods, mycoords,
     $                  ierr)
!
      call MPI_CART_SHIFT(comm_cart, 0, 1, iup, idown, ierr)
      call MPI_CART_SHIFT(comm_cart, 1, 1, jup, jdown, ierr)
      call MPI_CART_SHIFT(comm_cart, 2, 1, kup, kdown, ierr)
!      write(*,'(7(1x,i5))') myrank, iup, idown, jup, jdown, kup, kdown
!
      call PARA_RANGE( 1, nx, cart_dims(1), mycoords(1), ista, iend)
      call PARA_RANGE( 1, ny, cart_dims(2), mycoords(2), jsta, jend)
      call PARA_RANGE( 1, nz, cart_dims(3), mycoords(3), ksta, kend)
!      write(*,'(7(1x,i5))') myrank, ista, iend, jsta, jend, ksta, kend
!
#else
      myrank = 0
      ista = 1 ; iend = nx
      jsta = 1 ; jend = ny
      ksta = 1 ; kend = nz
      cart_dims(1) = 1
      cart_dims(2) = 1
      cart_dims(3) = 1
      mycoords(1) = 0
      mycoords(2) = 0
      mycoords(3) = 0
#endif /* _USE_MPI_ */
!
#ifdef _ARGUMENT_
      if( myrank .eq. 0 ) then
         write(*,*) "<<<<< Size and Division are overriden by arguments"
         write(*,'("NX   = ",i5," : NY   = ",i5," : NZ   = ",i5)')
     $   nx, ny, nz
         write(*,'("IDIM = ",i5," : JDIM = ",i5," : KDIM = ",i5)')
     $   cart_dims(1), cart_dims(2), cart_dims(3)
         write(*,*) ">>>>>"
      endif
#endif /* _ARGUMENT_ */
!
      return
      end subroutine MyMPI_init_comm_cart

#ifdef _USE_MPI_
! ----------------------------------------------------------------------
      subroutine PARA_RANGE(N1, N2, NPROCS, IRANK, ISTA, IEND)
! ----------------------------------------------------------------------
      implicit none
!
      integer*4, intent( in) :: N1, N2, NPROCS, IRANK
      integer*4, intent(out) :: ISTA, IEND
!
      integer*4 :: IWORK1, IWORK2
!
      IWORK1 = (N2-N1+1)/NPROCS
      IWORK2 = MOD(N2-N1+1,NPROCS)
      ISTA = IRANK*IWORK1 + N1 + MIN(IRANK,IWORK2)
      IEND = ISTA + IWORK1 -1
      IF (IWORK2 > IRANK) IEND = IEND +1
      end subroutine PARA_RANGE

! ----------------------------------------------------------------------
      subroutine MyMPI_init_type_vector
     $( isize, jsize, ksize,
     $  iband, jband, kband, vec)
! ----------------------------------------------------------------------
      implicit none
!
      include 'mpif.h'
!
      integer*4, intent( in) :: isize, jsize, ksize
      integer*4, intent( in) :: iband, jband, kband
      integer*4, intent(out) :: vec(3)
!
      integer*4 :: ierr, count, blocklen, stride
!
      count    = jsize * ksize
      blocklen = 1 * iband
      stride   = isize
      call MPI_TYPE_VECTOR(count, blocklen, stride, MPI_REAL8,
     $                     vec(1), ierr)
      call MPI_TYPE_COMMIT(vec(1), ierr)
!
      count    = ksize
      blocklen = isize * jband
      stride   = isize * jsize
      call MPI_TYPE_VECTOR(count, blocklen, stride, MPI_REAL8,
     $                     vec(2), ierr)
      call MPI_TYPE_COMMIT(vec(2), ierr)
!
      count    = 1
      blocklen = isize * jsize * kband
      stride   = isize * jsize * kband
!      count    = jsize * kband
!      blocklen = isize
!      stride   = isize
!      count    = kband
!      blocklen = isize * jsize
!      stride   = isize * jsize
      call MPI_TYPE_VECTOR(count, blocklen, stride, MPI_REAL8,
     $                     vec(3), ierr)
      call MPI_TYPE_COMMIT(vec(3), ierr)
!
      return
      end subroutine MyMPI_init_type_vector

#ifdef _BUFFER_SENDRECEIVE_
! ----------------------------------------------------------------------
      subroutine MyMPI_Allocate_Buffer_Array
! ----------------------------------------------------------------------
      use Array_Index
!
      implicit none
!
      integer*4 :: ib
      integer*4 :: jb
      integer*4 :: kb
!
! +++ P
      ib = -ista1m+(istam-1)
      allocate( p0_sbx1(iendm-ib:iendm, jsta1m:jend1m, ksta1m:kend1m) )
      allocate( p0_rbx0(ista1m:istam-1, jsta1m:jend1m, ksta1m:kend1m) )
      p0_sizex10 = (ib+1)*(jend1m-jsta1m+1)*(kend1m-ksta1m+1)
      ib = -(iendm+1)+iend1m
      allocate( p0_sbx0(istam:istam+ib, jsta1m:jend1m, ksta1m:kend1m) )
      allocate( p0_rbx1(iendm+1:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
      p0_sizex01 = (ib+1)*(jend1m-jsta1m+1)*(kend1m-ksta1m+1)
#ifdef  BUFFER_J 
      jb = -jsta1m+(jstam-1)
      allocate( p0_sby1(ista1m:iend1m, jendm-jb:jendm, ksta1m:kend1m) )
      allocate( p0_rby0(ista1m:iend1m, jsta1m:jstam-1, ksta1m:kend1m) )
      p0_sizey10 = (iend1m-ista1m+1)*(jb+1)*(kend1m-ksta1m+1)
      jb = -(jendm+1)+jend1m
      allocate( p0_sby0(ista1m:iend1m, jstam:jstam+jb, ksta1m:kend1m) )
      allocate( p0_rby1(ista1m:iend1m, jendm+1:jend1m, ksta1m:kend1m) )
      p0_sizey01 = (iend1m-ista1m+1)*(jb+1)*(kend1m-ksta1m+1)
#endif /* BUFFER_J */
#ifdef  BUFFER_K
      kb = -ksta1m+(kstam-1)
      allocate( p0_sbz1(ista1m:iend1m, jsta1m:jend1m, kendm-kb:kendm) )
      allocate( p0_rbz0(ista1m:iend1m, jsta1m:jend1m, ksta1m:kstam-1) )
      p0_sizez10 = (iend1m-ista1m+1)*(jend1m-jsta1m+1)*(kb+1)
      kb = -(kendm+1)+kend1m
      allocate( p0_sbz0(ista1m:iend1m, jsta1m:jend1m, kstam:kstam+kb) )
      allocate( p0_rbz1(ista1m:iend1m, jsta1m:jend1m, kendm+1:kend1m) )
      p0_sizez01 = (iend1m-ista1m+1)*(jend1m-jsta1m+1)*(kb+1)
#endif /* BUFFER_K */
! --- P
! +++ U
      ib = -ista2n+istan
      allocate( u_sbx1(iendn-ib:iendn, jsta2m:jend2m, ksta2m:kend2m) )
      allocate( u_rbx0(ista2n:istan  , jsta2m:jend2m, ksta2m:kend2m) )
      u_sizex10 = (ib+1)*(jend2m-jsta2m+1)*(kend2m-ksta2m+1)
      ib = -(iendn+1)+iend2n
      allocate(u_sbx0(istan+1:istan+1+ib, jsta2m:jend2m, ksta2m:kend2m))
      allocate(u_rbx1(iendn+1:iend2n    , jsta2m:jend2m, ksta2m:kend2m))
      u_sizex01 = (ib+1)*(jend2m-jsta2m+1)*(kend2m-ksta2m+1)
#ifdef  BUFFER_J 
      jb = -jsta2m+(jstam-1)
      allocate( u_sby1(ista2n:iend2n, jendm-jb:jendm, ksta2m:kend2m) )
      allocate( u_rby0(ista2n:iend2n, jsta2m:jstam-1, ksta2m:kend2m) )
      u_sizey10 = (iend2n-ista2n+1)*(jb+1)*(kend2m-ksta2m+1)
      jb = -(jendm+1)+jend2m
      allocate( u_sby0(ista2n:iend2n, jstam:jstam+jb, ksta2m:kend2m) )
      allocate( u_rby1(ista2n:iend2n, jendm+1:jend2m, ksta2m:kend2m) )
      u_sizey01 = (iend2n-ista2n+1)*(jb+1)*(kend2m-ksta2m+1)
#endif /* BUFFER_J */
#ifdef  BUFFER_K
      kb = -ksta2m+(kstam-1)
      allocate( u_sbz1(ista2n:iend2n, jsta2m:jend2m, kendm-kb:kendm) )
      allocate( u_rbz0(ista2n:iend2n, jsta2m:jend2m, ksta2m:kstam-1) )
      u_sizez10 = (iend2n-ista2n+1)*(jend2m-jsta2m+1)*(kb+1)
      kb = -(kendm+1)+kend2m
      allocate( u_sbz0(ista2n:iend2n, jsta2m:jend2m, kstam:kstam+kb) )
      allocate( u_rbz1(ista2n:iend2n, jsta2m:jend2m, kendm+1:kend2m) )
      u_sizez01 = (iend2n-ista2n+1)*(jend2m-jsta2m+1)*(kb+1)
#endif /* BUFFER_K */
! --- U
! +++ V
      ib = -ista2m+(istam-1)
      allocate( v_sbx1(iendm-ib:iendm, jsta2n:jend2n, ksta2m:kend2m) )
      allocate( v_rbx0(ista2m:istam-1, jsta2n:jend2n, ksta2m:kend2m) )
      v_sizex10 = (ib+1)*(jend2n-jsta2n+1)*(kend2m-ksta2m+1)
      ib = -(iendm+1)+iend2m
      allocate( v_sbx0(istam:istam+ib, jsta2n:jend2n, ksta2m:kend2m) )
      allocate( v_rbx1(iendm+1:iend2m, jsta2n:jend2n, ksta2m:kend2m) )
      v_sizex01 = (ib+1)*(jend2n-jsta2n+1)*(kend2m-ksta2m+1)
#ifdef  BUFFER_J 
      jb = -jsta2n+jstan
      allocate( v_sby1(ista2m:iend2m, jendn-jb:jendn, ksta2m:kend2m) )
      allocate( v_rby0(ista2m:iend2m, jsta2n:jstan  , ksta2m:kend2m) )
      v_sizey10 = (iend2m-ista2m+1)*(jb+1)*(kend2m-ksta2m+1)
      jb = -(jendn+1)+jend2n
      allocate(v_sby0(ista2m:iend2m, jstan+1:jstan+1+jb, ksta2m:kend2m))
      allocate(v_rby1(ista2m:iend2m, jendn+1:jend2n    , ksta2m:kend2m))
      v_sizey01 = (iend2m-ista2m+1)*(jb+1)*(kend2m-ksta2m+1)
#endif /* BUFFER_J */
#ifdef  BUFFER_K
      kb = -ksta2m+(kstam-1)
      allocate( v_sbz1(ista2m:iend2m, jsta2n:jend2n, kendm-kb:kendm) )
      allocate( v_rbz0(ista2m:iend2m, jsta2n:jend2n, ksta2m:kstam-1) )
      v_sizez10 = (iend2m-ista2m+1)*(jend2n-jsta2n+1)*(kb+1)
      kb = -(kendm+1)+kend2m
      allocate( v_sbz0(ista2m:iend2m, jsta2n:jend2n, kstam:kstam+kb) )
      allocate( v_rbz1(ista2m:iend2m, jsta2n:jend2n, kendm+1:kend2m) )
      v_sizez01 = (iend2m-ista2m+1)*(jend2n-jsta2n+1)*(kb+1)
#endif /* BUFFER_K */
! --- V
! +++ W
      ib = -ista2m+(istam-1)
      allocate( w_sbx1(iendm-ib:iendm, jsta2m:jend2m, ksta2n:kend2n) )
      allocate( w_rbx0(ista2m:istam-1, jsta2m:jend2m, ksta2n:kend2n) )
      w_sizex10 = (ib+1)*(jend2m-jsta2m+1)*(kend2n-ksta2n+1)
      ib = -(iendm+1)+iend2m
      allocate( w_sbx0(istam:istam+ib, jsta2m:jend2m, ksta2n:kend2n) )
      allocate( w_rbx1(iendm+1:iend2m, jsta2m:jend2m, ksta2n:kend2n) )
      w_sizex01 = (ib+1)*(jend2m-jsta2m+1)*(kend2n-ksta2n+1)
#ifdef  BUFFER_J
      jb = -jsta2m+(jstam-1)
      allocate( w_sby1(ista2m:iend2m, jendm-jb:jendm, ksta2n:kend2n) )
      allocate( w_rby0(ista2m:iend2m, jsta2m:jstam-1, ksta2n:kend2n) )
      w_sizey10 = (iend2m-ista2m+1)*(jb+1)*(kend2n-ksta2n+1)
      jb = -(jendm+1)+jend2m
      allocate( w_sby0(ista2m:iend2m, jstam:jstam+jb, ksta2n:kend2n) )
      allocate( w_rby1(ista2m:iend2m, jendm+1:jend2m, ksta2n:kend2n) )
      w_sizey01 = (iend2m-ista2m+1)*(jb+1)*(kend2n-ksta2n+1)
#endif /* BUFFER_J */
#ifdef  BUFFER_K 
      kb = -ksta2n+kstan
      allocate( w_sbz1(ista2m:iend2m, jsta2m:jend2m, kendn-kb:kendn) )
      allocate( w_rbz0(ista2m:iend2m, jsta2m:jend2m, ksta2n:kstan  ) )
      w_sizez10 = (iend2m-ista2m+1)*(jend2m-jsta2m+1)*(kb+1)
      kb = -(kendn+1)+kend2n
      allocate(w_sbz0(ista2m:iend2m, jsta2m:jend2m, kstan+1:kstan+1+kb))
      allocate(w_rbz1(ista2m:iend2m, jsta2m:jend2m, kendn+1:kend2n    ))
      w_sizez01 = (iend2m-ista2m+1)*(jend2m-jsta2m+1)*(kb+1)
#endif /* BUFFER_K */
! --- W
! +++ T
      ib = -ista2m+(istam-1)
      allocate( T_sbx1(iendm-ib:iendm, jsta2m:jend2m, ksta2m:kend2m) )
      allocate( T_rbx0(ista2m:istam-1, jsta2m:jend2m, ksta2m:kend2m) )
      T_sizex10 = (ib+1)*(jend2m-jsta2m+1)*(kend2m-ksta2m+1)
      ib = -(iendm+1)+iend2m
      allocate( T_sbx0(istam:istam+ib, jsta2m:jend2m, ksta2m:kend2m) )
      allocate( T_rbx1(iendm+1:iend2m, jsta2m:jend2m, ksta2m:kend2m) )
      T_sizex01 = (ib+1)*(jend2m-jsta2m+1)*(kend2m-ksta2m+1)
#ifdef  BUFFER_J
      jb = -jsta2m+(jstam-1)
      allocate( T_sby1(ista2m:iend2m, jendm-jb:jendm, ksta2m:kend2m) )
      allocate( T_rby0(ista2m:iend2m, jsta2m:jstam-1, ksta2m:kend2m) )
      T_sizey10 = (iend2m-ista2m+1)*(jb+1)*(kend2m-ksta2m+1)
      jb = -(jendm+1)+jend2m
      allocate( T_sby0(ista2m:iend2m, jstam:jstam+jb, ksta2m:kend2m) )
      allocate( T_rby1(ista2m:iend2m, jendm+1:jend2m, ksta2m:kend2m) )
      T_sizey01 = (iend2m-ista2m+1)*(jb+1)*(kend2m-ksta2m+1)
#endif /* BUFFER_J */
#ifdef  BUFFER_K
      kb = -ksta2m+(kstam-1)
      allocate( T_sbz1(ista2m:iend2m, jsta2m:jend2m, kendm-kb:kendm) )
      allocate( T_rbz0(ista2m:iend2m, jsta2m:jend2m, ksta2m:kstam-1) )
      T_sizez10 = (iend2m-ista2m+1)*(jend2m-jsta2m+1)*(kb+1)
      kb = -(kendm+1)+kend2m
      allocate( T_sbz0(ista2m:iend2m, jsta2m:jend2m, kstam:kstam+kb) )
      allocate( T_rbz1(ista2m:iend2m, jsta2m:jend2m, kendm+1:kend2m) )
      T_sizez01 = (iend2m-ista2m+1)*(jend2m-jsta2m+1)*(kb+1)
#endif /* BUFFER_K */
! --- T
!
      return
      end subroutine MyMPI_Allocate_Buffer_Array

! ----------------------------------------------------------------------
      subroutine MyMPI_SendReceive_p0(iband, jband, kband, p0, vec)
! ----------------------------------------------------------------------
      use Array_Index
!
      implicit none
!
      include 'mpif.h'
!
      integer*4, intent(in) :: iband, jband, kband
      real*8, intent(inout) ::
     $ p0( ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m )
!
      integer*4, intent(in) :: vec(3)
      integer*4 :: ireq(12)
!
      integer*4 :: i, j, k
!
      integer*4 :: ierr
      integer, dimension(MPI_STATUS_SIZE, 12) :: istatus
!
! +++ I-direction
      if ( iup >= 0 ) then
c      print *, "sbx0", myrank, iup, istam, istam-(iendm+1)+iend1m
         do k = ksta1m, kend1m
         do j = jsta1m, jend1m
         do i = istam , istam-(iendm+1)+iend1m
            p0_sbx0(i,j,k) = p0(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( idown >= 0 ) then
c      print *, "sbx1", myrank, idown, iendm+ista1m-(istam-1), iendm
         do k = ksta1m, kend1m
         do j = jsta1m, jend1m
         do i = iendm+ista1m-(istam-1), iendm
            p0_sbx1(i,j,k) = p0(i,j,k)
         enddo
         enddo
         enddo
      endif
!
      call MPI_IRECV( p0_rbx0,p0_sizex10,MPI_REAL8,
     $                iup  , 1, comm_cart, ireq( 1), ierr)
!
      call MPI_ISEND( p0_sbx1,p0_sizex10,MPI_REAL8,
     $                idown, 1, comm_cart, ireq( 2), ierr)
!
      call MPI_IRECV( p0_rbx1,p0_sizex01,MPI_REAL8,
     $                idown, 2, comm_cart, ireq( 3), ierr)
!
      call MPI_ISEND( p0_sbx0,p0_sizex01,MPI_REAL8,
     $                iup  , 2, comm_cart, ireq( 4), ierr)
      call MPI_WAITALL(4, ireq(1), istatus, ierr)
!
      if ( iup >= 0 ) then
c      print *, "rbx0", myrank, iup, ista1m, istam-1
         do k = ksta1m, kend1m
         do j = jsta1m, jend1m
         do i = ista1m, istam-1
            p0(i,j,k) = p0_rbx0(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( idown >= 0 ) then
c      print *, "rbx1", myrank, idown, iendm+1, iend1m
         do k = ksta1m, kend1m
         do j = jsta1m, jend1m
         do i = iendm+1, iend1m
            p0(i,j,k) = p0_rbx1(i,j,k)
         enddo
         enddo
         enddo
      endif
! --- I-direction
!
#ifdef  BUFFER_J
! +++ J-direction
      if ( jup >= 0 ) then
         do k = ksta1m, kend1m
         do j = jstam , jstam-(jendm+1)+jend1m
         do i = ista1m, iend1m
            p0_sby0(i,j,k) = p0(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( jdown >= 0 ) then
         do k = ksta1m, kend1m
         do j = jendm+jsta1m-(jstam-1), jendm
         do i = ista1m, iend1m
            p0_sby1(i,j,k) = p0(i,j,k)
         enddo
         enddo
         enddo
      endif
!
      call MPI_IRECV( p0_rby0, p0_sizey10, MPI_REAL8,
     $                jup  , 3, comm_cart, ireq( 5), ierr)
!
      call MPI_ISEND( p0_sby1, p0_sizey10, MPI_REAL8,
     $                jdown, 3, comm_cart, ireq( 6), ierr)
!
      call MPI_IRECV( p0_rby1, p0_sizey01, MPI_REAL8,
     $                jdown, 4, comm_cart, ireq( 7), ierr)
!
      call MPI_ISEND( p0_sby0, p0_sizey01, MPI_REAL8,
     $                jup  , 4, comm_cart, ireq( 8), ierr)
      call MPI_WAITALL(4, ireq(5), istatus, ierr)
!
      if ( jup >= 0 ) then
         do k = ksta1m, kend1m
         do j = jsta1m, jstam-1
         do i = ista1m, iend1m
            p0(i,j,k) = p0_rby0(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( jdown >= 0 ) then
         do k = ksta1m, kend1m
         do j = jendm+1, jend1m
         do i = ista1m , iend1m
            p0(i,j,k) = p0_rby1(i,j,k)
         enddo
         enddo
         enddo
      endif
!
! --- J-direction
#else  /* BUFFER_J */
! +++ J-direction
      call MPI_IRECV( p0(ista-iband, jsta-jband  , ksta-kband),1,vec(2),
     $                jup  , 3, comm_cart, ireq( 5), ierr)
!
      call MPI_ISEND( p0(ista-iband, jend-jband+1, ksta-kband),1,vec(2),
     $                jdown, 3, comm_cart, ireq( 6), ierr)
!
      call MPI_IRECV( p0(ista-iband, jend+1      , ksta-kband),1,vec(2),
     $                jdown, 4, comm_cart, ireq( 7), ierr)
!
      call MPI_ISEND( p0(ista-iband, jsta        , ksta-kband),1,vec(2),
     $                jup  , 4, comm_cart, ireq( 8), ierr)
      call MPI_WAITALL(4, ireq(5), istatus, ierr)
! --- J-direction
#endif /* BUFFER_J */
!
#ifdef BUFFER_K
! +++ K-direction
      if ( kup >= 0 ) then
c      print *, "sbz0", myrank, kup, kstam , kstam-(kendm+1)+kend1m
         do k = kstam , kstam-(kendm+1)+kend1m
         do j = jsta1m, jend1m
         do i = ista1m, iend1m
            p0_sbz0(i,j,k) = p0(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( kdown >= 0 ) then
c      print *, "sbz1", myrank, kdown, kendm+ksta1m-(kstam-1), kendm
         do k = kendm+ksta1m-(kstam-1), kendm
         do j = jsta1m, jend1m
         do i = ista1m, iend1m
            p0_sbz1(i,j,k) = p0(i,j,k)
         enddo
         enddo
         enddo
      endif
!
      call MPI_IRECV( p0_rbz0, p0_sizez10, MPI_REAL8,
     $                kup  , 5, comm_cart, ireq( 9), ierr)
!
      call MPI_ISEND( p0_sbz1, p0_sizez10, MPI_REAL8,
     $                kdown, 5, comm_cart, ireq(10), ierr)
!
      call MPI_IRECV( p0_rbz1, p0_sizez01, MPI_REAL8,
     $                kdown, 6, comm_cart, ireq(11), ierr)
!
      call MPI_ISEND( p0_sbz0, p0_sizez01, MPI_REAL8,
     $                kup  , 6, comm_cart, ireq(12), ierr)
      call MPI_WAITALL(4, ireq(9), istatus, ierr)
!
      if ( kup >= 0 ) then
c      print *, "rbz0", myrank, kup, ksta1m, kstam-1
         do k = ksta1m, kstam-1
         do j = jsta1m, jend1m
         do i = ista1m, iend1m
            p0(i,j,k) = p0_rbz0(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( kdown >= 0 ) then
c      print *, "rbz1", myrank, kdown, kendm+1, kend1m
         do k = kendm+1, kend1m
         do j = jsta1m , jend1m
         do i = ista1m , iend1m
            p0(i,j,k) = p0_rbz1(i,j,k)
         enddo
         enddo
         enddo
      endif
! --- K-direction
#else  /* BUFFER_K */
! +++ K-direction
      call MPI_IRECV( p0(ista-iband, jsta-jband, ksta-kband  ),1,vec(3),
     $                kup  , 5, comm_cart, ireq( 9), ierr)
!
      call MPI_ISEND( p0(ista-iband, jsta-jband, kend-kband+1),1,vec(3),
     $                kdown, 5, comm_cart, ireq(10), ierr)
!
      call MPI_IRECV( p0(ista-iband, jsta-jband, kend+1      ),1,vec(3),
     $                kdown, 6, comm_cart, ireq(11), ierr)
!
      call MPI_ISEND( p0(ista-iband, jsta-jband, ksta        ),1,vec(3),
     $                kup  , 6, comm_cart, ireq(12), ierr)
      call MPI_WAITALL(4, ireq(9), istatus, ierr)
! --- K-direction
#endif /* BUFFER_K */
!
      return
      end subroutine MyMPI_SendReceive_p0

! ----------------------------------------------------------------------
      subroutine MyMPI_SendReceive_u (iband, jband, kband, u , vec)
! ----------------------------------------------------------------------
      use Array_Index
!
      implicit none
!
      include 'mpif.h'
!
      integer*4, intent(in) :: iband, jband, kband
      real*8, intent(inout) ::
     $ u( ista2n:iend2n, jsta2m:jend2m, ksta2m:kend2m )
!
      integer*4, intent(in) :: vec(3)
      integer*4 :: ireq(12)
!
      integer*4 :: i, j, k, ib, jb, kb
!
      integer*4 :: ierr
      integer, dimension(MPI_STATUS_SIZE, 12) :: istatus
!
! +++ I-direction
      if ( iup >= 0 ) then
         ib = -(iendn+1)+iend2n
c      print *, "sbx0", myrank, iup, istan+1, istan+1+ib
         do k = ksta2m , kend2m
         do j = jsta2m , jend2m
         do i = istan+1, istan+1+ib
            u_sbx0(i,j,k) = u(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( idown >= 0 ) then
         ib = -ista2n+istan
c      print *, "sbx1", myrank, idown, iendn-ib, iendn
         do k = ksta2m  , kend2m
         do j = jsta2m  , jend2m
         do i = iendn-ib, iendn
            u_sbx1(i,j,k) = u(i,j,k)
         enddo
         enddo
         enddo
      endif
!
      call MPI_IRECV( u_rbx0,u_sizex10,MPI_REAL8,
     $                iup  , 1, comm_cart, ireq( 1), ierr)
!
      call MPI_ISEND( u_sbx1,u_sizex10,MPI_REAL8,
     $                idown, 1, comm_cart, ireq( 2), ierr)
!
      call MPI_IRECV( u_rbx1,u_sizex01,MPI_REAL8,
     $                idown, 2, comm_cart, ireq( 3), ierr)
!
      call MPI_ISEND( u_sbx0,u_sizex01,MPI_REAL8,
     $                iup  , 2, comm_cart, ireq( 4), ierr)
      call MPI_WAITALL(4, ireq(1), istatus, ierr)
!
      if ( iup >= 0 ) then
c      print *, "rbx0", myrank, iup, ista2n, istan
         do k = ksta2m, kend2m
         do j = jsta2m, jend2m
         do i = ista2n, istan
            u(i,j,k) = u_rbx0(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( idown >= 0 ) then
c      print *, "rbx1", myrank, idown, iendn+1, iend2n
         do k = ksta2m , kend2m
         do j = jsta2m , jend2m
         do i = iendn+1, iend2n
            u(i,j,k) = u_rbx1(i,j,k)
         enddo
         enddo
         enddo
      endif
! --- I-direction
!
#ifdef  BUFFER_J
! +++ J-direction
      if ( jup >= 0 ) then
         jb = -(jendm+1)+jend2m
         do k = ksta2m, kend2m
         do j = jstam , jstam+jb
         do i = ista2n, iend2n
            u_sby0(i,j,k) = u(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( jdown >= 0 ) then
         jb = -jsta2m+(jstam-1)
         do k = ksta2m  , kend2m
         do j = jendm-jb, jendm
         do i = ista2n  , iend2n
            u_sby1(i,j,k) = u(i,j,k)
         enddo
         enddo
         enddo
      endif
!
      call MPI_IRECV( u_rby0, u_sizey10, MPI_REAL8,
     $                jup  , 3, comm_cart, ireq( 5), ierr)
!
      call MPI_ISEND( u_sby1, u_sizey10, MPI_REAL8,
     $                jdown, 3, comm_cart, ireq( 6), ierr)
!
      call MPI_IRECV( u_rby1, u_sizey01, MPI_REAL8,
     $                jdown, 4, comm_cart, ireq( 7), ierr)
!
      call MPI_ISEND( u_sby0, u_sizey01, MPI_REAL8,
     $                jup  , 4, comm_cart, ireq( 8), ierr)
      call MPI_WAITALL(4, ireq(5), istatus, ierr)
!
      if ( jup >= 0 ) then
         do k = ksta2m, kend2m
         do j = jsta2m, jstam-1
         do i = ista2n, iend2n
            u(i,j,k) = u_rby0(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( jdown >= 0 ) then
         do k = ksta2m, kend2m
         do j = jendm+1, jend2m
         do i = ista2n , iend2n
            u(i,j,k) = u_rby1(i,j,k)
         enddo
         enddo
         enddo
      endif
!
! --- J-direction
#else  /* BUFFER_J */
! +++ J-direction
      call MPI_IRECV( u(ista-iband, jsta-jband  , ksta-kband),1,vec(2),
     $                jup  , 3, comm_cart, ireq( 5), ierr)
!
      call MPI_ISEND( u(ista-iband, jend-jband+1, ksta-kband),1,vec(2),
     $                jdown, 3, comm_cart, ireq( 6), ierr)
!
      call MPI_IRECV( u(ista-iband, jend+1      , ksta-kband),1,vec(2),
     $                jdown, 4, comm_cart, ireq( 7), ierr)
!
      call MPI_ISEND( u(ista-iband, jsta        , ksta-kband),1,vec(2),
     $                jup  , 4, comm_cart, ireq( 8), ierr)
      call MPI_WAITALL(4, ireq(5), istatus, ierr)
! --- J-direction
#endif /* BUFFER_J */
!
#ifdef BUFFER_K
! +++ K-direction
      if ( kup >= 0 ) then
c      print *, "sbz0", myrank, kup, kstam , kstam-(kendm+1)+kend1m
         kb = -(kendm+1)+kend2m
         do k = kstam , kstam+kb
         do j = jsta2m, jend2m
         do i = ista2n, iend2n
            u_sbz0(i,j,k) = u(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( kdown >= 0 ) then
c      print *, "sbz1", myrank, kdown, kendm+ksta1m-(kstam-1), kendm
         kb = -ksta2m+(kstam-1)
         do k = kendm-kb, kendm
         do j = jsta2m, jend2m
         do i = ista2n, iend2n
            u_sbz1(i,j,k) = u(i,j,k)
         enddo
         enddo
         enddo
      endif
!
      call MPI_IRECV( u_rbz0, u_sizez10, MPI_REAL8,
     $                kup  , 5, comm_cart, ireq( 9), ierr)
!
      call MPI_ISEND( u_sbz1, u_sizez10, MPI_REAL8,
     $                kdown, 5, comm_cart, ireq(10), ierr)
!
      call MPI_IRECV( u_rbz1, u_sizez01, MPI_REAL8,
     $                kdown, 6, comm_cart, ireq(11), ierr)
!
      call MPI_ISEND( u_sbz0, u_sizez01, MPI_REAL8,
     $                kup  , 6, comm_cart, ireq(12), ierr)
      call MPI_WAITALL(4, ireq(9), istatus, ierr)
!
      if ( kup >= 0 ) then
c      print *, "rbz0", myrank, kup, ksta1m, kstam-1
         do k = ksta2m, kstam-1
         do j = jsta2m, jend2m
         do i = ista2n, iend2n
            u(i,j,k) = u_rbz0(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( kdown >= 0 ) then
c      print *, "rbz1", myrank, kdown, kendm+1, kend1m
         do k = kendm+1, kend2m
         do j = jsta2m , jend2m
         do i = ista2n , iend2n
            u(i,j,k) = u_rbz1(i,j,k)
         enddo
         enddo
         enddo
      endif
! --- K-direction
#else  /* BUFFER_K */
! +++ K-direction
      call MPI_IRECV( u(ista-iband, jsta-jband, ksta-kband  ),1,vec(3),
     $                kup  , 5, comm_cart, ireq( 9), ierr)
!
      call MPI_ISEND( u(ista-iband, jsta-jband, kend-kband+1),1,vec(3),
     $                kdown, 5, comm_cart, ireq(10), ierr)
!
      call MPI_IRECV( u(ista-iband, jsta-jband, kend+1      ),1,vec(3),
     $                kdown, 6, comm_cart, ireq(11), ierr)
!
      call MPI_ISEND( u(ista-iband, jsta-jband, ksta        ),1,vec(3),
     $                kup  , 6, comm_cart, ireq(12), ierr)
      call MPI_WAITALL(4, ireq(9), istatus, ierr)
! --- K-direction
#endif /* BUFFER_K */
!
      return
      end subroutine MyMPI_SendReceive_u

! ----------------------------------------------------------------------
      subroutine MyMPI_SendReceive_v (iband, jband, kband, v , vec)
! ----------------------------------------------------------------------
      use Array_Index
!
      implicit none
!
      include 'mpif.h'
!
      integer*4, intent(in) :: iband, jband, kband
      real*8, intent(inout) ::
     $ v( ista2m:iend2m, jsta2n:jend2n, ksta2m:kend2m )
!
      integer*4, intent(in) :: vec(3)
      integer*4 :: ireq(12)
!
      integer*4 :: i, j, k, ib, jb, kb
!
      integer*4 :: ierr
      integer, dimension(MPI_STATUS_SIZE, 12) :: istatus
!
! +++ I-direction
      if ( iup >= 0 ) then
         ib = -(iendm+1)+iend2m
c      print *, "sbx0", myrank, iup, istan+1, istan+1+ib
         do k = ksta2m, kend2m
         do j = jsta2n, jend2n
         do i = istam , istam+ib
            v_sbx0(i,j,k) = v(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( idown >= 0 ) then
         ib = -ista2m+(istam-1)
c      print *, "sbx1", myrank, idown, iendn-ib, iendn
         do k = ksta2m  , kend2m
         do j = jsta2n  , jend2n
         do i = iendm-ib, iendm
            v_sbx1(i,j,k) = v(i,j,k)
         enddo
         enddo
         enddo
      endif
!
      call MPI_IRECV( v_rbx0,v_sizex10,MPI_REAL8,
     $                iup  , 1, comm_cart, ireq( 1), ierr)
!
      call MPI_ISEND( v_sbx1,v_sizex10,MPI_REAL8,
     $                idown, 1, comm_cart, ireq( 2), ierr)
!
      call MPI_IRECV( v_rbx1,v_sizex01,MPI_REAL8,
     $                idown, 2, comm_cart, ireq( 3), ierr)
!
      call MPI_ISEND( v_sbx0,v_sizex01,MPI_REAL8,
     $                iup  , 2, comm_cart, ireq( 4), ierr)
      call MPI_WAITALL(4, ireq(1), istatus, ierr)
!
      if ( iup >= 0 ) then
c      print *, "rbx0", myrank, iup, ista2n, istan
         do k = ksta2m, kend2m
         do j = jsta2n, jend2n
         do i = ista2m, istam-1
            v(i,j,k) = v_rbx0(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( idown >= 0 ) then
c      print *, "rbx1", myrank, idown, iendn+1, iend2n
         do k = ksta2m , kend2m
         do j = jsta2n , jend2n
         do i = iendm+1, iend2m
            v(i,j,k) = v_rbx1(i,j,k)
         enddo
         enddo
         enddo
      endif
! --- I-direction
!
#ifdef  BUFFER_J
! +++ J-direction
      if ( jup >= 0 ) then
         jb = -(jendn+1)+jend2n
         do k = ksta2m , kend2m
         do j = jstan+1, jsta+1+jb
         do i = ista2m , iend2m
            v_sby0(i,j,k) = v(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( jdown >= 0 ) then
         jb = -jsta2n+jstan
         do k = ksta2m  , kend2m
         do j = jendn-jb, jendn
         do i = ista2m  , iend2m
            v_sby1(i,j,k) = v(i,j,k)
         enddo
         enddo
         enddo
      endif
!
      call MPI_IRECV( v_rby0, v_sizey10, MPI_REAL8,
     $                jup  , 3, comm_cart, ireq( 5), ierr)
!
      call MPI_ISEND( v_sby1, v_sizey10, MPI_REAL8,
     $                jdown, 3, comm_cart, ireq( 6), ierr)
!
      call MPI_IRECV( v_rby1, v_sizey01, MPI_REAL8,
     $                jdown, 4, comm_cart, ireq( 7), ierr)
!
      call MPI_ISEND( v_sby0, v_sizey01, MPI_REAL8,
     $                jup  , 4, comm_cart, ireq( 8), ierr)
      call MPI_WAITALL(4, ireq(5), istatus, ierr)
!
      if ( jup >= 0 ) then
         do k = ksta2m, kend2m
         do j = jsta2n, jstan
         do i = ista2m, iend2m
            v(i,j,k) = v_rby0(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( jdown >= 0 ) then
         do k = ksta2m, kend2m
         do j = jendn+1, jend2n
         do i = ista2m , iend2m
            v(i,j,k) = v_rby1(i,j,k)
         enddo
         enddo
         enddo
      endif
!
! --- J-direction
#else  /* BUFFER_J */
! +++ J-direction
      call MPI_IRECV( v(ista-iband, jsta-jband  , ksta-kband),1,vec(2),
     $                jup  , 3, comm_cart, ireq( 5), ierr)
!
      call MPI_ISEND( v(ista-iband, jend-jband+1, ksta-kband),1,vec(2),
     $                jdown, 3, comm_cart, ireq( 6), ierr)
!
      call MPI_IRECV( v(ista-iband, jend+1      , ksta-kband),1,vec(2),
     $                jdown, 4, comm_cart, ireq( 7), ierr)
!
      call MPI_ISEND( v(ista-iband, jsta        , ksta-kband),1,vec(2),
     $                jup  , 4, comm_cart, ireq( 8), ierr)
      call MPI_WAITALL(4, ireq(5), istatus, ierr)
! --- J-direction
#endif /* BUFFER_J */
!
#ifdef BUFFER_K
! +++ K-direction
      if ( kup >= 0 ) then
c      print *, "sbz0", myrank, kup, kstam , kstam-(kendm+1)+kend1m
         kb = -(kendm+1)+kend2m
         do k = kstam , kstam+kb
         do j = jsta2n, jend2n
         do i = ista2m, iend2m
            v_sbz0(i,j,k) = v(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( kdown >= 0 ) then
c      print *, "sbz1", myrank, kdown, kendm+ksta1m-(kstam-1), kendm
         kb = -ksta2m+(kstam-1)
         do k = kendm-kb, kendm
         do j = jsta2n, jend2n
         do i = ista2m, iend2m
            v_sbz1(i,j,k) = v(i,j,k)
         enddo
         enddo
         enddo
      endif
!
      call MPI_IRECV( v_rbz0, v_sizez10, MPI_REAL8,
     $                kup  , 5, comm_cart, ireq( 9), ierr)
!
      call MPI_ISEND( v_sbz1, v_sizez10, MPI_REAL8,
     $                kdown, 5, comm_cart, ireq(10), ierr)
!
      call MPI_IRECV( v_rbz1, v_sizez01, MPI_REAL8,
     $                kdown, 6, comm_cart, ireq(11), ierr)
!
      call MPI_ISEND( v_sbz0, v_sizez01, MPI_REAL8,
     $                kup  , 6, comm_cart, ireq(12), ierr)
      call MPI_WAITALL(4, ireq(9), istatus, ierr)
!
      if ( kup >= 0 ) then
c      print *, "rbz0", myrank, kup, ksta1m, kstam-1
         do k = ksta2m, kstam-1
         do j = jsta2n, jend2n
         do i = ista2m, iend2m
            v(i,j,k) = v_rbz0(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( kdown >= 0 ) then
c      print *, "rbz1", myrank, kdown, kendm+1, kend1m
         do k = kendm+1, kend2m
         do j = jsta2n , jend2n
         do i = ista2m , iend2m
            v(i,j,k) = v_rbz1(i,j,k)
         enddo
         enddo
         enddo
      endif
! --- K-direction
#else  /* BUFFER_K */
! +++ K-direction
      call MPI_IRECV( v(ista-iband, jsta-jband, ksta-kband  ),1,vec(3),
     $                kup  , 5, comm_cart, ireq( 9), ierr)
!
      call MPI_ISEND( v(ista-iband, jsta-jband, kend-kband+1),1,vec(3),
     $                kdown, 5, comm_cart, ireq(10), ierr)
!
      call MPI_IRECV( v(ista-iband, jsta-jband, kend+1      ),1,vec(3),
     $                kdown, 6, comm_cart, ireq(11), ierr)
!
      call MPI_ISEND( v(ista-iband, jsta-jband, ksta        ),1,vec(3),
     $                kup  , 6, comm_cart, ireq(12), ierr)
      call MPI_WAITALL(4, ireq(9), istatus, ierr)
! --- K-direction
#endif /* BUFFER_K */
!
      return
      end subroutine MyMPI_SendReceive_v

! ----------------------------------------------------------------------
      subroutine MyMPI_SendReceive_w (iband, jband, kband, w , vec)
! ----------------------------------------------------------------------
      use Array_Index
!
      implicit none
!
      include 'mpif.h'
!
      integer*4, intent(in) :: iband, jband, kband
      real*8, intent(inout) ::
     $ w( ista2m:iend2m, jsta2m:jend2m, ksta2n:kend2n )
!
      integer*4, intent(in) :: vec(3)
      integer*4 :: ireq(12)
!
      integer*4 :: i, j, k, ib, jb, kb
!
      integer*4 :: ierr
      integer, dimension(MPI_STATUS_SIZE, 12) :: istatus
!
! +++ I-direction
      if ( iup >= 0 ) then
         ib = -(iendm+1)+iend2m
c      print *, "sbx0", myrank, iup, istan+1, istan+1+ib
         do k = ksta2n, kend2n
         do j = jsta2m, jend2m
         do i = istam , istam+ib
            w_sbx0(i,j,k) = w(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( idown >= 0 ) then
         ib = -ista2m+(istam-1)
c      print *, "sbx1", myrank, idown, iendn-ib, iendn
         do k = ksta2n  , kend2n
         do j = jsta2m  , jend2m
         do i = iendm-ib, iendm
            w_sbx1(i,j,k) = w(i,j,k)
         enddo
         enddo
         enddo
      endif
!
      call MPI_IRECV( w_rbx0,w_sizex10,MPI_REAL8,
     $                iup  , 1, comm_cart, ireq( 1), ierr)
!
      call MPI_ISEND( w_sbx1,w_sizex10,MPI_REAL8,
     $                idown, 1, comm_cart, ireq( 2), ierr)
!
      call MPI_IRECV( w_rbx1,w_sizex01,MPI_REAL8,
     $                idown, 2, comm_cart, ireq( 3), ierr)
!
      call MPI_ISEND( w_sbx0,w_sizex01,MPI_REAL8,
     $                iup  , 2, comm_cart, ireq( 4), ierr)
      call MPI_WAITALL(4, ireq(1), istatus, ierr)
!
      if ( iup >= 0 ) then
c      print *, "rbx0", myrank, iup, ista2n, istan
         do k = ksta2n, kend2n
         do j = jsta2m, jend2m
         do i = ista2m, istam-1
            w(i,j,k) = w_rbx0(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( idown >= 0 ) then
c      print *, "rbx1", myrank, idown, iendn+1, iend2n
         do k = ksta2n , kend2n
         do j = jsta2m , jend2m
         do i = iendm+1, iend2m
            w(i,j,k) = w_rbx1(i,j,k)
         enddo
         enddo
         enddo
      endif
! --- I-direction
!
#ifdef  BUFFER_J
! +++ J-direction
      if ( jup >= 0 ) then
         jb = -(jendm+1)+jend2m
         do k = ksta2n, kend2n
         do j = jstam , jstam+jb
         do i = ista2m, iend2m
            w_sby0(i,j,k) = w(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( jdown >= 0 ) then
         jb = -jsta2m+(jstam-1)
         do k = ksta2n  , kend2n
         do j = jendm-jb, jendm
         do i = ista2m  , iend2m
            w_sby1(i,j,k) = w(i,j,k)
         enddo
         enddo
         enddo
      endif
!
      call MPI_IRECV( w_rby0, w_sizey10, MPI_REAL8,
     $                jup  , 3, comm_cart, ireq( 5), ierr)
!
      call MPI_ISEND( w_sby1, w_sizey10, MPI_REAL8,
     $                jdown, 3, comm_cart, ireq( 6), ierr)
!
      call MPI_IRECV( w_rby1, w_sizey01, MPI_REAL8,
     $                jdown, 4, comm_cart, ireq( 7), ierr)
!
      call MPI_ISEND( w_sby0, w_sizey01, MPI_REAL8,
     $                jup  , 4, comm_cart, ireq( 8), ierr)
      call MPI_WAITALL(4, ireq(5), istatus, ierr)
!
      if ( jup >= 0 ) then
         do k = ksta2n, kend2n
         do j = jsta2m, jstam-1
         do i = ista2m, iend2m
            w(i,j,k) = w_rby0(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( jdown >= 0 ) then
         do k = ksta2n , kend2n
         do j = jendm+1, jend2m
         do i = ista2m , iend2m
            w(i,j,k) = w_rby1(i,j,k)
         enddo
         enddo
         enddo
      endif
!
! --- J-direction
#else  /* BUFFER_J */
! +++ J-direction
      call MPI_IRECV( w(ista-iband, jsta-jband  , ksta-kband),1,vec(2),
     $                jup  , 3, comm_cart, ireq( 5), ierr)
!
      call MPI_ISEND( w(ista-iband, jend-jband+1, ksta-kband),1,vec(2),
     $                jdown, 3, comm_cart, ireq( 6), ierr)
!
      call MPI_IRECV( w(ista-iband, jend+1      , ksta-kband),1,vec(2),
     $                jdown, 4, comm_cart, ireq( 7), ierr)
!
      call MPI_ISEND( w(ista-iband, jsta        , ksta-kband),1,vec(2),
     $                jup  , 4, comm_cart, ireq( 8), ierr)
      call MPI_WAITALL(4, ireq(5), istatus, ierr)
! --- J-direction
#endif /* BUFFER_J */
!
#ifdef BUFFER_K
! +++ K-direction
      if ( kup >= 0 ) then
c      print *, "sbz0", myrank, kup, kstam , kstam-(kendm+1)+kend1m
         kb = -(kendn+1)+kend2n
         do k = kstan+1, kstam+1+kb
         do j = jsta2m , jend2m
         do i = ista2m , iend2m
            w_sbz0(i,j,k) = w(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( kdown >= 0 ) then
c      print *, "sbz1", myrank, kdown, kendm+ksta1m-(kstam-1), kendm
         kb = -ksta2n+kstan
         do k = kendn-kb, kendn
         do j = jsta2m  , jend2m
         do i = ista2m  , iend2m
            w_sbz1(i,j,k) = w(i,j,k)
         enddo
         enddo
         enddo
      endif
!
      call MPI_IRECV( w_rbz0, w_sizez10, MPI_REAL8,
     $                kup  , 5, comm_cart, ireq( 9), ierr)
!
      call MPI_ISEND( w_sbz1, w_sizez10, MPI_REAL8,
     $                kdown, 5, comm_cart, ireq(10), ierr)
!
      call MPI_IRECV( w_rbz1, w_sizez01, MPI_REAL8,
     $                kdown, 6, comm_cart, ireq(11), ierr)
!
      call MPI_ISEND( w_sbz0, w_sizez01, MPI_REAL8,
     $                kup  , 6, comm_cart, ireq(12), ierr)
      call MPI_WAITALL(4, ireq(9), istatus, ierr)
!
      if ( kup >= 0 ) then
c      print *, "rbz0", myrank, kup, ksta1m, kstam-1
         do k = ksta2n, kstan
         do j = jsta2m, jend2m
         do i = ista2m, iend2m
            w(i,j,k) = w_rbz0(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( kdown >= 0 ) then
c      print *, "rbz1", myrank, kdown, kendm+1, kend1m
         do k = kendn+1, kend2n
         do j = jsta2m , jend2m
         do i = ista2m , iend2m
            w(i,j,k) = w_rbz1(i,j,k)
         enddo
         enddo
         enddo
      endif
! --- K-direction
#else  /* BUFFER_K */
! +++ K-direction
      call MPI_IRECV( w(ista-iband, jsta-jband, ksta-kband  ),1,vec(3),
     $                kup  , 5, comm_cart, ireq( 9), ierr)
!
      call MPI_ISEND( w(ista-iband, jsta-jband, kend-kband+1),1,vec(3),
     $                kdown, 5, comm_cart, ireq(10), ierr)
!
      call MPI_IRECV( w(ista-iband, jsta-jband, kend+1      ),1,vec(3),
     $                kdown, 6, comm_cart, ireq(11), ierr)
!
      call MPI_ISEND( w(ista-iband, jsta-jband, ksta        ),1,vec(3),
     $                kup  , 6, comm_cart, ireq(12), ierr)
      call MPI_WAITALL(4, ireq(9), istatus, ierr)
! --- K-direction
#endif /* BUFFER_K */
!
      return
      end subroutine MyMPI_SendReceive_w

! ----------------------------------------------------------------------
      subroutine MyMPI_SendReceive_T (iband, jband, kband, T , vec)
! ----------------------------------------------------------------------
      use Array_Index
!
      implicit none
!
      include 'mpif.h'
!
      integer*4, intent(in) :: iband, jband, kband
      real*8, intent(inout) ::
     $ T( ista2m:iend2m, jsta2m:jend2m, ksta2m:kend2m )
!
      integer*4, intent(in) :: vec(3)
      integer*4 :: ireq(12)
!
      integer*4 :: i, j, k, ib, jb, kb
!
      integer*4 :: ierr
      integer, dimension(MPI_STATUS_SIZE, 12) :: istatus
!
! +++ I-direction
      if ( iup >= 0 ) then
         ib = -(iendm+1)+iend2m
c         print *, myrank, iup, istam, istam+ib, T_sizex10
         do k = ksta2m, kend2m
         do j = jsta2m, jend2m
         do i = istam , istam+ib
            T_sbx0(i,j,k) = T(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( idown >= 0 ) then
         ib = -ista2m+(istam-1)
c         print *, myrank, idown, iendm-ib, iendm, T_sizex01
         do k = ksta2m  , kend2m
         do j = jsta2m  , jend2m
         do i = iendm-ib, iendm
            T_sbx1(i,j,k) = T(i,j,k)
         enddo
         enddo
         enddo
      endif
!
      call MPI_IRECV( T_rbx0,T_sizex10,MPI_REAL8,
     $                iup  , 1, comm_cart, ireq( 1), ierr)
!
      call MPI_ISEND( T_sbx1,T_sizex10,MPI_REAL8,
     $                idown, 1, comm_cart, ireq( 2), ierr)
!
      call MPI_IRECV( T_rbx1,T_sizex01,MPI_REAL8,
     $                idown, 2, comm_cart, ireq( 3), ierr)
!
      call MPI_ISEND( T_sbx0,T_sizex01,MPI_REAL8,
     $                iup  , 2, comm_cart, ireq( 4), ierr)
      call MPI_WAITALL(4, ireq(1), istatus, ierr)
!
      if ( iup >= 0 ) then
c         print *, myrank, iup, ista2m, istam-1, T_sizex10
         do k = ksta2m, kend2m
         do j = jsta2m, jend2m
         do i = ista2m, istam-1
            T(i,j,k) = T_rbx0(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( idown >= 0 ) then
c         print *, myrank, idown, iendm+1, iend2m, T_sizex01
         do k = ksta2m , kend2m
         do j = jsta2m , jend2m
         do i = iendm+1, iend2m
            T(i,j,k) = T_rbx1(i,j,k)
         enddo
         enddo
         enddo
      endif
! --- I-direction
!
#ifdef  BUFFER_J
! +++ J-direction
      if ( jup >= 0 ) then
         jb = -(jendm+1)+jend2m
         do k = ksta2m, kend2m
         do j = jstam , jstam+jb
         do i = ista2m, iend2m
            T_sby0(i,j,k) = T(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( jdown >= 0 ) then
         jb = -jsta2m+(jstam-1)
         do k = ksta2m  , kend2m
         do j = jendm-jb, jendm
         do i = ista2m  , iend2m
            T_sby1(i,j,k) = T(i,j,k)
         enddo
         enddo
         enddo
      endif
!
      call MPI_IRECV( T_rby0, T_sizey10, MPI_REAL8,
     $                jup  , 3, comm_cart, ireq( 5), ierr)
!
      call MPI_ISEND( T_sby1, T_sizey10, MPI_REAL8,
     $                jdown, 3, comm_cart, ireq( 6), ierr)
!
      call MPI_IRECV( T_rby1, T_sizey01, MPI_REAL8,
     $                jdown, 4, comm_cart, ireq( 7), ierr)
!
      call MPI_ISEND( T_sby0, T_sizey01, MPI_REAL8,
     $                jup  , 4, comm_cart, ireq( 8), ierr)
      call MPI_WAITALL(4, ireq(5), istatus, ierr)
!
      if ( jup >= 0 ) then
         do k = ksta2m, kend2m
         do j = jsta2m, jstam-1
         do i = ista2m, iend2m
            T(i,j,k) = T_rby0(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( jdown >= 0 ) then
         do k = ksta2m , kend2m
         do j = jendm+1, jend2m
         do i = ista2m , iend2m
            T(i,j,k) = T_rby1(i,j,k)
         enddo
         enddo
         enddo
      endif
!
! --- J-direction
#else  /* BUFFER_J */
! +++ J-direction
      call MPI_IRECV( T(ista-iband, jsta-jband  , ksta-kband),1,vec(2),
     $                jup  , 3, comm_cart, ireq( 5), ierr)
!
      call MPI_ISEND( T(ista-iband, jend-jband+1, ksta-kband),1,vec(2),
     $                jdown, 3, comm_cart, ireq( 6), ierr)
!
      call MPI_IRECV( T(ista-iband, jend+1      , ksta-kband),1,vec(2),
     $                jdown, 4, comm_cart, ireq( 7), ierr)
!
      call MPI_ISEND( T(ista-iband, jsta        , ksta-kband),1,vec(2),
     $                jup  , 4, comm_cart, ireq( 8), ierr)
      call MPI_WAITALL(4, ireq(5), istatus, ierr)
! --- J-direction
#endif /* BUFFER_J */
!
#ifdef BUFFER_K
! +++ K-direction
      if ( kup >= 0 ) then
         kb = -(kendm+1)+kend2m
         do k = kstam , kstam+kb
         do j = jsta2m, jend2m
         do i = ista2m, iend2m
            T_sbz0(i,j,k) = T(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( kdown >= 0 ) then
         kb = -ksta2m+(kstam-1)
         do k = kendm-kb, kendm
         do j = jsta2m, jend2m
         do i = ista2m, iend2m
            T_sbz1(i,j,k) = T(i,j,k)
         enddo
         enddo
         enddo
      endif
!
      call MPI_IRECV( T_rbz0, T_sizez10, MPI_REAL8,
     $                kup  , 5, comm_cart, ireq( 9), ierr)
!
      call MPI_ISEND( T_sbz1, T_sizez10, MPI_REAL8,
     $                kdown, 5, comm_cart, ireq(10), ierr)
!
      call MPI_IRECV( T_rbz1, T_sizez01, MPI_REAL8,
     $                kdown, 6, comm_cart, ireq(11), ierr)
!
      call MPI_ISEND( T_sbz0, T_sizez01, MPI_REAL8,
     $                kup  , 6, comm_cart, ireq(12), ierr)
      call MPI_WAITALL(4, ireq(9), istatus, ierr)
!
      if ( kup >= 0 ) then
         do k = ksta2m, kstam-1
         do j = jsta2m, jend2m
         do i = ista2m, iend2m
            T(i,j,k) = T_rbz0(i,j,k)
         enddo
         enddo
         enddo
      endif
      if ( kdown >= 0 ) then
         do k = kendm+1, kend2m
         do j = jsta2m , jend2m
         do i = ista2m , iend2m
            T(i,j,k) = T_rbz1(i,j,k)
         enddo
         enddo
         enddo
      endif
! --- K-direction
#else  /* BUFFER_K */
! +++ K-direction
      call MPI_IRECV( T(ista-iband, jsta-jband, ksta-kband  ),1,vec(3),
     $                kup  , 5, comm_cart, ireq( 9), ierr)
!
      call MPI_ISEND( T(ista-iband, jsta-jband, kend-kband+1),1,vec(3),
     $                kdown, 5, comm_cart, ireq(10), ierr)
!
      call MPI_IRECV( T(ista-iband, jsta-jband, kend+1      ),1,vec(3),
     $                kdown, 6, comm_cart, ireq(11), ierr)
!
      call MPI_ISEND( T(ista-iband, jsta-jband, ksta        ),1,vec(3),
     $                kup  , 6, comm_cart, ireq(12), ierr)
      call MPI_WAITALL(4, ireq(9), istatus, ierr)
! --- K-direction
#endif /* BUFFER_K */
!
      return
      end subroutine MyMPI_SendReceive_T
#endif /* _BUFFER_SENDRECEIVE_ */

! ----------------------------------------------------------------------
      subroutine MyMPI_sendp(iband, jband, kband, p, vec, ireq)
! ----------------------------------------------------------------------
      implicit none
!
      include 'mpif.h'
!
      integer*4, intent(in) :: iband, jband, kband
      real*8, dimension( ista-iband:iend+iband
     $                 , jsta-jband:jend+jband
     $                 , ksta-kband:kend+kband), intent(inout) :: p
      integer*4, intent(in) :: vec(3)
!
      integer*4, intent(out) :: ireq(12)
!
      integer*4 :: ierr
      integer, dimension(MPI_STATUS_SIZE, 12) :: istatus
!
! +++ I-direction
      call MPI_IRECV( p( ista-iband  , jsta-jband, ksta-kband),1,vec(1),
     $                iup  , 1, comm_cart, ireq( 1), ierr)
!
      call MPI_ISEND( p( iend-iband+1, jsta-jband, ksta-kband),1,vec(1),
     $                idown, 1, comm_cart, ireq( 2), ierr)
!
      call MPI_IRECV( p( iend+1      , jsta-jband, ksta-kband),1,vec(1),
     $                idown, 2, comm_cart, ireq( 3), ierr)
!
      call MPI_ISEND( p( ista        , jsta-jband, ksta-kband),1,vec(1),
     $                iup  , 2, comm_cart, ireq( 4), ierr)
      call MPI_WAITALL(4, ireq(1), istatus, ierr)
! --- I-direction
!
! +++ J-direction
      call MPI_IRECV( p( ista-iband, jsta-jband  , ksta-kband),1,vec(2),
     $                jup  , 3, comm_cart, ireq( 5), ierr)
!
      call MPI_ISEND( p( ista-iband, jend-jband+1, ksta-kband),1,vec(2),
     $                jdown, 3, comm_cart, ireq( 6), ierr)
!
      call MPI_IRECV( p( ista-iband, jend+1      , ksta-kband),1,vec(2),
     $                jdown, 4, comm_cart, ireq( 7), ierr)
!
      call MPI_ISEND( p( ista-iband, jsta        , ksta-kband),1,vec(2),
     $                jup  , 4, comm_cart, ireq( 8), ierr)
      call MPI_WAITALL(4, ireq(5), istatus, ierr)
! --- J-direction
!
! +++ K-direction
      call MPI_IRECV( p( ista-iband, jsta-jband, ksta-kband  ),1,vec(3),
     $                kup  , 5, comm_cart, ireq( 9), ierr)
!
      call MPI_ISEND( p( ista-iband, jsta-jband, kend-kband+1),1,vec(3),
     $                kdown, 5, comm_cart, ireq(10), ierr)
!
      call MPI_IRECV( p( ista-iband, jsta-jband, kend+1      ),1,vec(3),
     $                kdown, 6, comm_cart, ireq(11), ierr)
!
      call MPI_ISEND( p( ista-iband, jsta-jband, ksta        ),1,vec(3),
     $                kup  , 6, comm_cart, ireq(12), ierr)
      call MPI_WAITALL(4, ireq(9), istatus, ierr)
! --- K-direction
!
      return
      end subroutine MyMPI_sendp
#endif /* _USE_MPI_ */

! <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
      end module MyMPI
! <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      module Parts
! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      implicit none
!
      TYPE partial_data
         integer*4 :: id
         integer*4 :: ista, iend, jsta, jend, ksta, kend
         real*8, dimension(:,:,:), allocatable :: x
      END TYPE partial_data
!
#ifdef _PIEZO_DATA_LIST_
      integer :: nfp
      TYPE( partial_data ), dimension(:), allocatable :: fp
#endif
!
#ifdef _PARTS_DATA_LIST_
      integer :: nep
      TYPE( partial_data ), dimension(:), allocatable :: ep
#endif
!
      contains
!
! ----------------------------------------------------------------------
      subroutine dtloadfp(fname, delta, orig, sp)
! ----------------------------------------------------------------------
      use MyMPI , only : ista, jsta, ksta
!
      implicit none
!
      character*256, intent(in) :: fname
      real*8, intent(out) :: delta(4)
      real*8, intent(out) :: orig(3)
      TYPE( partial_data ), intent(out) :: sp
!
      integer :: i, j, k
! +++ Sph Data File Format
      integer*4 :: svType, dType
      integer*4 :: IMAX, JMAX, KMAX
      real*4 :: XORG, YORG, ZORG
      real*4 :: XPITCH, YPITCH, ZPITCH
      integer*4 :: STEP ! time step
      real*4 :: TIME ! time
      real*4, dimension( :, :, : ), allocatable :: SDATA
      real*4 :: dims0(3)
      integer*4 :: istap, jstap, kstap, band
      integer*4 :: ip, jp, kp
!
      read(fname(index(fname,".sph")-2:index(fname,".sph")-1),*) sp% id
!
      open(10, file=fname, status='old', form='unformatted')
      read(10) svType, dType
      read(10) IMAX, JMAX, KMAX
      read(10) XORG, YORG, ZORG
      read(10) XPITCH, YPITCH, ZPITCH
      read(10) STEP, TIME
!
      allocate( SDATA( IMAX, JMAX, KMAX) )
!
      read(10) SDATA
      read(10) dims0
      read(10) istap, jstap, kstap
      read(10) band
      close(10)

      sp % ista = istap
      sp % iend = istap + IMAX - 1
      sp % jsta = jstap
      sp % jend = jstap + JMAX - 1
      sp % ksta = kstap
      sp % kend = kstap + KMAX - 1
      allocate( sp % x( sp % ista : sp % iend,
     $                  sp % jsta : sp % jend,
     $                  sp % ksta : sp % kend ) )
      do k = 1, KMAX
         kp = k + sp % ksta - 1
      do j = 1, JMAX
         jp = j + sp % jsta - 1
      do i = 1, IMAX
         ip = i + sp % ista - 1
         sp % x(ip,jp,kp) = dble( SDATA(i,j,k) )
      enddo
      enddo
      enddo
!
      deallocate( SDATA )
!
      delta(1)=dble(XPITCH)
      delta(2)=dble(YPITCH)
      delta(3)=dble(ZPITCH)
      XORG = XORG + real( ista - istap ) * XPITCH
      YORG = YORG + real( jsta - jstap ) * YPITCH
      ZORG = ZORG + real( ksta - kstap ) * ZPITCH
      orig(1)=dble(XORG)
      orig(2)=dble(YORG)
      orig(3)=dble(ZORG)
!
      return
      end subroutine dtloadfp
!
! ----------------------------------------------------------------------
      subroutine Parts_Load( fname, nsp, sp )
! ----------------------------------------------------------------------
      implicit none
!
      character*256, intent(in) :: fname
      integer*4, intent(out) :: nsp
      TYPE( partial_data ), dimension(:), allocatable, intent(out) :: sp
!
      integer*4 :: n
      character*256 :: fnamein
      real*8 :: delta(4), orig(3)
!
      write(*,*) "Check list of ", fname
      open(2, file=fname, status='old')
      read(2,*) nsp
      write(*,*) "nsp =", nsp
!
      allocate( sp(0:nsp) )
!
      if ( nsp /= 0 ) then
!
      do n = 1, nsp
         read(2,'(a)') fnamein
         write(*,*) "Data load from ", fnamein
! Load Piezo
         call dtloadfp(fnamein, delta, orig, sp(n))
!
         write(*,*) "Piezo ID is ", sp(n)% id
! Check pitch
         if ( real(_PITCH_) /= real(delta(1)) .or.
     $        real(_PITCH_) /= real(delta(2)) .or.
     $        real(_PITCH_) /= real(delta(3)) ) then
            write(*,'("Pitch is not matched !!!")')
            write(*,*) fnamein, _PITCH_, delta(1)
         endif
      enddo
!
      endif
!
      close(2)
!
      return
      end subroutine Parts_Load

! ----------------------------------------------------------------------
      subroutine Parts_SDFtoVF( nsp, sp )
! ----------------------------------------------------------------------
      implicit none
!
      integer*4, intent(in) :: nsp
      TYPE( partial_data ), intent(inout) :: sp(0:nsp)
!
      real*8, parameter :: eps = _PITCH_
      real*8 :: pi
      real*8 :: dist
      integer*4 :: i, j, k, n
!
      if ( nsp /= 0 ) then
!
      pi = dacos(-1.d0)
!
! smoothed Heaviside function
      do n = 1, nsp
      do k = sp(n)% ksta, sp(n)% kend
      do j = sp(n)% jsta, sp(n)% jend
      do i = sp(n)% ista, sp(n)% iend
!
         dist = sp(n)% x(i,j,k)
!
         if (eps <= dist) then
            sp(n)% x(i,j,k) = 1.d0
         else if (dist <= -eps) then
            sp(n)% x(i,j,k) = 0.d0
         else if ((-eps < dist).or.(dist < eps)) then
            sp(n)% x(i,j,k) =
     $               (dist+eps)/(2.d0*eps) + dsin(pi*dist/eps)/(2.d0*pi)
         endif
      enddo
      enddo
      enddo
      enddo
!
      endif
!
      return
      end subroutine Parts_SDFtoVF

! <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
      end module Parts
! <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      module PML
! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      implicit none
!
      real*8, dimension(:), allocatable :: sigma_mx, sigma_my, sigma_mz
      real*8, dimension(:), allocatable :: sigma_nx, sigma_ny, sigma_nz
!
      contains
!
! ----------------------------------------------------------------------
      subroutine PML_Init
! ----------------------------------------------------------------------
      use Array_Index 
!
      implicit none
!
      integer*4, parameter :: npml=_NPML_
      real*8 :: sigma_max
      integer*4 :: i, j, k, n
      integer*4 :: itmps, itmpe, jtmps, jtmpe, ktmps, ktmpe
!
      allocate( sigma_mx(ista1m:iend1m) ) ; sigma_mx = 0.d0
      allocate( sigma_my(jsta1m:jend1m) ) ; sigma_my = 0.d0
      allocate( sigma_mz(ksta1m:kend1m) ) ; sigma_mz = 0.d0
      allocate( sigma_nx(ista2n:iend2n) ) ; sigma_nx = 0.d0
      allocate( sigma_ny(jsta2n:jend2n) ) ; sigma_ny = 0.d0
      allocate( sigma_nz(ksta2n:kend2n) ) ; sigma_nz = 0.d0
!
#define _PML_SIGMA_ 1.d-10
#define _PML_N_ 2
! +++ PML damping coefficient
      sigma_max = -dlog((_PML_SIGMA_))*.5d0*(dble(_PML_N_)+1.d0)
     $        /((_PITCH_)*dble(npml))
!
#if defined( _PML_X0_ )
      if ( ista1m <= ista_G + npml - 1 ) then
         itmps = MAX( ista1m, ista_G + ista1m - istam )
         itmpe = MIN( iend1m, ista_G + npml - 1)
         do i = itmps , itmpe
            n = npml - ( i - ista_G )
            sigma_mx(i)=sigma_max*((dble(n)-.5d0)/dble(npml))**(_PML_N_)
         enddo
      endif
#endif
!
#if defined( _PML_X1_ )
      if ( iend_G - npml + 1 <= iend1m ) then
         itmps = MAX( ista1m, iend_G - npml + 1 )
         itmpe = MIN( iend1m, iend_G + iend1m - iendm )
         do i = itmps , itmpe
            n = npml + ( i - iend_G )
            sigma_mx(i)=sigma_max*((dble(n)-.5d0)/dble(npml))**(_PML_N_)
         enddo
      endif
#endif
!
#if defined( _PML_Y0_ )
      if ( jsta1m <= jsta_G + npml - 1 ) then
         jtmps = MAX( jsta1m, jsta_G + jsta1m - jstam )
         jtmpe = MIN( jend1m, jsta_G + npml - 1 )
         do j = jtmps , jtmpe
            n = npml - ( j - jsta_G )
            sigma_my(j)=sigma_max*((dble(n)-.5d0)/dble(npml))**(_PML_N_)
         enddo
      endif
#endif
!
#if defined( _PML_Y1_ )
      if ( jend_G - npml + 1 <= jend1m ) then
         jtmps = MAX( jsta1m, jend_G - npml + 1 )
         jtmpe = MIN( jend1m, jend_G + jend1m - jendm )
         do j = jtmps, jtmpe
            n = npml + ( j - jend_G )
            sigma_my(j)=sigma_max*((dble(n)-.5d0)/dble(npml))**(_PML_N_)
         enddo
      endif
#endif
!
#if defined( _PML_Z0_ )
      if ( ksta1m <= ksta_G + npml - 1 ) then
         ktmps = MAX( ksta1m, ksta_G + ksta1m - kstam )
         ktmpe = MIN( kend1m, ksta_G + npml - 1 )
         do k = ktmps , ktmpe
            n = npml - ( k - ksta_G )
            sigma_mz(k)=sigma_max*((dble(n)-.5d0)/dble(npml))**(_PML_N_)
         enddo
      endif
#endif
!
#if defined( _PML_Z1_ )
      if ( kend_G - npml + 1 <= kend1m ) then
         ktmps = MAX( ksta1m, kend_G - npml + 1 )
         ktmpe = MIN( kend1m, kend_G + kend1m - kendm )
         do k = ktmps , ktmpe
            n = npml + ( k - kend_G )
            sigma_mz(k)=sigma_max*((dble(n)-.5d0)/dble(npml))**(_PML_N_)
         enddo
      endif
#endif
!
#if defined( _PML_X0_ )
      if ( ista2n <= ista_G - 1 + npml - 1 ) then
         itmps = MAX( ista2n, ista_G - 1 + ista2n - istan )
         itmpe = MIN( iend2n, ista_G - 1 + npml - 1 )
         do i = itmps , itmpe
            n = npml - ( i - ista_G + 1 )
            sigma_nx(i) = sigma_max*(dble(n)/dble(npml))**(_PML_N_)
         enddo
      endif
#endif
!
#if defined( _PML_X1_ )
      if ( iend_G - npml + 1 <= iend2n ) then
         itmps = MAX( ista2n, iend_G - npml + 1 )
         itmpe = MIN( iend2n, iend_G + iend2n - iendn )
         do i = itmps , itmpe
            n = npml + ( i - iend_G )
            sigma_nx(i) = sigma_max*(dble(n)/dble(npml))**(_PML_N_)
         enddo
      endif
#endif
!
#if defined( _PML_Y0_ )
      if ( jsta2n <= jsta_G - 1 + npml - 1 ) then
         jtmps = MAX( jsta2n, jsta_G - 1 + jsta2n - jstan )
         jtmpe = MIN( jend2n, jsta_G - 1 + npml - 1 )
         do j = jtmps , jtmpe
            n = npml - ( j - jsta_G + 1 )
            sigma_ny(j) = sigma_max*(dble(n)/dble(npml))**(_PML_N_)
         enddo
      endif
#endif
!
#if defined( _PML_Y1_ )
      if ( jend_G - npml + 1 <= jend2n ) then
         jtmps = MAX( jsta2n, jend_G - npml + 1 )
         jtmpe = MIN( jend2n, jend_G + jend2n - jendn )
         do j = jtmps , jtmpe
            n = npml + ( j - jend_G )
            sigma_ny(j) = sigma_max*(dble(n)/dble(npml))**(_PML_N_)
         enddo
      endif
#endif
!
#if defined( _PML_Z0_ )
      if ( ksta2n <= ksta_G - 1 + npml - 1 ) then
         ktmps = MAX( ksta2n, ksta_G - 1 + ksta2n - kstan )
         ktmpe = MIN( kend2n, ksta_G - 1 + npml - 1 )
         do k = ktmps , ktmpe
            n = npml - ( k - ksta_G + 1 )
            sigma_nz(k) = sigma_max*(dble(n)/dble(npml))**(_PML_N_)
         enddo
      endif
#endif
!
#if defined( _PML_Z1_ )
      if ( kend_G - npml + 1 <= kend2n ) then
         ktmps = MAX( ksta2n, kend_G - npml + 1 )
         ktmpe = MIN( kend2n, kend_G + kend2n - kendn )
         do k = ktmps , ktmpe
            n = npml + ( k - kend_G )
            sigma_nz(k) = sigma_max*(dble(n)/dble(npml))**(_PML_N_)
         enddo
      endif
#endif
!
      return
      end subroutine PML_Init
!
! <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
      end module PML
! <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      module HIFU3D
! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      implicit none
!
! +++ Restart properties
      real*8, dimension ( :, :, :), allocatable :: p0
      real*8, dimension ( :, :, :), allocatable :: px, py, pz
      real*8, dimension ( :, :, :), allocatable :: u, v, w
#ifdef _IMPLICIT_VIS_
      real*8, dimension ( :, :, :), allocatable :: un, vn, wn
#endif
#ifdef _TEMPERATURE_
      real*8, dimension ( :, :, :), allocatable :: T
#endif
#ifdef _THERMAL_DOSE_
      real*8, dimension ( :, :, :), allocatable :: Dose
#endif
#ifdef _DENATURATION_
      real*8, dimension ( :, :, :), allocatable :: phi
#endif
! --- Restart properties
!
      real*8, dimension ( :, :, :), allocatable :: pmax
      real*8, dimension ( :, :, :), allocatable :: rhom, csm2
!TUNE20**************************************************************>>>
!     real*8, dimension ( :, :, :), allocatable :: exx, eyy, ezz, ekk
!TUNE20-----------------------------------------------------------------
      real*8, dimension ( :, :, :), allocatable :: exx, eyy, ezz
!TUNE20END<<<***********************************************************
      real*8, dimension ( :, :, :), allocatable :: exy, eyz, ezx
!
      integer*4, dimension (:, :, :), allocatable :: mask
!TUNE09**************************************************************>>>
      real*8, dimension (:, :, :, :), allocatable :: mask1
!TUNE09END<<<***********************************************************
      real*8, dimension (:, :, :, :), allocatable :: mask2
#ifdef _VOLUME_OBJECT_
      real*8, dimension ( :, :, :), allocatable :: fb
#endif
#ifdef _ARRAY_CONST_
      real*8, dimension (:, :, :), allocatable :: Mum, Kappam
      real*8, dimension (:, :, :), allocatable :: Betam, rCpm
#endif
!     Work array
      real*8, dimension ( :, :, :), allocatable :: w1, w2
!TUNE16**************************************************************>>>
      real*8, dimension ( :, :, :), allocatable :: w3, w4
!TUNE16END<<<***********************************************************
!
      real*8 :: dt
      real*8 :: dtau
!
      contains
!
! ----------------------------------------------------------------------
      subroutine HIFU3D_Allocate
! ----------------------------------------------------------------------
      use Array_Index
!
! +++ Allocation 
      allocate( p0(ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
      allocate( px(ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
      allocate( py(ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
      allocate( pz(ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
      allocate( u (ista2n:iend2n, jsta2m:jend2m, ksta2m:kend2m) )
      allocate( v (ista2m:iend2m, jsta2n:jend2n, ksta2m:kend2m) )
      allocate( w (ista2m:iend2m, jsta2m:jend2m, ksta2n:kend2n) )
#ifdef _IMPLICIT_VIS_
      allocate( un(ista2n:iend2n, jsta2m:jend2m, ksta2m:kend2m) )
      allocate( vn(ista2m:iend2m, jsta2n:jend2n, ksta2m:kend2m) )
      allocate( wn(ista2m:iend2m, jsta2m:jend2m, ksta2n:kend2n) )
#endif
#ifdef _TEMPERATURE_
      allocate( T (ista2m:iend2m, jsta2m:jend2m, ksta2m:kend2m) )
#endif
#ifdef _THERMAL_DOSE_
      allocate( Dose(ista2m:iend2m, jsta2m:jend2m, ksta2m:kend2m) )
#endif
#ifdef _DENATURATION_
      allocate( phi(ista2m:iend2m, jsta2m:jend2m, ksta2m:kend2m) )
#endif
!
      allocate( pmax(ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
!
      allocate( rhom(ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
      allocate( csm2(ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
!
!TUNE20**************************************************************>>>
!     allocate( ekk (ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
!TUNE20END<<<***********************************************************
      allocate( exx (ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
      allocate( eyy (ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
      allocate( ezz (ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
      allocate( exy (ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
      allocate( eyz (ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
      allocate( ezx (ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
!
      allocate( mask(ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
!TUNE09**************************************************************>>>
      allocate( mask1(3, istam:iendm, jstam:jendm, kstam:kendm) )
!TUNE09END<<<***********************************************************
      allocate( mask2(3, ista1n:iend1n, jsta1n:jend1n, ksta1n:kend1n) )
#ifdef _VOLUME_OBJECT_
      allocate( fb(ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
#endif
#ifdef _ARRAY_CONST_
      allocate( Mum(ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
      allocate( Kappam(ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
      allocate( Betam(ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
      allocate( rCpm(ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
#endif
!
      allocate( w1(ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
      allocate( w2(ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
!TUNE16**************************************************************>>>
      allocate( w3(ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
      allocate( w4(ista1m:iend1m, jsta1m:jend1m, ksta1m:kend1m) )
!TUNE16END<<<***********************************************************
!
      return
      end subroutine HIFU3D_Allocate

! ----------------------------------------------------------------------
      subroutine HIFU3D_Initialize_Array
! ----------------------------------------------------------------------
      implicit none
!
      p0 = 0.d0
      px = 0.d0
      py = 0.d0
      pz = 0.d0
      u  = 0.d0
      v  = 0.d0
      w  = 0.d0
#ifdef _IMPLICIT_VIS_
      un = 0.d0
      vn = 0.d0
      wn = 0.d0
#endif
#ifdef _TEMPERATURE_
      T  = 0.d0
#endif
#ifdef _THERMAL_DOSE_
      Dose = 0.d0
#endif
#ifdef _DENATURATION_
      phi = 1.d0
#endif
!
      pmax = 0.d0
!
      rhom = 0.d0
      csm2 = 0.d0
!
!TUNE20**************************************************************>>>
!     ekk = 0.d0
!TUNE20END<<<***********************************************************
      exx = 0.d0
      eyy = 0.d0
      ezz = 0.d0
      exy = 0.d0
      eyz = 0.d0
      ezx = 0.d0
!
      mask = 0
#ifdef _VOLUME_OBJECT_
      fb = 0.d0
#endif
#ifdef _ARRAY_CONST_
      Mum    = 0.d0
      Kappam = 0.d0
      Betam  = 0.d0
      rCpm   = 0.d0
#endif
!
      w1 = 0.d0
      w2 = 0.d0
!TUNE16**************************************************************>>>
      w3 = 0.d0
      w4 = 0.d0
!TUNE16END<<<***********************************************************
!
      return
      end subroutine HIFU3D_Initialize_Array

! ----------------------------------------------------------------------
      subroutine HIFU3D_Restart(fname, ifrag, nstep)
! ----------------------------------------------------------------------
      implicit none
      character*256, intent(in   ) :: fname
      integer*4,     intent(in   ) :: ifrag
      integer*4,     intent(inout) :: nstep
!
      if ( ifrag == 0 ) then
         open(10,file=fname, status='unknown', form='unformatted')
         write(10) nstep
         write(10) p0
         write(10) u
         write(10) v
         write(10) w
         write(10) px
         write(10) py
         write(10) pz
#ifdef _TEMPERATURE_
         write(10) T
#endif
#ifdef _THERMAL_DOSE_
         write(10) Dose
#endif
#ifdef _DENATURATION_
         write(10) phi
#endif
         close(10)
      else if ( ifrag == 1 ) then
         open(11,file=fname, status='old', form='unformatted')
         read(11) nstep
         read(11) p0
         read(11) u
         read(11) v
         read(11) w
         read(11) px
         read(11) py
         read(11) pz
#ifdef _RECALCULATION_TEMP_
         nstep = 0
#else
#ifdef _TEMPERATURE_
         read(11) T
#endif
#ifdef _THERMAL_DOSE_
         read(11) Dose
#endif
#ifdef _DENATURATION_
         read(11) phi
#endif
#endif // _RECALCULALTION_TEMP_
         close(11)
         write(*,*) 'Restart from nstep=',nstep
      endif
!
      return
      end subroutine HIFU3D_Restart
!
! ----------------------------------------------------------------------
      subroutine HIFU3D_Constant_Mixture_Properties
! ----------------------------------------------------------------------
      use Array_Index
!
      use Physical_Properties
!
      use Parts
!
      implicit none
!
      integer*4 :: i, j, k
      integer*4 :: ID
!
#ifdef _ARRAY_CONST_
!ocl temp( ID )
      do k = ksta1m, kend1m
      do j = jsta1m, jend1m
      do i = ista1m, iend1m
         ID = mask(i,j,k)
         Mum(i,j,k) = Mu(ID)
         Kappam(i,j,k) = Kappa(ID)
#ifdef _ZERO_HEATCONDUCTION_AIR_
     $                  * DBLE( 1 - AirID(ID) )
#endif /* _ZERO_HEATCONDUCTION_AIR_ */
         Betam(i,j,k) = Beta(ID)
      enddo
      enddo
      enddo
! --- 
#endif /* _ARRAY_CONST_ */
!
      end subroutine HIFU3D_Constant_Mixture_Properties
!
! ----------------------------------------------------------------------
      subroutine HIFU3D_Update_Mixture_Properties
! ----------------------------------------------------------------------
      use Array_Index
!
      use Physical_Properties
!
      use Parts
!
      implicit none
!
      real*8 :: s0, s1, s2, s3
      integer*4 :: ID, i, j, k, n
#ifdef _DENATURATION_
      real*8, parameter :: rho_coagulated = 1.d0
      real*8, parameter :: rK_coagulated = 1.d0/(_DENATURATION_CS_)**2
      real*8 :: fc
#endif /* _DENATURATION_ */
!
! +++ Density & sound speed of liquid
!
!ocl xfill
      rhom = 0.d0
!ocl xfill
      csm2 = 0.d0
#ifdef _ARRAY_CONST_
!ocl xfill
      rCpm = 0.d0
#endif
! If volume fraction of object is defined, w1 can be replaced by it.
!ocl xfill
      w1 = 0.d0
!
#if defined( _PIEZO_DATA_LIST_ )
      if ( nfp /= 0 ) then
#define _PROPERTY_ID_PIEZO_ 255
c#define _PROPERTY_ID_PIEZO_ 0
         ID = _PROPERTY_ID_PIEZO_
      do n = 1, nfp
!ocl temp(s1, s2, s3)
      do k = fp(n)% ksta, fp(n)% kend
      do j = fp(n)% jsta, fp(n)% jend
      do i = fp(n)% ista, fp(n)% iend
         s1 = rho0( ID )
         s2 = rho0( ID ) * cs0( ID ) * cs0( ID )
         s3 = rho0( ID ) * Cp( ID )
         rhom(i,j,k) = rhom(i,j,k) + fp(n)% x(i,j,k) * s1
         csm2(i,j,k) = csm2(i,j,k) + fp(n)% x(i,j,k) / s2
#ifdef _ARRAY_CONST_
         rCpm(i,j,k) = rCpm(i,j,k) + fp(n)% x(i,j,k) * s3
#endif
!         
         w1(i,j,k) = w1(i,j,k) + fp(n)% x(i,j,k)
      enddo
      enddo
      enddo
      enddo
      endif
#endif /* _PIEZO_DATA_LIST_ */
!
#if defined( _PARTS_DATA_LIST_ )
      if ( nep /= 0 ) then
      do n = 1, nep
         ID = ep(n)% ID
!ocl temp(s1, s2, s3)
      do k = ep(n)% ksta, ep(n)% kend
      do j = ep(n)% jsta, ep(n)% jend
      do i = ep(n)% ista, ep(n)% iend
         s1 = rho0( ID )
         s2 = rho0( ID ) * cs0( ID ) * cs0( ID )
         s3 = rho0( ID ) * Cp( ID )
         rhom(i,j,k) = rhom(i,j,k) + ep(n)% x(i,j,k) * s1
         csm2(i,j,k) = csm2(i,j,k) + ep(n)% x(i,j,k) / s2
#ifdef _ARRAY_CONST_
         rCpm(i,j,k) = rCpm(i,j,k) + ep(n)% x(i,j,k) * s3
#endif
!         
         w1(i,j,k) = w1(i,j,k) + ep(n)% x(i,j,k)
      enddo
      enddo
      enddo
      enddo
      endif
#endif /* _PARTS_DATA_LIST_ */
!
!ocl temp(ID, s0, s1, s2, s3, fc)
      do k = ksta1m, kend1m
      do j = jsta1m, jend1m
      do i = ista1m, iend1m
!
#ifdef _DENATURATION_
         if ( phi(i,j,k) <= 0.d0 ) then
           fc = 1.d0
         else if ( 1.d0 <= phi(i,j,k) ) then
           fc = 0.d0
         else
!TUNE15**************************************************************>>>
!          fc = 1.d0 - phi(i,j,k)*phi(i,j,k)*(3.d0-2.d0*phi(i,j,k))
!TUNE15-----------------------------------------------------------------
           fc = 1.0d0 - 3.0d0*phi(i,j,k)*phi(i,j,k)
     $                + 2.0d0*phi(i,j,k)*phi(i,j,k)*phi(i,j,k)
!TUNE15END<<<***********************************************************
         endif
#endif /* _DENATURATION_ */
!
#ifdef _EOS_TAIT_
         ID = mask(i,j,k)
         s1 = Coef_Rho(ID)*(p0(i,j,k)+p_inf+A1(ID))**rGamma(ID)
c         s1 = Coef_Rho(ID)*dexp(rGamma(ID)*dlog(p0(i,j,k)+p_inf+A1(ID)))
         s2 = Gamma(ID)*(p0(i,j,k)+p_inf+A1(ID))
         s2 = 1.d0 / s2
         s3 = s1 * Cp(ID)
#else
         ID = mask(i,j,k)
         s1 = rho0( ID )
         s2 = rho0( ID ) * cs0( ID ) * cs0( ID )
         s2 = 1.d0 / s2
         s3 = rho0( ID ) * Cp( ID )
#endif /* _EOS_TAIT_ */
!
#ifdef _DENATURATION_
         s0 = 1.d0 - w1(i,j,k) - fc
         s1 = rhom(i,j,k) + s0 * s1 + fc * s1 * rho_coagulated
         s2 = csm2(i,j,k) + s0 * s2 + fc * s2 * rK_coagulated
#ifdef _ARRAY_CONST_
         s3 = rCpm(i,j,k) + s0 * s3 + fc * s3
#endif
#else
         s0 = 1.d0 - w1(i,j,k)
         s1 = rhom(i,j,k) + s0 * s1
         s2 = csm2(i,j,k) + s0 * s2
#ifdef _ARRAY_CONST_
         s3 = rCpm(i,j,k) + s0 * s3
#endif
#endif /* _DENATURATION_ */
!
         rhom(i,j,k) = s1
         csm2(i,j,k) = 1.d0 / ( s1 * s2 )
#ifdef _ARRAY_CONST_
         rCpm(i,j,k) = s1 / s3
#endif
!
      enddo
      enddo
      enddo
!
      end subroutine HIFU3D_Update_Mixture_Properties

! ----------------------------------------------------------------------
      subroutine HIFU3D_BC_Mirror( dims, sp )
! ----------------------------------------------------------------------
      use Array_Index
!
      implicit none
!
      integer, intent(in) :: dims(0:1, 3)
      real*8, dimension(  dims(0,1):dims(1,1)
     $                  , dims(0,2):dims(1,2)
     $                  , dims(0,3):dims(1,3) ), intent(inout) :: sp
!
      integer*4 :: i, j, k
! +++ +++ B.C.
      if ( istam == ista_G ) then
         do k = kstam , kendm
         do j = jstam , jendm
         do i = ista1m, istam-1
#if defined( _MIRROR_X0_ )
            sp(i,j,k) = sp(istam-i,j,k)
#else
            sp(i,j,k) = sp(istam,j,k)
#endif
         enddo
         enddo
         enddo
      endif
!
      if ( iendm == iend_G ) then
         do k = kstam  , kendm
         do j = jstam  , jendm
         do i = iendm+1, iend1m
#if defined( _MIRROR_X1_ )
            sp(i,j,k) = sp(iendm-(i-iendm-1),j,k)
#else
            sp(i,j,k) = sp(iendm,j,k)
#endif
         enddo
         enddo
         enddo
      endif
!      
      if ( jstam == jsta_G ) then
         do k = kstam , kendm
         do j = jsta1m, jstam-1
         do i = istam , iendm
#if defined( _MIRROR_Y0_ )
            sp(i,j,k) = sp(i,jstam-j,k)
#else
            sp(i,j,k) = sp(i,jstam,k)
#endif
         enddo
         enddo
         enddo
      endif
!
      if ( jendm == jend_G ) then
         do k = kstam  , kendm
         do j = jendm+1, jend1m
         do i = istam  , iendm
#if defined( _MIRROR_Y1_ )
            sp(i,j,k) = sp(i,jendm-(j-jendm-1),k)
#else
            sp(i,j,k) = sp(i,jendm,k)
#endif
         enddo
         enddo
         enddo
      endif
!
      if ( kstam == ksta_G ) then
         do k = ksta1m, kstam-1
         do j = jstam , jendm
         do i = istam , iendm
#if defined( _MIRROR_Z0_ )
            sp(i,j,k) = sp(i,j,kstam-k)
#else
            sp(i,j,k) = sp(i,j,kstam)
#endif
         enddo
         enddo
         enddo
      endif
!
      if ( kendm == kend_G ) then
         do k = kendm+1, kend1m
         do j = jstam  , jendm
         do i = istam  , iendm
#if defined( _MIRROR_Z1_ )
            sp(i,j,k) = sp(i,j,kendm-(k-kendm-1))
#else
            sp(i,j,k) = sp(i,j,kendm)
#endif
         enddo
         enddo
         enddo
      endif
! ----------------------------------------------------------------------
      end subroutine HIFU3D_BC_Mirror
! ----------------------------------------------------------------------

! ----------------------------------------------------------------------
      subroutine HIFU3D_Cal_Viscous
! ----------------------------------------------------------------------
      use FDM
      use Array_Index
      use Physical_Properties
!
      implicit none
!
      integer*4 :: i, j, k
      integer*4 :: ID
      real*8 :: s0, s1, s2, s3, s4, s00, s01, s10, s11
!
      call START_COLLECTION("vis_loop1")
! +++ Exx, Eyy, Ezz
!TUNE20**************************************************************>>>
c!ocl temp(s0, s1, s2, s3, ID)
!TUNE20-----------------------------------------------------------------
c!ocl temp(s0, s1, s2, s3, s4, ID)
!ocl temp(ID, s0, s1, s2, s3, s4, s00, s10, s01, s11)
!TUNE20END<<<***********************************************************
      do k = ksta1m, kend1m
      do j = jsta1m, jend1m
      do i = ista1m, iend1m
#ifdef _IMPLICIT_VIS_
!
#ifdef _UNROLL_FDM_
         s1 =  d1m(ns  )*u (i+ns  ,j,k)
     $        +d1m(ns+1)*u (i+ns+1,j,k)
     $        +d1m(ns+2)*u (i+ns+2,j,k)
     $        +d1m(ns+3)*u (i+ns+3,j,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*u (i+ns+4,j,k)
     $        +d1m(ns+5)*u (i+ns+5,j,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*u (i+ns+6,j,k)
     $        +d1m(ns+7)*u (i+ns+7,j,k)
#endif
     $        +d1m(ns  )*un(i+ns  ,j,k)
     $        +d1m(ns+1)*un(i+ns+1,j,k)
     $        +d1m(ns+2)*un(i+ns+2,j,k)
     $        +d1m(ns+3)*un(i+ns+3,j,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*un(i+ns+4,j,k)
     $        +d1m(ns+5)*un(i+ns+5,j,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*un(i+ns+6,j,k)
     $        +d1m(ns+7)*un(i+ns+7,j,k)
#endif
         s2 =  d1m(ns  )*v (i,j+ns  ,k)
     $        +d1m(ns+1)*v (i,j+ns+1,k)
     $        +d1m(ns+2)*v (i,j+ns+2,k)
     $        +d1m(ns+3)*v (i,j+ns+3,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*v (i,j+ns+4,k)
     $        +d1m(ns+5)*v (i,j+ns+5,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*v (i,j+ns+6,k)
     $        +d1m(ns+7)*v (i,j+ns+7,k)
#endif
     $        +d1m(ns  )*vn(i,j+ns  ,k)
     $        +d1m(ns+1)*vn(i,j+ns+1,k)
     $        +d1m(ns+2)*vn(i,j+ns+2,k)
     $        +d1m(ns+3)*vn(i,j+ns+3,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*vn(i,j+ns+4,k)
     $        +d1m(ns+5)*vn(i,j+ns+5,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*vn(i,j+ns+6,k)
     $        +d1m(ns+7)*vn(i,j+ns+7,k)
#endif
         s3 =  d1m(ns  )*w (i,j,k+ns  )
     $        +d1m(ns+1)*w (i,j,k+ns+1)
     $        +d1m(ns+2)*w (i,j,k+ns+2)
     $        +d1m(ns+3)*w (i,j,k+ns+3)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*w (i,j,k+ns+4)
     $        +d1m(ns+5)*w (i,j,k+ns+5)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*w (i,j,k+ns+6)
     $        +d1m(ns+7)*w (i,j,k+ns+7)
#endif
     $        +d1m(ns  )*wn(i,j,k+ns  )
     $        +d1m(ns+1)*wn(i,j,k+ns+1)
     $        +d1m(ns+2)*wn(i,j,k+ns+2)
     $        +d1m(ns+3)*wn(i,j,k+ns+3)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*wn(i,j,k+ns+4)
     $        +d1m(ns+5)*wn(i,j,k+ns+5)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*wn(i,j,k+ns+6)
     $        +d1m(ns+7)*wn(i,j,k+ns+7)
#endif
#else /* _UNROLL_FDM_ */
         s1=0.d0
         s2=0.d0
         s3=0.d0
         do n = ns, ne
            s1 = s1 + d1m(n)*u(i+n,j,k) + d1m(n)*un(i+n,j,k)
            s2 = s2 + d1m(n)*v(i,j+n,k) + d1m(n)*vn(i,j+n,k)
            s3 = s3 + d1m(n)*w(i,j,k+n) + d1m(n)*wn(i,j,k+n)
         enddo
#endif /* _UNROLL_FDM_ */
!
         s1 = .5d0 * s1
         s2 = .5d0 * s2
         s3 = .5d0 * s3
!
#else /* _IMPLICIT_VIS_ */
!
#ifdef _UNROLL_FDM_
         s1 =  d1m(ns  )*u(i+ns  ,j,k)
     $        +d1m(ns+1)*u(i+ns+1,j,k)
     $        +d1m(ns+2)*u(i+ns+2,j,k)
     $        +d1m(ns+3)*u(i+ns+3,j,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*u(i+ns+4,j,k)
     $        +d1m(ns+5)*u(i+ns+5,j,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*u(i+ns+6,j,k)
     $        +d1m(ns+7)*u(i+ns+7,j,k)
#endif
         s2 =  d1m(ns  )*v(i,j+ns  ,k)
     $        +d1m(ns+1)*v(i,j+ns+1,k)
     $        +d1m(ns+2)*v(i,j+ns+2,k)
     $        +d1m(ns+3)*v(i,j+ns+3,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*v(i,j+ns+4,k)
     $        +d1m(ns+5)*v(i,j+ns+5,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*v(i,j+ns+6,k)
     $        +d1m(ns+7)*v(i,j+ns+7,k)
#endif
         s3 =  d1m(ns  )*w(i,j,k+ns  )
     $        +d1m(ns+1)*w(i,j,k+ns+1)
     $        +d1m(ns+2)*w(i,j,k+ns+2)
     $        +d1m(ns+3)*w(i,j,k+ns+3)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*w(i,j,k+ns+4)
     $        +d1m(ns+5)*w(i,j,k+ns+5)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*w(i,j,k+ns+6)
     $        +d1m(ns+7)*w(i,j,k+ns+7)
#endif
#else /* _UNROLL_FDM_ */
         s1=0.d0
         s2=0.d0
         s3=0.d0
         do n = ns, ne
            s1 = s1 + d1m(n)*u(i+n,j,k)
            s2 = s2 + d1m(n)*v(i,j+n,k)
            s3 = s3 + d1m(n)*w(i,j,k+n)
         enddo
#endif /* _UNROLL_FDM_ */
!
#endif /* _IMPLICIT_VIS_ */
!
         s0 = s1 + s2 + s3
#ifdef _ARRAY_CONST_
!TUNE20**************************************************************>>>
!        ekk(i,j,k) = s0 * Lambda(ID)
!        exx(i,j,k) = s1 * 2.d0 * Mum(i,j,k)
!        eyy(i,j,k) = s2 * 2.d0 * Mum(i,j,k)
!        ezz(i,j,k) = s3 * 2.d0 * Mum(i,j,k)
!TUNE20-----------------------------------------------------------------
         s4 = s0 * 2.d0 * Mum(i,j,k) - p0(i,j,k)
         exx(i,j,k) = s1 * 2.d0 * Mum(i,j,k) + s4
         eyy(i,j,k) = s2 * 2.d0 * Mum(i,j,k) + s4
         ezz(i,j,k) = s3 * 2.d0 * Mum(i,j,k) + s4
!TUNE20END<<<***********************************************************
         w1(i,j,k) =  2.d0 * Mum(i,j,k) * s0*s0
     $              + 2.d0 * Mum(i,j,k) * ( s1*s1 + s2*s2 + s3*s3 )
#else
         ID = mask(i,j,k)
!TUNE20**************************************************************>>>
!        ekk(i,j,k) = s0 * Lambda(ID)
!        exx(i,j,k) = s1 * 2.d0 * Mu(ID)
!        eyy(i,j,k) = s2 * 2.d0 * Mu(ID)
!        ezz(i,j,k) = s3 * 2.d0 * Mu(ID)
!TUNE20-----------------------------------------------------------------
         s4 = s0 * 2.d0 * Mu(ID) - p0(i,j,k)
         exx(i,j,k) = s1 * 2.d0 * Mu(ID) + s4
         eyy(i,j,k) = s2 * 2.d0 * Mu(ID) + s4
         ezz(i,j,k) = s3 * 2.d0 * Mu(ID) + s4
!TUNE20END<<<***********************************************************
         w1(i,j,k) =  2.d0 * Mu(ID) * s0*s0
     $              + 2.d0 * Mu(ID) * ( s1*s1 + s2*s2 + s3*s3 )
#endif /* _ARRAY_CONST_ */
      enddo
c      enddo
c      enddo
! --- Exx, Eyy, Ezz
!
! +++ Exy
c!ocl temp(s0, s1, s2, s3, s4, s00, s10, s01, s11)
c      do k = kstam , kendm
c      do j = jsta1n, jend1n
      do i = ista1n, iend1n
#ifdef _IMPLICIT_VIS_
!
#ifdef _UNROLL_FDM_
         s1 =  d1n(ms  )*u (i,j+ms  ,k)
     $        +d1n(ms+1)*u (i,j+ms+1,k)
     $        +d1n(ms+2)*u (i,j+ms+2,k)
     $        +d1n(ms+3)*u (i,j+ms+3,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*u (i,j+ms+4,k)
     $        +d1n(ms+5)*u (i,j+ms+5,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*u (i,j+ms+6,k)
     $        +d1n(ms+7)*u (i,j+ms+7,k)
#endif
     $        +d1n(ms  )*un(i,j+ms  ,k)
     $        +d1n(ms+1)*un(i,j+ms+1,k)
     $        +d1n(ms+2)*un(i,j+ms+2,k)
     $        +d1n(ms+3)*un(i,j+ms+3,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*un(i,j+ms+4,k)
     $        +d1n(ms+5)*un(i,j+ms+5,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*un(i,j+ms+6,k)
     $        +d1n(ms+7)*un(i,j+ms+7,k)
#endif
         s2 =  d1n(ms  )*v (i+ms  ,j,k)
     $        +d1n(ms+1)*v (i+ms+1,j,k)
     $        +d1n(ms+2)*v (i+ms+2,j,k)
     $        +d1n(ms+3)*v (i+ms+3,j,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*v (i+ms+4,j,k)
     $        +d1n(ms+5)*v (i+ms+5,j,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*v (i+ms+6,j,k)
     $        +d1n(ms+7)*v (i+ms+7,j,k)
#endif
     $        +d1n(ms  )*vn(i+ms  ,j,k)
     $        +d1n(ms+1)*vn(i+ms+1,j,k)
     $        +d1n(ms+2)*vn(i+ms+2,j,k)
     $        +d1n(ms+3)*vn(i+ms+3,j,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*vn(i+ms+4,j,k)
     $        +d1n(ms+5)*vn(i+ms+5,j,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*vn(i+ms+6,j,k)
     $        +d1n(ms+7)*vn(i+ms+7,j,k)
#endif
#else /* _UNROLL_FDM_ */
         s1 = 0.d0
         s2 = 0.d0
         do m = ms, me
            s1 = s1 + d1n(m)*u(i,j+m,k) + d1n(m)*un(i,j+m,k)
            s2 = s2 + d1n(m)*v(i+m,j,k) + d1n(m)*vn(i+m,j,k)
         enddo
#endif /* _UNROLL_FDM_ */
!
         s1 = .5d0 * s1
         s2 = .5d0 * s2
!
#else /* _IMPLICIT_VIS_ */
!
#ifdef _UNROLL_FDM_
         s1 =  d1n(ms  )*u(i,j+ms  ,k)
     $        +d1n(ms+1)*u(i,j+ms+1,k)
     $        +d1n(ms+2)*u(i,j+ms+2,k)
     $        +d1n(ms+3)*u(i,j+ms+3,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*u(i,j+ms+4,k)
     $        +d1n(ms+5)*u(i,j+ms+5,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*u(i,j+ms+6,k)
     $        +d1n(ms+7)*u(i,j+ms+7,k)
#endif
         s2 =  d1n(ms  )*v(i+ms  ,j,k)
     $        +d1n(ms+1)*v(i+ms+1,j,k)
     $        +d1n(ms+2)*v(i+ms+2,j,k)
     $        +d1n(ms+3)*v(i+ms+3,j,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*v(i+ms+4,j,k)
     $        +d1n(ms+5)*v(i+ms+5,j,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*v(i+ms+6,j,k)
     $        +d1n(ms+7)*v(i+ms+7,j,k)
#endif
#else /* _UNROLL_FDM_ */
         s1 = 0.d0
         s2 = 0.d0
         do m = ms, me
            s1 = s1 + d1n(m)*u(i,j+m,k)
            s2 = s2 + d1n(m)*v(i+m,j,k)
         enddo
#endif /* _UNROLL_FDM_ */
!
#endif /* _IMPLICIT_VIS_ */
!
#ifdef _ARRAY_CONST_
         s00 = Mum(i  ,j  ,k  )
         s10 = Mum(i+1,j  ,k  )
         s01 = Mum(i  ,j+1,k  )
         s11 = Mum(i+1,j+1,k  )
#else
         s00 = Mu( mask(i  ,j  ,k  ) )
         s10 = Mu( mask(i+1,j  ,k  ) )
         s01 = Mu( mask(i  ,j+1,k  ) )
         s11 = Mu( mask(i+1,j+1,k  ) )
#endif /* _ARRAY_CONST_ */
         s0 =   s00 * s10 * s01
     $        + s00 * s10       * s11
     $        + s00       * s01 * s11
     $        +       s10 * s01 * s11
         if ( s0 .ne. 0.d0 ) then
            s0 = 4.d0 * s00 * s10 * s01 * s11 / s0
         else
            s0 = 0.d0
         endif
!
         s3 = .5d0 * ( s1 + s2 )
         exy(i,j,k) = s3 * 2.d0 * s0
!
         s4 = s3*s3*.25d0*2.d0*s0
         w2(i,j,k) = s4
      enddo
c      enddo
c      enddo

!TUNE16**************************************************************>>>
!     do k = kstam, kendm
!     do j = jstam, jendm
!     do i = istam, iendm
!!! Caution !!!
!        w1(i,j,k) = w1(i,j,k)
!    $               + w2(i-1,j-1,k  )
!    $               + w2(i  ,j-1,k  )
!    $               + w2(i-1,j  ,k  )
!    $               + w2(i  ,j  ,k  )
!     enddo
!     enddo
!     enddo
!TUNE16END<<<***********************************************************
! --- Exy
!
! +++ Eyz
c!ocl temp(s0, s1, s2, s3, s4, s00, s10, s01, s11)
c      do k = ksta1n, kend1n
c      do j = jsta1n, jend1n
      do i = istam , iendm
#ifdef _IMPLICIT_VIS_
!
#ifdef _UNROLL_FDM_
         s1 =  d1n(ms  )*v (i,j,k+ms  )
     $        +d1n(ms+1)*v (i,j,k+ms+1)
     $        +d1n(ms+2)*v (i,j,k+ms+2)
     $        +d1n(ms+3)*v (i,j,k+ms+3)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*v (i,j,k+ms+4)
     $        +d1n(ms+5)*v (i,j,k+ms+5)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*v (i,j,k+ms+6)
     $        +d1n(ms+7)*v (i,j,k+ms+7)
#endif
     $        +d1n(ms  )*vn(i,j,k+ms  )
     $        +d1n(ms+1)*vn(i,j,k+ms+1)
     $        +d1n(ms+2)*vn(i,j,k+ms+2)
     $        +d1n(ms+3)*vn(i,j,k+ms+3)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*vn(i,j,k+ms+4)
     $        +d1n(ms+5)*vn(i,j,k+ms+5)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*vn(i,j,k+ms+6)
     $        +d1n(ms+7)*vn(i,j,k+ms+7)
#endif
         s2 =  d1n(ms  )*w (i,j+ms  ,k)
     $        +d1n(ms+1)*w (i,j+ms+1,k)
     $        +d1n(ms+2)*w (i,j+ms+2,k)
     $        +d1n(ms+3)*w (i,j+ms+3,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*w (i,j+ms+4,k)
     $        +d1n(ms+5)*w (i,j+ms+5,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*w (i,j+ms+6,k)
     $        +d1n(ms+7)*w (i,j+ms+7,k)
#endif
     $        +d1n(ms  )*wn(i,j+ms  ,k)
     $        +d1n(ms+1)*wn(i,j+ms+1,k)
     $        +d1n(ms+2)*wn(i,j+ms+2,k)
     $        +d1n(ms+3)*wn(i,j+ms+3,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*wn(i,j+ms+4,k)
     $        +d1n(ms+5)*wn(i,j+ms+5,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*wn(i,j+ms+6,k)
     $        +d1n(ms+7)*wn(i,j+ms+7,k)
#endif
#else /* _UNROLL_FDM_ */
         s1 = 0.d0
         s2 = 0.d0
         do m = ms, me
            s1 = s1 + d1n(m)*v(i,j,k+m) + d1n(m)*vn(i,j,k+m)
            s2 = s2 + d1n(m)*w(i,j+m,k) + d1n(m)*wn(i,j+m,k)
         enddo
#endif /* _UNROLL_FDM_ */
!
         s1 = .5d0 * s1
         s2 = .5d0 * s2
!
#else /* _IMPLICIT_VIS_ */
!
#ifdef _UNROLL_FDM_
         s1 =  d1n(ms  )*v(i,j,k+ms  )
     $        +d1n(ms+1)*v(i,j,k+ms+1)
     $        +d1n(ms+2)*v(i,j,k+ms+2)
     $        +d1n(ms+3)*v(i,j,k+ms+3)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*v(i,j,k+ms+4)
     $        +d1n(ms+5)*v(i,j,k+ms+5)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*v(i,j,k+ms+6)
     $        +d1n(ms+7)*v(i,j,k+ms+7)
#endif
         s2 =  d1n(ms  )*w(i,j+ms  ,k)
     $        +d1n(ms+1)*w(i,j+ms+1,k)
     $        +d1n(ms+2)*w(i,j+ms+2,k)
     $        +d1n(ms+3)*w(i,j+ms+3,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*w(i,j+ms+4,k)
     $        +d1n(ms+5)*w(i,j+ms+5,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*w(i,j+ms+6,k)
     $        +d1n(ms+7)*w(i,j+ms+7,k)
#endif
#else /* _UNROLL_FDM_ */
         s1 = 0.d0
         s2 = 0.d0
         do m = ms, me
            s1 = s1 + d1n(m)*v(i,j,k+m)
            s2 = s2 + d1n(m)*w(i,j+m,k)
         enddo
#endif /* _UNROLL_FDM_ */
!
#endif /* _IMPLICIT_VIS_ */
!
#ifdef _ARRAY_CONST_
         s00 = Mum(i  ,j  ,k  )
         s10 = Mum(i  ,j+1,k  )
         s01 = Mum(i  ,j  ,k+1)
         s11 = Mum(i  ,j+1,k+1)
#else
         s00 = Mu( mask(i  ,j  ,k  ) )
         s10 = Mu( mask(i  ,j+1,k  ) )
         s01 = Mu( mask(i  ,j  ,k+1) )
         s11 = Mu( mask(i  ,j+1,k+1) )
#endif /* _ARRAY_CONST_ */
         s0 =   s00 * s10 * s01
     $        + s00 * s10       * s11
     $        + s00       * s01 * s11
     $        +       s10 * s01 * s11
         if ( s0 .ne. 0.d0 ) then
            s0 = 4.d0 * s00 * s10 * s01 * s11 / s0
         else
            s0 = 0.d0
         endif
!
         s3 = .5d0 * ( s1 + s2 )
         eyz(i,j,k) = s3 * 2.d0 * s0
!
         s4 = s3*s3*.25d0*2.d0*s0
!TUNE16**************************************************************>>>
!        w2(i,j,k) = s4
!TUNE16-----------------------------------------------------------------
         w3(i,j,k) = s4
!TUNE16END<<<***********************************************************
      enddo
c      enddo
c      enddo

!TUNE16**************************************************************>>>
!     do k = kstam, kendm
!     do j = jstam, jendm
!     do i = istam, iendm
!!! Caution !!!
!        w1(i,j,k) = w1(i,j,k)
!    $               + w2(i  ,j-1,k-1)
!    $               + w2(i  ,j  ,k-1)
!    $               + w2(i  ,j-1,k  )
!    $               + w2(i  ,j  ,k  )
!     enddo
!     enddo
!     enddo
!TUNE16END<<<***********************************************************
! --- Eyz
!
! +++ Ezx
c!ocl temp(s0, s1, s2, s3, s4, s00, s10, s01, s11)
c      do k = ksta1n, kend1n
c      do j = jstam , jendm
      do i = ista1n, iend1n
#ifdef _IMPLICIT_VIS_
!
#ifdef _UNROLL_FDM_
         s1 =  d1n(ms  )*w (i+ms  ,j,k)
     $        +d1n(ms+1)*w (i+ms+1,j,k)
     $        +d1n(ms+2)*w (i+ms+2,j,k)
     $        +d1n(ms+3)*w (i+ms+3,j,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*w (i+ms+4,j,k)
     $        +d1n(ms+5)*w (i+ms+5,j,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*w (i+ms+6,j,k)
     $        +d1n(ms+7)*w (i+ms+7,j,k)
#endif
     $        +d1n(ms  )*wn(i+ms  ,j,k)
     $        +d1n(ms+1)*wn(i+ms+1,j,k)
     $        +d1n(ms+2)*wn(i+ms+2,j,k)
     $        +d1n(ms+3)*wn(i+ms+3,j,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*wn(i+ms+4,j,k)
     $        +d1n(ms+5)*wn(i+ms+5,j,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*wn(i+ms+6,j,k)
     $        +d1n(ms+7)*wn(i+ms+7,j,k)
#endif
         s2 =  d1n(ms  )*u (i,j,k+ms  )
     $        +d1n(ms+1)*u (i,j,k+ms+1)
     $        +d1n(ms+2)*u (i,j,k+ms+2)
     $        +d1n(ms+3)*u (i,j,k+ms+3)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*u (i,j,k+ms+4)
     $        +d1n(ms+5)*u (i,j,k+ms+5)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*u (i,j,k+ms+6)
     $        +d1n(ms+7)*u (i,j,k+ms+7)
#endif
     $        +d1n(ms  )*un(i,j,k+ms  )
     $        +d1n(ms+1)*un(i,j,k+ms+1)
     $        +d1n(ms+2)*un(i,j,k+ms+2)
     $        +d1n(ms+3)*un(i,j,k+ms+3)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*un(i,j,k+ms+4)
     $        +d1n(ms+5)*un(i,j,k+ms+5)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*un(i,j,k+ms+6)
     $        +d1n(ms+7)*un(i,j,k+ms+7)
#endif
#else /* _UNROLL_FDM_ */
         s1 = 0.d0
         s2 = 0.d0
         do m = ms, me
            s1 = s1 + d1n(m)*w(i+m,j,k) + d1n(m)*w(i+m,j,k)
            s2 = s2 + d1n(m)*u(i,j,k+m) + d1n(m)*u(i,j,k+m)
         enddo
#endif /* _UNROLL_FDM_ */
!
         s1 = .5d0 * s1
         s2 = .5d0 * s2
!
#else /* _IMPLICIT_VIS_ */
!
#ifdef _UNROLL_FDM_
         s1 =  d1n(ms  )*w(i+ms  ,j,k)
     $        +d1n(ms+1)*w(i+ms+1,j,k)
     $        +d1n(ms+2)*w(i+ms+2,j,k)
     $        +d1n(ms+3)*w(i+ms+3,j,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*w(i+ms+4,j,k)
     $        +d1n(ms+5)*w(i+ms+5,j,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*w(i+ms+6,j,k)
     $        +d1n(ms+7)*w(i+ms+7,j,k)
#endif
         s2 =  d1n(ms  )*u(i,j,k+ms  )
     $        +d1n(ms+1)*u(i,j,k+ms+1)
     $        +d1n(ms+2)*u(i,j,k+ms+2)
     $        +d1n(ms+3)*u(i,j,k+ms+3)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*u(i,j,k+ms+4)
     $        +d1n(ms+5)*u(i,j,k+ms+5)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*u(i,j,k+ms+6)
     $        +d1n(ms+7)*u(i,j,k+ms+7)
#endif
#else /* _UNROLL_FDM_ */
         s1 = 0.d0
         s2 = 0.d0
         do m = ms, me
            s1 = s1 + d1n(m)*w(i+m,j,k)
            s2 = s2 + d1n(m)*u(i,j,k+m)
         enddo
#endif /* _UNROLL_FDM_ */
!
#endif /* _IMPLICIT_VIS_ */
!
#ifdef _ARRAY_CONST_
         s00 = Mum(i+1,j  ,k  )
         s10 = Mum(i  ,j  ,k  )
         s01 = Mum(i+1,j  ,k+1)
         s11 = Mum(i  ,j  ,k+1)
#else
         s00 = Mu( mask(i+1,j  ,k  ) )
         s10 = Mu( mask(i  ,j  ,k  ) )
         s01 = Mu( mask(i+1,j  ,k+1) )
         s11 = Mu( mask(i  ,j  ,k+1) )
#endif /* _ARRAY_CONST_ */
         s0 =   s00 * s10 * s01
     $        + s00 * s10       * s11
     $        + s00       * s01 * s11
     $        +       s10 * s01 * s11
         if ( s0 .ne. 0.d0 ) then
            s0 = 4.d0 * s00 * s10 * s01 * s11 / s0
         else
            s0 = 0.d0
         endif
!
         s3 = .5d0 * ( s1 + s2 )
         ezx(i,j,k) = s3 * 2.d0 * s0
!
         s4 = s3*s3*.25d0*2.d0*s0
!TUNE16**************************************************************>>>
!        w2(i,j,k) = s4
!TUNE16-----------------------------------------------------------------
         w4(i,j,k) = s4
!TUNE16END<<<***********************************************************
      enddo
      enddo
      enddo
      call STOP_COLLECTION("vis_loop1")

      call START_COLLECTION("vis_loop2")
      do k = kstam, kendm
      do j = jstam, jendm
      do i = istam, iendm
!!! Caution !!!
         w1(i,j,k) = w1(i,j,k)
!TUNE16**************************************************************>>>
!    $               + w2(i-1,j  ,k-1)
!    $               + w2(i  ,j  ,k-1)
!    $               + w2(i-1,j  ,k  )
!    $               + w2(i  ,j  ,k  )
!TUNE16-----------------------------------------------------------------
     $               + w2(i-1,j-1,k  )
     $               + w2(i  ,j-1,k  )
     $               + w2(i-1,j  ,k  )
     $               + w2(i  ,j  ,k  )
!TUNE16-----------------------------------------------------------------
     $               + w3(i  ,j-1,k-1)
     $               + w3(i  ,j  ,k-1)
     $               + w3(i  ,j-1,k  )
     $               + w3(i  ,j  ,k  )
!TUNE16-----------------------------------------------------------------
     $               + w4(i-1,j  ,k-1)
     $               + w4(i  ,j  ,k-1)
     $               + w4(i-1,j  ,k  )
     $               + w4(i  ,j  ,k  )
!TUNE16END<<<***********************************************************
      enddo
      enddo
      enddo
! --- Ezx
      call STOP_COLLECTION("vis_loop2")
!
      return
      end subroutine HIFU3D_Cal_Viscous
!
! ----------------------------------------------------------------------
      subroutine HIFU3D_Cal_U
! ----------------------------------------------------------------------
      use FDM
      use Array_Index
      use Physical_Properties
      use PML , only : sigma_nx
!
      implicit none
!
      integer*4 :: i, j, k
      integer*4 :: ID
      real*8 :: s0, s1, s2, s3, s4, s5, s6, s7
! +++ U
      s0 = .5d0 * dt
!ocl temp(s1, s2, s3, s4, s5, s6, s7)
      do k = kstam , kendm
      do j = jstam , jendm
      do i = istan , iendn
#ifdef _UNROLL_FDM_
         s1 =  d0n(ms  )*rhom(i+ms  ,j,k)
     $        +d0n(ms+1)*rhom(i+ms+1,j,k)
     $        +d0n(ms+2)*rhom(i+ms+2,j,k)
     $        +d0n(ms+3)*rhom(i+ms+3,j,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d0n(ms+4)*rhom(i+ms+4,j,k)
     $        +d0n(ms+5)*rhom(i+ms+5,j,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d0n(ms+6)*rhom(i+ms+6,j,k)
     $        +d0n(ms+7)*rhom(i+ms+7,j,k)
#endif
!TUNE20**************************************************************>>>
!        s2 =  d1n(ms  )*p0(i+ms  ,j,k)
!    $        +d1n(ms+1)*p0(i+ms+1,j,k)
!    $        +d1n(ms+2)*p0(i+ms+2,j,k)
!    $        +d1n(ms+3)*p0(i+ms+3,j,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
!    $        +d1n(ms+4)*p0(i+ms+4,j,k)
!    $        +d1n(ms+5)*p0(i+ms+5,j,k)
#endif
!        s3 =  d1n(ms  )*ekk(i+ms  ,j,k)
!    $        +d1n(ms+1)*ekk(i+ms+1,j,k)
!    $        +d1n(ms+2)*ekk(i+ms+2,j,k)
!    $        +d1n(ms+3)*ekk(i+ms+3,j,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
!    $        +d1n(ms+4)*ekk(i+ms+4,j,k)
!    $        +d1n(ms+5)*ekk(i+ms+5,j,k)
#endif
!TUNE20END<<<***********************************************************
         s4 =  d1n(ms  )*exx(i+ms  ,j,k)
     $        +d1n(ms+1)*exx(i+ms+1,j,k)
     $        +d1n(ms+2)*exx(i+ms+2,j,k)
     $        +d1n(ms+3)*exx(i+ms+3,j,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*exx(i+ms+4,j,k)
     $        +d1n(ms+5)*exx(i+ms+5,j,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*exx(i+ms+6,j,k)
     $        +d1n(ms+7)*exx(i+ms+7,j,k)
#endif
         s5 =  d1m(ns  )*exy(i,j+ns  ,k)
     $        +d1m(ns+1)*exy(i,j+ns+1,k)
     $        +d1m(ns+2)*exy(i,j+ns+2,k)
     $        +d1m(ns+3)*exy(i,j+ns+3,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*exy(i,j+ns+4,k)
     $        +d1m(ns+5)*exy(i,j+ns+5,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*exy(i,j+ns+6,k)
     $        +d1m(ns+7)*exy(i,j+ns+7,k)
#endif
         s6  = d1m(ns  )*ezx(i,j,k+ns  )
     $        +d1m(ns+1)*ezx(i,j,k+ns+1)
     $        +d1m(ns+2)*ezx(i,j,k+ns+2)
     $        +d1m(ns+3)*ezx(i,j,k+ns+3)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*ezx(i,j,k+ns+4)
     $        +d1m(ns+5)*ezx(i,j,k+ns+5)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*ezx(i,j,k+ns+6)
     $        +d1m(ns+7)*ezx(i,j,k+ns+7)
#endif
#else /* _UNROLL_FDM_ */
         s1 = 0.d0
!TUNE20**************************************************************>>>
!        s2 = 0.d0
!        s3 = 0.d0
!TUNE20END<<<***********************************************************
         s4 = 0.d0
         do m = ms, me
            s1 = s1 + d0n(m)*rhom(i+m,j,k)
!TUNE20**************************************************************>>>
!           s2 = s2 + d1n(m)*p0  (i+m,j,k)
!           s3 = s3 + d1n(m)*ekk (i+m,j,k)
!TUNE20END<<<***********************************************************
            s4 = s4 + d1n(m)*exx (i+m,j,k)
         enddo
         s5 = 0.d0
         s6 = 0.d0
         do n = ns, ne
            s5 = s5 + d1m(n)*exy(i,j+n,k)
            s6 = s6 + d1m(n)*ezx(i,j,k+n)
         enddo
#endif /* _UNROLL_FDM_ */
! rhom * ( dudt + sigma * u ) = - dpdx + viscous
         s7 =  .5d0*csm2(i  ,j,k)
     $        +.5d0*csm2(i+1,j,k)
         s7 = s0 * DSQRT(s7) * sigma_nx(i)
#ifdef _IMPLICIT_VIS_
         u(i,j,k) =( un(i,j,k) * ( 1.d0 - s7 ) 
!TUNE20**************************************************************>>>
!    $              + dt * ( - s2 + s3 + s4 + s5 + s6) / s1
!TUNE20-----------------------------------------------------------------
     $              + dt * ( s4 + s5 + s6) / s1
!TUNE20END<<<***********************************************************
     $             )           / ( 1.d0 + s7 )
#else /* _IMPLICIT_VIS_ */
         u(i,j,k) =( u (i,j,k) * ( 1.d0 - s7 ) 
!TUNE20**************************************************************>>>
!    $              + dt * ( - s2 + s3 + s4 + s5 + s6) / s1
!TUNE20-----------------------------------------------------------------
     $              + dt * ( s4 + s5 + s6 ) / s1
!TUNE20END<<<***********************************************************
     $             )           / ( 1.d0 + s7 )
#endif /* _IMPLICIT_VIS_ */
      enddo
      enddo
      enddo
!
      return
      end subroutine HIFU3D_Cal_U

! ----------------------------------------------------------------------
      subroutine HIFU3D_Update_Boundary_U
! ----------------------------------------------------------------------
      use Array_Index
!
      implicit none
!
      integer*4 :: i, j, k
!
! +++ +++ B.C. Mirror
#if defined( _MIRROR_X0_ )
      if ( istan ==  ista_G - 1 ) then
         do k = kstam , kendm
         do j = jstam , jendm
         do i = ista2n, istan-1 ! 0+ms+ns, -1
            u(i,j,k) = - u(istan-i,j,k)
         enddo
            i = istan
            u(i,j,k) = 0.d0
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_X1_ )
      if ( iendn == iend_G ) then
         do k = kstam  , kendm
         do j = jstam  , jendm
            i = iendn ! nx
            u(i,j,k) = 0.d0
         do i = iendn+1, iend2n ! nx+1, nx+me+ne
            u(i,j,k) = - u(iendn-(i-iendn),j,k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_Y0_ )
      if ( jstam ==  jsta_G ) then
         do k = kstam , kendm
         do j = jsta2m, jstam-1 ! 1+ns+ms, 0
         do i = istan , iendn
            u(i,j,k) = u(i,jstam-j,k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_Y1_ )
      if ( jendm == jend_G ) then
         do k = kstam  , kendm
         do j = jendm+1, jend2m ! ny+1, ny+ne+me
         do i = istan  , iendn
            u(i,j,k) = u(i,jendm-(j-jendm-1),k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_Z0_ )
      if ( kstam ==  ksta_G ) then
         do k = ksta2m, kstam-1 ! 1+ns+ms, 0
         do j = jstam , jendm
         do i = istan , iendn
            u(i,j,k) = u(i,j,kstam-k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_Z1_ )
      if ( kendm == kend_G ) then
         do k = kendm+1, kend2m ! nz+1, nz+ne+me
         do j = jstam  , jendm
         do i = istan  , iendn
            u(i,j,k) = u(i,j,kendm-(k-kendm-1))
         enddo
         enddo
         enddo
      endif
#endif
! --- --- B.C. Mirror
!
      return
      end subroutine HIFU3D_Update_Boundary_U

! ----------------------------------------------------------------------
      subroutine HIFU3D_Cal_V
! ----------------------------------------------------------------------
      use FDM
      use Array_Index
      use Physical_Properties
      use PML , only : sigma_ny
!
      implicit none
!
      integer*4 :: i, j, k
      integer*4 :: ID
      real*8 :: s0, s1, s2, s3, s4, s5, s6, s7
!
      s0 = .5d0 * dt
!ocl temp(s1, s2, s3, s4, s5, s6, s7)
      do k = kstam , kendm
      do j = jstan , jendn
      do i = istam , iendm
#ifdef _UNROLL_FDM_
         s1 =  d0n(ms  )*rhom(i,j+ms  ,k)
     $        +d0n(ms+1)*rhom(i,j+ms+1,k)
     $        +d0n(ms+2)*rhom(i,j+ms+2,k)
     $        +d0n(ms+3)*rhom(i,j+ms+3,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d0n(ms+4)*rhom(i,j+ms+4,k)
     $        +d0n(ms+5)*rhom(i,j+ms+5,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d0n(ms+6)*rhom(i,j+ms+6,k)
     $        +d0n(ms+7)*rhom(i,j+ms+7,k)
#endif
!TUNE20**************************************************************>>>
!        s2 =  d1n(ms  )*p0(i,j+ms  ,k)
!    $        +d1n(ms+1)*p0(i,j+ms+1,k)
!    $        +d1n(ms+2)*p0(i,j+ms+2,k)
!    $        +d1n(ms+3)*p0(i,j+ms+3,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
!    $        +d1n(ms+4)*p0(i,j+ms+4,k)
!    $        +d1n(ms+5)*p0(i,j+ms+5,k)
#endif
!        s3 =  d1n(ms  )*ekk(i,j+ms  ,k)
!    $        +d1n(ms+1)*ekk(i,j+ms+1,k)
!    $        +d1n(ms+2)*ekk(i,j+ms+2,k)
!    $        +d1n(ms+3)*ekk(i,j+ms+3,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
!    $        +d1n(ms+4)*ekk(i,j+ms+4,k)
!    $        +d1n(ms+5)*ekk(i,j+ms+5,k)
#endif
!TUNE20END<<<***********************************************************
         s4 =  d1n(ms  )*eyy(i,j+ms  ,k)
     $        +d1n(ms+1)*eyy(i,j+ms+1,k)
     $        +d1n(ms+2)*eyy(i,j+ms+2,k)
     $        +d1n(ms+3)*eyy(i,j+ms+3,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*eyy(i,j+ms+4,k)
     $        +d1n(ms+5)*eyy(i,j+ms+5,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*eyy(i,j+ms+6,k)
     $        +d1n(ms+7)*eyy(i,j+ms+7,k)
#endif
         s5 =  d1m(ns  )*exy(i+ns  ,j,k)
     $        +d1m(ns+1)*exy(i+ns+1,j,k)
     $        +d1m(ns+2)*exy(i+ns+2,j,k)
     $        +d1m(ns+3)*exy(i+ns+3,j,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*exy(i+ns+4,j,k)
     $        +d1m(ns+5)*exy(i+ns+5,j,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*exy(i+ns+6,j,k)
     $        +d1m(ns+7)*exy(i+ns+7,j,k)
#endif
         s6 =  d1m(ns  )*eyz(i,j,k+ns  )
     $        +d1m(ns+1)*eyz(i,j,k+ns+1)
     $        +d1m(ns+2)*eyz(i,j,k+ns+2)
     $        +d1m(ns+3)*eyz(i,j,k+ns+3)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*eyz(i,j,k+ns+4)
     $        +d1m(ns+5)*eyz(i,j,k+ns+5)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*eyz(i,j,k+ns+6)
     $        +d1m(ns+7)*eyz(i,j,k+ns+7)
#endif
#else /* _UNROLL_FDM_ */
         s1 = 0.d0
!TUNE20**************************************************************>>>
!        s2 = 0.d0
!        s3 = 0.d0
!TUNE20END<<<***********************************************************
         s4 = 0.d0
         do m = ms, me
            s1 = s1 + d0n(m)*rhom(i,j+m,k)
!TUNE20**************************************************************>>>
!           s2 = s2 + d1n(m)*p0  (i,j+m,k)
!           s3 = s3 + d1n(m)*ekk (i,j+m,k)
!TUNE20END<<<***********************************************************
            s4 = s4 + d1n(m)*eyy (i,j+m,k)
         enddo
         s5 = 0.d0
         s6 = 0.d0
         do n = ns, ne
            s5 = s5 + d1m(n)*exy(i+n,j,k)
            s6 = s6 + d1m(n)*eyz(i,j,k+n)
         enddo
#endif /* _UNROLL_FDM_ */
! rhom * ( dvdt + sigma * v ) = - dpdy + viscous
         s7 =  .5d0*csm2(i,j  ,k)
     $        +.5d0*csm2(i,j+1,k)
         s7 = s0 * DSQRT(s7) * sigma_ny(j)
#ifdef _IMPLICIT_VIS_
         v(i,j,k) = ( vn(i,j,k) * ( 1.d0 - s7 ) 
!TUNE20**************************************************************>>>
!    $               + dt * ( - s2 + s3 + s4 + s5 + s6) / s1
!TUNE20-----------------------------------------------------------------
     $               + dt * ( s4 + s5 + s6) / s1
!TUNE20END<<<***********************************************************
     $              )           / ( 1.d0 + s7 )
#else /* _IMPLICIT_VIS_ */
         v(i,j,k) = ( v (i,j,k) * ( 1.d0 - s7 ) 
!TUNE20**************************************************************>>>
!    $               + dt * ( - s2 + s3 + s4 + s5 + s6) / s1
!TUNE20-----------------------------------------------------------------
     $               + dt * ( s4 + s5 + s6) / s1
!TUNE20END<<<***********************************************************
     $              )           / ( 1.d0 + s7 )
#endif /* _IMPLICIT_VIS_ */
      enddo
      enddo
      enddo
!
      return
      end subroutine HIFU3D_Cal_V

! ----------------------------------------------------------------------
      subroutine HIFU3D_Update_Boundary_V
! ----------------------------------------------------------------------
      use Array_Index
!
      implicit none
!
      integer*4 :: i, j, k
!
! +++ +++ B.C. Mirror
#if defined( _MIRROR_X0_ )
      if ( istam ==  ista_G ) then
         do k = kstam , kendm
         do j = jstan , jendn
         do i = ista2m, istam-1 ! 1+ns+ms, 0
            v(i,j,k) = v(istam-i,j,k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_X1_ )
      if ( iendm == iend_G ) then
         do k = kstam  , kendm
         do j = jstan  , jendn
         do i = iendm+1, iend2m ! nx+1, nx+ne+me
            v(i,j,k) = v(iendm-(i-iendm-1),j,k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_Y0_ )
      if ( jstan ==  jsta_G - 1 ) then
         do k = kstam , kendm
         do j = jsta2n, jstan-1 ! 0+ms+ns, -1
         do i = istam , iendm
            v(i,j,k) = - v(i,jstan-j,k)
         enddo
         enddo
            j = jstan
         do i = istam , iendm
            v(i,j,k) = 0.d0
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_Y1_ )
      if ( jendn == jend_G ) then
         do k = kstam  , kendm
            j = jendn
         do i = istam  , iendm
            v(i,j,k) = 0.d0
         enddo
         do j = jendn+1, jend2n ! ny+1, ny+ne+me
         do i = istam  , iendm
            v(i,j,k) = - v(i, jendn-(j-jendn),k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_Z0_ )
      if ( kstam ==  ksta_G ) then
         do k = ksta2m, kstam-1 ! 1+ns+ms, 0
         do j = jstan , jendn
         do i = istam , iendm
            v(i,j,k) = v(i,j,kstam-k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_Z1_ )
      if ( kendm == kend_G ) then
         do k = kendm+1, kend2m ! nz+1, nz+ne+me
         do j = jstan  , jendn
         do i = istam  , iendm
            v(i,j,k) = v(i,j,kendm-(k-kendm-1))
         enddo
         enddo
         enddo
      endif
#endif
! --- --- B.C. Mirror
!
      return
      end subroutine HIFU3D_Update_Boundary_V

! ----------------------------------------------------------------------
      subroutine HIFU3D_Cal_W
! ----------------------------------------------------------------------
      use FDM
      use Array_Index
      use Physical_Properties
      use PML , only : sigma_nz
!
      implicit none
!
      integer*4 :: i, j, k
      integer*4 :: ID
      real*8 :: s0, s1, s2, s3, s4, s5, s6, s7
!
      s0 = .5d0 * dt
!ocl temp(s1, s2, s3, s4, s5, s6, s7)
      do k = kstan , kendn
      do j = jstam , jendm
      do i = istam , iendm
#ifdef _UNROLL_FDM_
         s1 =  d0n(ms  )*rhom(i,j,k+ms  )
     $        +d0n(ms+1)*rhom(i,j,k+ms+1)
     $        +d0n(ms+2)*rhom(i,j,k+ms+2)
     $        +d0n(ms+3)*rhom(i,j,k+ms+3)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d0n(ms+4)*rhom(i,j,k+ms+4)
     $        +d0n(ms+5)*rhom(i,j,k+ms+5)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d0n(ms+6)*rhom(i,j,k+ms+6)
     $        +d0n(ms+7)*rhom(i,j,k+ms+7)
#endif
!TUNE20**************************************************************>>>
!        s2 =  d1n(ms  )*p0(i,j,k+ms  )
!    $        +d1n(ms+1)*p0(i,j,k+ms+1)
!    $        +d1n(ms+2)*p0(i,j,k+ms+2)
!    $        +d1n(ms+3)*p0(i,j,k+ms+3)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
!    $        +d1n(ms+4)*p0(i,j,k+ms+4)
!    $        +d1n(ms+5)*p0(i,j,k+ms+5)
#endif
!        s3 =  d1n(ms  )*ekk(i,j,k+ms  )
!    $        +d1n(ms+1)*ekk(i,j,k+ms+1)
!    $        +d1n(ms+2)*ekk(i,j,k+ms+2)
!    $        +d1n(ms+3)*ekk(i,j,k+ms+3)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
!    $        +d1n(ms+4)*ekk(i,j,k+ms+4)
!    $        +d1n(ms+5)*ekk(i,j,k+ms+5)
#endif
!TUNE20END<<<***********************************************************
         s4 =  d1n(ms  )*ezz(i,j,k+ms  )
     $        +d1n(ms+1)*ezz(i,j,k+ms+1)
     $        +d1n(ms+2)*ezz(i,j,k+ms+2)
     $        +d1n(ms+3)*ezz(i,j,k+ms+3)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*ezz(i,j,k+ms+4)
     $        +d1n(ms+5)*ezz(i,j,k+ms+5)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*ezz(i,j,k+ms+6)
     $        +d1n(ms+7)*ezz(i,j,k+ms+7)
#endif
         s5 =  d1m(ns  )*ezx(i+ns  ,j,k)
     $        +d1m(ns+1)*ezx(i+ns+1,j,k)
     $        +d1m(ns+2)*ezx(i+ns+2,j,k)
     $        +d1m(ns+3)*ezx(i+ns+3,j,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*ezx(i+ns+4,j,k)
     $        +d1m(ns+5)*ezx(i+ns+5,j,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*ezx(i+ns+6,j,k)
     $        +d1m(ns+7)*ezx(i+ns+7,j,k)
#endif
         s6 =  d1m(ns  )*eyz(i,j+ns  ,k)
     $        +d1m(ns+1)*eyz(i,j+ns+1,k)
     $        +d1m(ns+2)*eyz(i,j+ns+2,k)
     $        +d1m(ns+3)*eyz(i,j+ns+3,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*eyz(i,j+ns+4,k)
     $        +d1m(ns+5)*eyz(i,j+ns+5,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*eyz(i,j+ns+6,k)
     $        +d1m(ns+7)*eyz(i,j+ns+7,k)
#endif
#else /* _UNROLL_FDM_ */
         s1 = 0.d0
!TUNE20**************************************************************>>>
!        s2 = 0.d0
!        s3 = 0.d0
!TUNE20END<<<***********************************************************
         s4 = 0.d0
         do m = ms, me
            s1 = s1 + d0n(m)*rhom(i,j,k+m)
!TUNE20**************************************************************>>>
!           s2 = s2 + d1n(m)*p0  (i,j,k+m)
!           s3 = s3 + d1n(m)*ekk (i,j,k+m)
!TUNE20END<<<***********************************************************
            s4 = s4 + d1n(m)*ezz (i,j,k+m)
         enddo
         s5 = 0.d0
         s6 = 0.d0
         do n = ns, ne
            s5 = s5 + d1m(n)*ezx(i+n,j,k)
            s6 = s6 + d1m(n)*eyz(i,j+n,k)
         enddo
#endif /* _UNROLL_FDM_ */
! rhom * ( dvdt + sigma * v ) = - dpdy + viscous
         s7 =  .5d0*csm2(i,j,k  )
     $        +.5d0*csm2(i,j,k+1)
         s7 = s0 * DSQRT(s7) * sigma_nz(k)
#ifdef _IMPLICIT_VIS_
         w(i,j,k) = ( wn(i,j,k) * ( 1.d0 - s7 ) 
!TUNE20**************************************************************>>>
!    $               + dt * ( - s2 + s3 + s4 + s5 + s6) / s1
!TUNE20-----------------------------------------------------------------
     $               + dt * ( s4 + s5 + s6 ) / s1
!TUNE20END<<<***********************************************************
     $              )           / ( 1.d0 + s7 )
#else /* _IMPLICIT_VIS_ */
         w(i,j,k) = ( w (i,j,k) * ( 1.d0 - s7 ) 
!TUNE20**************************************************************>>>
!    $               + dt * ( - s2 + s3 + s4 + s5 + s6) / s1
!TUNE20-----------------------------------------------------------------
     $               + dt * ( s4 + s5 + s6 ) / s1
!TUNE20END<<<***********************************************************
     $              )           / ( 1.d0 + s7 )
#endif /* _IMPLICIT_VIS_ */
      enddo
      enddo
      enddo
!
      return
      end subroutine HIFU3D_Cal_W
!
! ----------------------------------------------------------------------
      subroutine HIFU3D_Update_Boundary_W
! ----------------------------------------------------------------------
      use Array_Index
!
      implicit none
!
      integer*4 :: i, j, k
!
! +++ +++ B.C. Mirror
#if defined( _MIRROR_X0_ )
      if ( istam ==  ista_G ) then
         do k = kstan , kendn
         do j = jstam , jendm
         do i = ista2m, istam-1 ! 1+ns+ms, 0
            w(i,j,k) = w(istam-i,j,k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_X1_ )
      if ( iendm == iend_G ) then
         do k = kstan  , kendn
         do j = jstam  , jendm
         do i = iendm+1, iend2m ! nx+1, nx+ne+me
            w(i,j,k) = w(iendm-(i-iendm-1),j,k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_Y0_ )
      if ( jstam ==  jsta_G ) then
         do k = kstan , kendn
         do j = jsta2m, jstam-1 ! 1+ns+ms, 0
         do i = istam , iendm
            w(i,j,k) = w(i,jstam-j,k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_Y1_ )
      if ( jendm == jend_G ) then
         do k = kstan  , kendn
         do j = jendm+1, jend2m ! ny+1, ny+ne+me
         do i = istam  , iendm
            w(i,j,k) = w(i,jendm-(j-jendm-1),k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_Z0_ )
      if ( kstan ==  ksta_G - 1 ) then
         do k = ksta2n, kstan-1 ! 0+ms+ns, -1
         do j = jstam , jendm
         do i = istam , iendm
            w(i,j,k) = - w(i,j,kstan-k)
         enddo
         enddo
         enddo
!
            k = kstan
         do j = jstam , jendm
         do i = istam , iendm
            w(i,j,k) = 0.d0
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_Z1_ )
      if ( kendn == kend_G ) then
            k = kendn
         do j = jstam , jendm
         do i = istam , iendm
            w(i,j,k) = 0.d0
         enddo
         enddo
         do k = kendn+1, kend2n ! nz+1, nz+me+ne
         do j = jstam , jendm
         do i = istam , iendm
            w(i,j,k) = - w(i,j,kendn-(k-kendn))
         enddo
         enddo
         enddo
      endif
#endif
! --- --- B.C. Mirror
!
      return
      end subroutine HIFU3D_Update_Boundary_W

! ----------------------------------------------------------------------
      subroutine HIFU3D_Cal_P
! ----------------------------------------------------------------------
      use FDM
      use Array_Index
      use Physical_Properties
      use PML , only : sigma_mx, sigma_my, sigma_mz
!
      implicit none
!
      integer*4 :: i, j, k
      real*8 :: s0, s1, s2, s3, s4, s5, s6, s7, s8
!
      s0 = .5d0 * dt
!ocl temp(s1, s2, s3, s4, s5, s6, s7, s8)
      do k = kstam , kendm
      do j = jstam , jendm
      do i = istam , iendm
#ifdef _UNROLL_FDM_
         s1 =  d1m(ns  )*u(i+ns  ,j,k)
     $        +d1m(ns+1)*u(i+ns+1,j,k)
     $        +d1m(ns+2)*u(i+ns+2,j,k)
     $        +d1m(ns+3)*u(i+ns+3,j,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*u(i+ns+4,j,k)
     $        +d1m(ns+5)*u(i+ns+5,j,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*u(i+ns+6,j,k)
     $        +d1m(ns+7)*u(i+ns+7,j,k)
#endif
         s2 =  d1m(ns  )*v(i,j+ns  ,k)
     $        +d1m(ns+1)*v(i,j+ns+1,k)
     $        +d1m(ns+2)*v(i,j+ns+2,k)
     $        +d1m(ns+3)*v(i,j+ns+3,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*v(i,j+ns+4,k)
     $        +d1m(ns+5)*v(i,j+ns+5,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*v(i,j+ns+6,k)
     $        +d1m(ns+7)*v(i,j+ns+7,k)
#endif
         s3 =  d1m(ns  )*w(i,j,k+ns  )
     $        +d1m(ns+1)*w(i,j,k+ns+1)
     $        +d1m(ns+2)*w(i,j,k+ns+2)
     $        +d1m(ns+3)*w(i,j,k+ns+3)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*w(i,j,k+ns+4)
     $        +d1m(ns+5)*w(i,j,k+ns+5)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*w(i,j,k+ns+6)
     $        +d1m(ns+7)*w(i,j,k+ns+7)
#endif
#else /* _UNROLL_FDM_ */
         s1 = 0.d0
         s2 = 0.d0
         s3 = 0.d0
         do n = ns, ne
            s1 = s1 + d1m(n)*u(i+n,j,k)
            s2 = s2 + d1m(n)*v(i,j+n,k)
            s3 = s3 + d1m(n)*w(i,j,k+n)
         enddo
#endif /* _UNROLL_FDM_ */
!
         s4 = rhom(i,j,k) * csm2(i,j,k) * dt
!TUNE21**************************************************************>>>
#ifdef _CONST_PRESSURE_AIR_
     $        * DBLE( 1 - AirID( mask(i,j,k) ) )
#endif /* _CONST_PRESSURE_AIR_ */
!TUNE21END<<<***********************************************************
!
         s5 = s0 * DSQRT(csm2(i,j,k))
! dpxdt + sigma_x * px = rhom * csm2 * ( source - dudx )
         s6 = s5 * sigma_mx(i)
         px(i,j,k) = ( px(i,j,k)*(1.d0-s6) + s4*( w1(i,j,k) - s1 ) 
     $               )          /(1.d0+s6)
! dpydt + sigma_y * py = rhom * csm2 * ( source - dvdy )
         s7 = s5 * sigma_my(j)
         py(i,j,k) = ( py(i,j,k)*(1.d0-s7) + s4*( w1(i,j,k) - s2 ) 
     $               )          /(1.d0+s7)
! dpzdt + sigma_z * pz = rhom * csm2 * ( source - dwdz )
         s8 = s5 * sigma_mz(k)
         pz(i,j,k) = ( pz(i,j,k)*(1.d0-s8) + s4*( w1(i,j,k) - s3 ) 
     $               )          /(1.d0+s8)
!TUNE21**************************************************************>>>
!
         p0(i,j,k) = px(i,j,k) + py(i,j,k) + pz(i,j,k)
!
!TUNE21END<<<***********************************************************
      enddo
      enddo
      enddo
!
      return
      end subroutine HIFU3D_Cal_P

! ----------------------------------------------------------------------
      subroutine HIFU3D_Cal_Boundary_P
! ----------------------------------------------------------------------
      use Grid
      use Array_Index
      use Physical_Properties
!
      implicit none
!
      integer*4 :: i, j, k
      real*8 :: s1
!
! +++ +++ B.C. for PML
#if defined( _PML_X0_ )
      if ( istam ==  ista_G ) then
!ocl temp(s1)
         do k = kstam , kendm
         do j = jstam , jendm
         do i = ista1m, istam-1 ! i = -1, 0
            s1 = (-3.d0*p0(i  ,j,k)
     $            +4.d0*p0(i+1,j,k)
     $            -1.d0*p0(i+2,j,k)
     $           )*rdx*.5d0
            p0(i,j,k) = p0(i,j,k) + DSQRT( csm2(i,j,k) ) * dt * s1
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _PML_X1_ )
      if ( iendm == iend_G ) then
!ocl temp(s1)
         do k = kstam , kendm
         do j = jstam , jendm
         do i = iend1m, iendm+1, -1 ! i = nx+2, nx+1, -1
            s1 = ( 1.d0*p0(i-2,j,k)
     $            -4.d0*p0(i-1,j,k)
     $            +3.d0*p0(i  ,j,k)
     $           )*rdx*.5d0
            p0(i,j,k) = p0(i,j,k) - DSQRT( csm2(i,j,k) ) * dt * s1
         enddo
         enddo
         enddo
      endif
#endif
!      
#if defined( _PML_Y0_ )
      if ( jstam ==  jsta_G ) then
!ocl temp(s1)
         do k = kstam , kendm
         do j = jsta1m, jstam-1 ! jsta-2, jsta-1 ! j = -1, 0
         do i = istam , iendm
            s1 = (-3.d0*p0(i,j  ,k)
     $            +4.d0*p0(i,j+1,k)
     $            -1.d0*p0(i,j+2,k)
     $           )*rdy*.5d0
            p0(i,j,k) = p0(i,j,k) + DSQRT( csm2(i,j,k) ) * dt * s1
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _PML_Y1_ )
      if ( jendm == jend_G ) then
!ocl temp(s1)
         do k = kstam , kendm
         do j = jend1m, jendm+1, -1 ! jend+2, jend+1, -1 ! j = ny+2, ny+1, -1
         do i = istam , iendm
            s1 = ( 1.d0*p0(i,j-2,k)
     $            -4.d0*p0(i,j-1,k)
     $            +3.d0*p0(i,j  ,k)
     $           )*rdy*.5d0
            p0(i,j,k) = p0(i,j,k) - DSQRT( csm2(i,j,k) ) * dt * s1
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _PML_Z0_ )
      if ( kstam ==  ksta_G ) then
         do k = ksta1m, kstam-1 ! ksta-2, ksta-1 ! k = -1, 0
!ocl temp(s1)
         do j = jstam , jendm
         do i = istam , iendm
            s1 = (-3.d0*p0(i,j,k  )
     $            +4.d0*p0(i,j,k+1)
     $            -1.d0*p0(i,j,k+2)
     $           )*rdz*.5d0
            p0(i,j,k) = p0(i,j,k) + DSQRT( csm2(i,j,k) ) * dt * s1
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _PML_Z1_ )
      if ( kendm == kend_G ) then
         do k = kend1m, kendm+1, -1 ! kend+2, kend+1, -1 ! k = nz+2, nz+1, -1
!ocl temp(s1)
         do j = jstam , jendm
         do i = istam , iendm
            s1 = ( 1.d0*p0(i,j,k-2)
     $            -4.d0*p0(i,j,k-1)
     $            +3.d0*p0(i,j,k  )
     $           )*rdz*.5d0
            p0(i,j,k) = p0(i,j,k) - DSQRT( csm2(i,j,k) ) * dt * s1
         enddo
         enddo
         enddo
      endif
#endif
! --- --- B.C. for PML
!
      return
      end subroutine HIFU3D_Cal_Boundary_P

! ----------------------------------------------------------------------
      subroutine HIFU3D_Update_Boundary_P
! ----------------------------------------------------------------------
      use Array_Index
!
      implicit none
!
      integer*4 :: i, j, k
!
! +++ +++ B.C. Mirror
#if defined( _MIRROR_X0_ )
      if ( istam == ista_G ) then
         do k = kstam , kendm
         do j = jstam , jendm
         do i = ista1m, istam-1
            p0(i,j,k) = p0(istam-i,j,k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_X1_ )
      if ( iendm == iend_G ) then
         do k = kstam  , kendm
         do j = jstam  , jendm
         do i = iendm+1, iend1m
            p0(i,j,k) = p0(iendm-(i-iendm-1),j,k)
         enddo
         enddo
         enddo
      endif
#endif
!      
#if defined( _MIRROR_Y0_ )
      if ( jstam == jsta_G ) then
         do k = kstam , kendm
         do j = jsta1m, jstam-1
         do i = istam , iendm
            p0(i,j,k) = p0(i,jstam-j,k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_Y1_ )
      if ( jendm == jend_G ) then
         do k = kstam  , kendm
         do j = jendm+1, jend1m
         do i = istam  , iendm
            p0(i,j,k) = p0(i,jendm-(j-jendm-1),k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_Z0_ )
      if ( kstam == ksta_G ) then
         do k = ksta1m, kstam-1
         do j = jstam , jendm
         do i = istam , iendm
            p0(i,j,k) = p0(i,j,kstam-k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_Z1_ )
      if ( kendm == kend_G ) then
         do k = kendm+1, kend1m
         do j = jstam  , jendm
         do i = istam  , iendm
            p0(i,j,k) = p0(i,j,kendm-(k-kendm-1))
         enddo
         enddo
         enddo
      endif
#endif
! --- --- B.C. Mirror
!
      return
      end subroutine HIFU3D_Update_Boundary_P

#ifdef _TEMPERATURE_

! ----------------------------------------------------------------------
      subroutine HIFU3D_Cal_Grad_T
! ----------------------------------------------------------------------
      use Grid
      use FDM
      use Array_Index
      use Physical_Properties
!
      implicit none
!
      integer*4 :: i, j, k
      integer*4 :: ID
      real*8 :: s0, s1, s2, s3, s4
      real*8 :: s10, s100, s110, s20, s200, s210, s30, s300, s310
!
! +++ +++ Tx => Exx, Ty => Eyy, Tz => Ezz
!TUNE22**************************************************************>>>
c!ocl temp(s0, s1, s00, s10)
!     do k = kstam , kendm
!     do j = jstam , jendm
!     do i = ista1n, iend1n
!TUNE22-----------------------------------------------------------------
!ocl temp(s1, s2, s3, s10, s100, s110, s20, s200, s210, s30, s300, s310)
      do k = ksta1n, kend1n
      do j = jsta1n, jend1n
      do i = ista1n, iend1n
!TUNE22END<<<***********************************************************
c      if ( mask(i  ,j,k) == mask(i+1,j,k) ) then
#ifdef _UNROLL_FDM_
         s1 =  d1n(ms  )*T(i+ms  ,j,k)
     $        +d1n(ms+1)*T(i+ms+1,j,k)
     $        +d1n(ms+2)*T(i+ms+2,j,k)
     $        +d1n(ms+3)*T(i+ms+3,j,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*T(i+ms+4,j,k)
     $        +d1n(ms+5)*T(i+ms+5,j,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*T(i+ms+6,j,k)
     $        +d1n(ms+7)*T(i+ms+7,j,k)
#endif
#else /* _UNROLL_FDM_ */
         s1 = 0.d0
         do m = ms, me
            s1 = s1 + d1n(m)*T(i+m,j,k)
         enddo
#endif /* _UNROLL_FDM_ */
         s1 = s1 * mask2(1,i,j,k)
     $       +(-T(i  ,j,k)+T(i+1,j,k))*rdx*(1.d0-mask2(1,i,j,k))
!
#ifdef _ARRAY_CONST_
         s100 = Kappam(i  ,j,k)
         s110 = Kappam(i+1,j,k)
#else
         s100 = Kappa(mask(i  ,j,k))
         s110 = Kappa(mask(i+1,j,k))
#endif
         s10 = s100 + s110
         if ( s10 .ne. 0.d0 ) then
            s10 = 2.d0 * s100 * s110 / s10
         else
            s10 = 0.d0
         endif
!
         exx(i,j,k) = s10 * s1
!TUNE22**************************************************************>>>
!     enddo
!     enddo
!     enddo
!TUNE22END<<<***********************************************************
!
!TUNE22**************************************************************>>>
c!ocl temp(s0, s2, s00, s10)
!     do k = kstam , kendm 
!     do j = jsta1n, jend1n
!     do i = istam , iendm 
!TUNE22END<<<***********************************************************
c      if ( mask(i,j  ,k) == mask(i,j+1,k) ) then
#ifdef _UNROLL_FDM_
         s2 =  d1n(ms  )*T(i,j+ms  ,k)
     $        +d1n(ms+1)*T(i,j+ms+1,k)
     $        +d1n(ms+2)*T(i,j+ms+2,k)
     $        +d1n(ms+3)*T(i,j+ms+3,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*T(i,j+ms+4,k)
     $        +d1n(ms+5)*T(i,j+ms+5,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*T(i,j+ms+6,k)
     $        +d1n(ms+7)*T(i,j+ms+7,k)
#endif
#else /* _UNROLL_FDM_ */
         s2 = 0.d0
         do m = ms, me
            s2 = s2 + d1n(m)*T(i,j+m,k)
         enddo
#endif /* _UNROLL_FDM_ */
         s2 = s2 * mask2(2,i,j,k)
     $       +(-T(i,j  ,k)+T(i,j+1,k))*rdy*(1.d0-mask2(2,i,j,k))
!
#ifdef _ARRAY_CONST_
         s200 = Kappam(i,j  ,k)
         s210 = Kappam(i,j+1,k)
#else
         s200 = Kappa(mask(i,j  ,k))
         s210 = Kappa(mask(i,j+1,k))
#endif
         s20 = s200 + s210
         if ( s20 .ne. 0.d0 ) then
            s20 = 2.d0 * s200 * s210 / s20
         else
            s20 = 0.d0
         endif
!
         eyy(i,j,k) = s20 * s2
!TUNE22**************************************************************>>>
!     enddo
!     enddo
!     enddo
!TUNE22END<<<***********************************************************
!
!TUNE22**************************************************************>>>
c!ocl temp(s0, s3, s00, s10)
!     do k = ksta1n, kend1n
!     do j = jstam , jendm
!     do i = istam , iendm 
!TUNE22END<<<***********************************************************
c      if ( mask(i,j,k  ) == mask(i,j,k+1) ) then
#ifdef _UNROLL_FDM_
         s3 =  d1n(ms  )*T(i,j,k+ms  )
     $        +d1n(ms+1)*T(i,j,k+ms+1)
     $        +d1n(ms+2)*T(i,j,k+ms+2)
     $        +d1n(ms+3)*T(i,j,k+ms+3)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*T(i,j,k+ms+4)
     $        +d1n(ms+5)*T(i,j,k+ms+5)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*T(i,j,k+ms+6)
     $        +d1n(ms+7)*T(i,j,k+ms+7)
#endif
#else /* _UNROLL_FDM_ */
         s3 = 0.d0
         do m = ms, me
            s3 = s3 + d1n(m)*T(i,j,k+m)
         enddo
#endif /* _UNROLL_FDM_ */
         s3 = s3 * mask2(3,i,j,k)
     $       +(-T(i,j,k  )+T(i,j,k+1))*rdz*(1.d0-mask2(3,i,j,k))
!
#ifdef _ARRAY_CONST_
         s300 = Kappam(i,j,k  )
         s310 = Kappam(i,j,k+1)
#else
         s300 = Kappa(mask(i,j,k  ))
         s310 = Kappa(mask(i,j,k+1))
#endif
         s30 = s300 + s310
         if ( s30 .ne. 0.d0 ) then
            s30 = 2.d0 * s300 * s310 / s30
         else
            s30 = 0.d0
         endif
!
         ezz(i,j,k) = s30 * s3
      enddo
      enddo
      enddo
! --- --- Tx => Exx, Ty => Eyy, Tz => Ezz
!
      return
      end subroutine HIFU3D_Cal_Grad_T

! ----------------------------------------------------------------------
      subroutine HIFU3D_Cal_T
! ----------------------------------------------------------------------
      use Grid
      use FDM
      use Array_Index
      use Physical_Properties
!
      implicit none
!
      integer*4 :: i, j, k
      integer*4 :: ID
      real*8 :: s0, s1, s2, s3
!
!ocl temp(ID, s0, s1, s2, s3)
      do k = kstam , kendm
      do j = jstam , jendm
      do i = istam , iendm
!
!TUNE09**************************************************************>>>
!     if ( mask(i-1,j,k) == mask(i  ,j,k) .and.
!    $     mask(i  ,j,k) == mask(i+1,j,k) ) then
!TUNE09END<<<***********************************************************
#ifdef _UNROLL_FDM_
!TUNE09**************************************************************>>>
         s1 =  d1m(ns  )*exx(i+ns  ,j,k)
!TUNE09-----------------------------------------------------------------
!        s1 =( d1m(ns  )*exx(i+ns  ,j,k)
!TUNE09END<<<***********************************************************
     $        +d1m(ns+1)*exx(i+ns+1,j,k)
     $        +d1m(ns+2)*exx(i+ns+2,j,k)
     $        +d1m(ns+3)*exx(i+ns+3,j,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*exx(i+ns+4,j,k)
     $        +d1m(ns+5)*exx(i+ns+5,j,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*exx(i+ns+6,j,k)
     $        +d1m(ns+7)*exx(i+ns+7,j,k)
#endif
!TUNE09**************************************************************>>>
!    $                                   ) * mask1(1,i,j,k)
!TUNE09END<<<***********************************************************
#else /* _UNROLL_FDM_ */
         s1 = 0.d0
         do n = ns, ne
            s1 = s1 + d1m(n)*exx(i+n,j,k)
         enddo
#endif /* _UNROLL_FDM_ */
!TUNE09**************************************************************>>>
!     else
!        s1=(-exx(i-1,j,k)
!    $       +exx(i  ,j,k)
!    $      ) * rdx
!     endif
!TUNE09-----------------------------------------------------------------
c         s1 = s1 * mask1(1,i,j,k)
c     $       +(-exx(i-1,j,k)+exx(i  ,j,k))*rdx*(1.0d0-mask1(1,i,j,k))
!TUNE09END<<<***********************************************************
!
!TUNE09**************************************************************>>>
!     if ( mask(i,j-1,k) == mask(i,j  ,k) .and.
!    $     mask(i,j  ,k) == mask(i,j+1,k) ) then
!TUNE09END<<<***********************************************************
#ifdef _UNROLL_FDM_
!TUNE09**************************************************************>>>
         s2 =  d1m(ns  )*eyy(i,j+ns  ,k)
!TUNE09-----------------------------------------------------------------
!        s2 =( d1m(ns  )*eyy(i,j+ns  ,k)
!TUNE09END<<<***********************************************************
     $        +d1m(ns+1)*eyy(i,j+ns+1,k)
     $        +d1m(ns+2)*eyy(i,j+ns+2,k)
     $        +d1m(ns+3)*eyy(i,j+ns+3,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*eyy(i,j+ns+4,k)
     $        +d1m(ns+5)*eyy(i,j+ns+5,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*eyy(i,j+ns+6,k)
     $        +d1m(ns+7)*eyy(i,j+ns+7,k)
#endif
!TUNE09**************************************************************>>>
!    $                                   ) * mask1(2,i,j,k)
!TUNE09END<<<***********************************************************
#else /* _UNROLL_FDM_ */
         s2 = 0.d0
         do n = ns, ne
            s2 = s2 + d1m(n)*eyy(i,j+n,k)
         enddo
#endif /* _UNROLL_FDM_ */
!TUNE09**************************************************************>>>
!     else
!        s2=(-eyy(i,j-1,k)
!    $       +eyy(i,j  ,k)
!    $      ) * rdy
!     endif
!TUNE09-----------------------------------------------------------------
c         s2 = s2 * mask1(2,i,j,k)
c     $       +(-eyy(i,j-1,k)+eyy(i,j  ,k))*rdy*(1.0d0-mask1(2,i,j,k))
!TUNE09END<<<***********************************************************
!
!TUNE09**************************************************************>>>
!     if ( mask(i,j,k-1) == mask(i,j,k  ) .and.
!    $     mask(i,j,k  ) == mask(i,j,k+1) ) then
!TUNE09END<<<***********************************************************
#ifdef _UNROLL_FDM_
!TUNE09**************************************************************>>>
         s3 =  d1m(ns  )*ezz(i,j,k+ns  )
!TUNE09-----------------------------------------------------------------
!        s3 =( d1m(ns  )*ezz(i,j,k+ns  )
!TUNE09END<<<***********************************************************
     $        +d1m(ns+1)*ezz(i,j,k+ns+1)
     $        +d1m(ns+2)*ezz(i,j,k+ns+2)
     $        +d1m(ns+3)*ezz(i,j,k+ns+3)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*ezz(i,j,k+ns+4)
     $        +d1m(ns+5)*ezz(i,j,k+ns+5)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*ezz(i,j,k+ns+6)
     $        +d1m(ns+7)*ezz(i,j,k+ns+7)
#endif
!TUNE09**************************************************************>>>
!    $                                   ) * mask1(3,i,j,k)
!TUNE09END<<<***********************************************************
#else /* _UNROLL_FDM_ */
         s3 = 0.d0
         do n = ns, ne
            s3 = s3 + d1m(n)*ezz(i,j,k+n)
         enddo
#endif /* _UNROLL_FDM_ */
!TUNE09**************************************************************>>>
!     else
!        s3=(-ezz(i,j,k-1)
!    $       +ezz(i,j,k  )
!    $      ) * rdz
!     endif
!TUNE09-----------------------------------------------------------------
         s1 = s1 * mask1(1,i,j,k)
     $       +(-exx(i-1,j,k)+exx(i  ,j,k))*rdx*(1.0d0-mask1(1,i,j,k))
         s2 = s2 * mask1(2,i,j,k)
     $       +(-eyy(i,j-1,k)+eyy(i,j  ,k))*rdy*(1.0d0-mask1(2,i,j,k))
         s3 = s3 * mask1(3,i,j,k)
     $       +(-ezz(i,j,k-1)+ezz(i,j,k  ))*rdz*(1.0d0-mask1(3,i,j,k))
!TUNE09END<<<***********************************************************
!
#ifdef _ARRAY_CONST_
         s0 = 1.d0 + Betam(i,j,k) * ( T(i,j,k) + T_inf )
     $                    * Betam(i,j,k) * csm2(i,j,k) * rCpm(i,j,k)
         T(i,j,k) = T(i,j,k)
     $             + dtau * s0 * (  w1(i,j,k) + s1 + s2 + s3
     $                           ) / rhom(i,j,k) * rCpm(i,j,k)
#else
         ID = mask(i,j,k)
         s0 = 1.d0 + Beta(ID) * ( T(i,j,k) + T_inf )
     $                    * Beta(ID) * csm2(i,j,k) / Cp(ID)
         T(i,j,k) = T(i,j,k)
     $             + dtau * s0 * (  w1(i,j,k) + s1 + s2 + s3
     $                           ) / ( rhom(i,j,k) * Cp(ID) )
#endif /* _ARRAY_CONST_ */
!
!
!         beta_m(i,j,k) = fL * beta_L + (1.-fL)*beta_G
!         C_pm(i,j,k) = fL * rho_L * C_pL + (1.-fL)*rho_G * C_pG
!     $                / (fL * C_pL + (1.-fL)*C_pG )
      enddo
      enddo
      enddo
!
      return
      end subroutine HIFU3D_Cal_T

! ----------------------------------------------------------------------
      subroutine HIFU3D_Update_Boundary_T
! ----------------------------------------------------------------------
      use Array_Index
!
      implicit none
!
      integer*4 :: i, j, k
!
! +++ +++ B.C. Mirror
#if defined( _MIRROR_X0_ )
      if ( istam == ista_G ) then
         do k = kstam , kendm
         do j = jstam , jendm
         do i = ista2m, istam-1
            T(i,j,k) = T(istam-i,j,k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_X1_ )
      if ( iendm == iend_G ) then
         do k = kstam  , kendm
         do j = jstam  , jendm
         do i = iendm+1, iend2m
            T(i,j,k) = T(iendm-(i-iendm-1),j,k)
         enddo
         enddo
         enddo
      endif
#endif
!      
#if defined( _MIRROR_Y0_ )
      if ( jstam == jsta_G ) then
         do k = kstam , kendm
         do j = jsta2m, jstam-1
         do i = istam , iendm
            T(i,j,k) = T(i,jstam-j,k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_Y1_ )
      if ( jendm == jend_G ) then
         do k = kstam  , kendm
         do j = jendm+1, jend2m
         do i = istam  , iendm
            T(i,j,k) = T(i,jendm-(j-jendm-1),k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_Z0_ )
      if ( kstam == ksta_G ) then
         do k = ksta2m, kstam-1
         do j = jstam , jendm
         do i = istam , iendm
            T(i,j,k) = T(i,j,kstam-k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_Z1_ )
      if ( kendm == kend_G ) then
         do k = kendm+1, kend2m
         do j = jstam  , jendm
         do i = istam  , iendm
            T(i,j,k) = T(i,j,kendm-(k-kendm-1))
         enddo
         enddo
         enddo
      endif
#endif
! --- --- B.C. Mirror
!
      return
      end subroutine HIFU3D_Update_Boundary_T

#endif /* _TEMPERATURE_ */

#ifdef _THERMAL_DOSE_

! ----------------------------------------------------------------------
      subroutine HIFU3D_Cal_Dose
! ----------------------------------------------------------------------
      use Grid
      use FDM
      use Array_Index
      use Physical_Properties
!
      implicit none
!
      integer*4 :: i, j, k
      integer*4 :: ID
      real*8 :: s0, s1, s2, s3
!
      real*8, parameter :: T_43 = 6.d0
!ocl temp(s0)
      do k = kstam , kendm
      do j = jstam , jendm
      do i = istam , iendm
         s0 = T_43 - T(i,j,k)
         if ( s0 < 0.d0 ) then 
            Dose(i,j,k) = Dose(i,j,k) + dtau*.50d0**s0
         else
            Dose(i,j,k) = Dose(i,j,k) + dtau*.25d0**s0
         endif
      enddo
      enddo
      enddo
!
      return
      end subroutine HIFU3D_Cal_Dose

#endif /* _THERMAL_DOSE_ */

#ifdef _DENATURATION_

! ----------------------------------------------------------------------
      subroutine HIFU3D_Cal_Grad_Phi
! ----------------------------------------------------------------------
      use Grid
      use FDM
      use Array_Index
      use Physical_Properties
!
      implicit none
!
      integer*4 :: i, j, k
      real*8 :: s1, s2, s3
!
! +++ +++ Phix => Exx, Phiy => Eyy, Phiz => Ezz
c!ocl temp(s1)
!ocl temp(s1, s2, s3)
      do k = ksta1n, kend1n
      do j = jsta1n, jend1n
c      do k = kstam , kendm
c      do j = jstam , jendm
      do i = ista1n, iend1n
#ifdef _UNROLL_FDM_
         exx(i,j,k) =
     $         d1n(ms  )*phi(i+ms  ,j,k)
     $        +d1n(ms+1)*phi(i+ms+1,j,k)
     $        +d1n(ms+2)*phi(i+ms+2,j,k)
     $        +d1n(ms+3)*phi(i+ms+3,j,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*phi(i+ms+4,j,k)
     $        +d1n(ms+5)*phi(i+ms+5,j,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*phi(i+ms+6,j,k)
     $        +d1n(ms+7)*phi(i+ms+7,j,k)
#endif
#else /* _UNROLL_FDM_ */
         s1 = 0.d0
         do m = ms, me
            s1 = s1 + d1n(m)*phi(i+m,j,k)
         enddo
         exx(i,j,k) = s1
#endif /* _UNROLL_FDM_ */
!
      enddo
c      enddo
c      enddo
!
c!ocl temp(s2)
c      do k = kstam , kendm
c      do j = jsta1n, jend1n
      do i = istam , iendm
#ifdef _UNROLL_FDM_
         eyy(i,j,k) =
     $         d1n(ms  )*phi(i,j+ms  ,k)
     $        +d1n(ms+1)*phi(i,j+ms+1,k)
     $        +d1n(ms+2)*phi(i,j+ms+2,k)
     $        +d1n(ms+3)*phi(i,j+ms+3,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*phi(i,j+ms+4,k)
     $        +d1n(ms+5)*phi(i,j+ms+5,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*phi(i,j+ms+6,k)
     $        +d1n(ms+7)*phi(i,j+ms+7,k)
#endif
#else /* _UNROLL_FDM_ */
         s2 = 0.d0
         do m = ms, me
            s2 = s2 + d1n(m)*phi(i,j+m,k)
         enddo
         eyy(i,j,k) = s2
#endif /* _UNROLL_FDM_ */
!
      enddo
c      enddo
c      enddo
!
c!ocl temp(s3)
c      do k = ksta1n, kend1n
c      do j = jstam , jendm
      do i = istam , iendm
#ifdef _UNROLL_FDM_
         ezz(i,j,k) =
     $         d1n(ms  )*phi(i,j,k+ms  )
     $        +d1n(ms+1)*phi(i,j,k+ms+1)
     $        +d1n(ms+2)*phi(i,j,k+ms+2)
     $        +d1n(ms+3)*phi(i,j,k+ms+3)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*phi(i,j,k+ms+4)
     $        +d1n(ms+5)*phi(i,j,k+ms+5)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*phi(i,j,k+ms+6)
     $        +d1n(ms+7)*phi(i,j,k+ms+7)
#endif
#else /* _UNROLL_FDM_ */
         s3 = 0.d0
         do m = ms, me
            s3 = s3 + d1n(m)*phi(i,j,k+m)
         enddo
         ezz(i,j,k) = s3
#endif /* _UNROLL_FDM_ */
!
      enddo
      enddo
      enddo
!
! --- --- Phix => Exx, Phiy => Eyy, Phiz => Ezz
!
      return
      end subroutine HIFU3D_Cal_Grad_Phi

! ----------------------------------------------------------------------
      subroutine HIFU3D_Cal_Phi
! ----------------------------------------------------------------------
      use Grid
      use FDM
      use Array_Index
      use Physical_Properties
!
      implicit none
!
! +++ Denaturation properties
      real*8, parameter :: T_m = 13.d0, T_c = 18.d0
!      real*8, parameter :: T_m = 23.d0, T_c = 33.d0
      real*8, parameter :: Mobility = 1.d0
      real*8, parameter :: Well = 2.d0
!
      integer*4 :: i, j, k
      integer*4 :: ID
      real*8 :: s0, s1, s2, s3, s4
      real*8 :: eps
!
      eps = 2.d0 * dx
!ocl temp(s1, s2, s3, s4)
      do k = kstam , kendm
      do j = jstam , jendm
      do i = istam , iendm
#ifdef _UNROLL_FDM_
         s1 =  d1m(ns  )*exx(i+ns  ,j,k)
     $        +d1m(ns+1)*exx(i+ns+1,j,k)
     $        +d1m(ns+2)*exx(i+ns+2,j,k)
     $        +d1m(ns+3)*exx(i+ns+3,j,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*exx(i+ns+4,j,k)
     $        +d1m(ns+5)*exx(i+ns+5,j,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*exx(i+ns+6,j,k)
     $        +d1m(ns+7)*exx(i+ns+7,j,k)
#endif
         s2 =  d1m(ns  )*eyy(i,j+ns  ,k)
     $        +d1m(ns+1)*eyy(i,j+ns+1,k)
     $        +d1m(ns+2)*eyy(i,j+ns+2,k)
     $        +d1m(ns+3)*eyy(i,j+ns+3,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*eyy(i,j+ns+4,k)
     $        +d1m(ns+5)*eyy(i,j+ns+5,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*eyy(i,j+ns+6,k)
     $        +d1m(ns+7)*eyy(i,j+ns+7,k)
#endif
         s3 =  d1m(ns  )*ezz(i,j,k+ns  )
     $        +d1m(ns+1)*ezz(i,j,k+ns+1)
     $        +d1m(ns+2)*ezz(i,j,k+ns+2)
     $        +d1m(ns+3)*ezz(i,j,k+ns+3)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1m(ns+4)*ezz(i,j,k+ns+4)
     $        +d1m(ns+5)*ezz(i,j,k+ns+5)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1m(ns+6)*ezz(i,j,k+ns+6)
     $        +d1m(ns+7)*ezz(i,j,k+ns+7)
#endif
#else /* _UNROLL_FDM_ */
         s1 = 0.d0
         s2 = 0.d0
         s3 = 0.d0
         do n = ns, ne
            s1 = s1 + d1m(n)*exx(i+n,j,k)
            s2 = s2 + d1m(n)*eyy(i,j+n,k)
            s3 = s3 + d1m(n)*ezz(i,j,k+n)
         enddo
#endif /* _UNROLL_FDM_ */
!
         s1 = - eps * eps * ( s1 + s2 + s3 )
         s2 = phi(i,j,k)*(phi(i,j,k)-1.d0)*(phi(i,j,k)-.5d0)
         if ( T(i,j,k) <= T_m ) then
            s3 = 0.d0
            s4 = 0.d0
         else if ( T(i,j,k) <= T_c ) then
            s3 = .5d0 * (T(i,j,k)-T_m)/(T_c-T_m)
     $          * phi(i,j,k)*(1.d0-phi(i,j,k))
            s4 = 0.d0
!TUNE02**************************************************************>>>
!        else if ( T(i,j,k) > T_c ) then
!TUNE02-----------------------------------------------------------------
         else
!TUNE02END<<<***********************************************************
            s3 =  .5d0 * phi(i,j,k)*(1.d0-phi(i,j,k))
            s4 = (T(i,j,k)-T_c)/(T_c-T_m) * phi(i,j,k)
         endif
!
         phi(i,j,k) = phi(i,j,k)
     $             - dtau * Mobility * ( s1 + Well*( s2 + s3 + s4 ) )
!
      enddo
      enddo
      enddo
!
      return
      end subroutine HIFU3D_Cal_Phi

! ----------------------------------------------------------------------
      subroutine HIFU3D_Update_Boundary_Phi
! ----------------------------------------------------------------------
      use Array_Index
!
      implicit none
!
      integer*4 :: i, j, k
!
! +++ +++ B.C. Mirror
#if defined( _MIRROR_X0_ )
      if ( istam == ista_G ) then
         do k = kstam , kendm
         do j = jstam , jendm
         do i = ista2m, istam-1
            phi(i,j,k) = phi(istam-i,j,k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_X1_ )
      if ( iendm == iend_G ) then
         do k = kstam  , kendm
         do j = jstam  , jendm
         do i = iendm+1, iend2m
            phi(i,j,k) = phi(iendm-(i-iendm-1),j,k)
         enddo
         enddo
         enddo
      endif
#endif
!      
#if defined( _MIRROR_Y0_ )
      if ( jstam == jsta_G ) then
         do k = kstam , kendm
         do j = jsta2m, jstam-1
         do i = istam , iendm
            phi(i,j,k) = phi(i,jstam-j,k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_Y1_ )
      if ( jendm == jend_G ) then
         do k = kstam  , kendm
         do j = jendm+1, jend2m
         do i = istam  , iendm
            phi(i,j,k) = phi(i,jendm-(j-jendm-1),k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_Z0_ )
      if ( kstam == ksta_G ) then
         do k = ksta2m, kstam-1
         do j = jstam , jendm
         do i = istam , iendm
            phi(i,j,k) = phi(i,j,kstam-k)
         enddo
         enddo
         enddo
      endif
#endif
!
#if defined( _MIRROR_Z1_ )
      if ( kendm == kend_G ) then
         do k = kendm+1, kend2m
         do j = jstam  , jendm
         do i = istam  , iendm
            phi(i,j,k) = phi(i,j,kendm-(k-kendm-1))
         enddo
         enddo
         enddo
      endif
#endif
! --- --- B.C. Mirror
!
      return
      end subroutine HIFU3D_Update_Boundary_Phi
!
#endif /* _DENATURATION_ */

! <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
      end module HIFU3D
! <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

! **********************************************************************
      program ZZ_HIFU_K
! **********************************************************************
!
      use FDM
!
      use Array_Index
!
      use MyMPI
!
      use Parts
!
      use HIFU3D
!
      use PML
!
      use Physical_Properties
!
      implicit none
!
#ifdef _USE_MPI_
      include 'mpif.h'
#endif /* _USE_MPI_ */
!
      real*8, dimension(:), allocatable :: phase, amp, source
!
      integer :: istat
      integer :: dir, ics
      integer :: ierr
!
      real*8, parameter :: xfp = _FocalPointX_
      real*8, parameter :: yfp = _FocalPointY_
      real*8, parameter :: zfp = _FocalPointZ_
      integer :: ifp, jfp, kfp
      integer :: fp_flag = 0
!
#ifdef _RecordPatPiezo_
      integer, dimension(:), allocatable :: idofpc, iofpc, jofpc, kofpc
      real*8, dimension(:), allocatable :: xofpc, yofpc, zofpc
      real*8 :: x0, y0, z0, x1, y1, z1
      integer :: nrec, fileid=20
#else
#ifdef _HearingMode_
      integer :: fileid=20
#else
#ifdef _RecordPatFocalPoint_
      integer :: fileid=20
#endif /* _RecordPatFocalPoint_ */
#endif /* _HearingMode_ */
#endif /* _RecordPatPiezo_ */
!
      integer :: isize, jsize, ksize
      integer :: ibandp, jbandp, kbandp
      integer :: ibandu, jbandu, kbandu
      integer :: ibandv, jbandv, kbandv
      integer :: ibandw, jbandw, kbandw
#ifdef _TEMPERATURE_
      integer :: ibandT, jbandT, kbandT
#endif
      integer :: mpi_vecu(3), mpi_vecv(3), mpi_vecw(3), mpi_vecp(3)
#ifdef _TEMPERATURE_
      integer :: mpi_vecT(3)
#endif

#ifdef _USE_MPI_
      integer, dimension(MPI_STATUS_SIZE, 12) :: istatus
      integer, dimension(12) :: ireqp0 = mpi_request_null
      integer, dimension(12) :: irequ = mpi_request_null
      integer, dimension(12) :: ireqv = mpi_request_null
      integer, dimension(12) :: ireqw = mpi_request_null
#ifdef _TEMPERATURE_
      integer, dimension(12) :: ireqT = mpi_request_null
#endif
#ifdef _THERMAL_DOSE_
      integer, dimension(12) :: ireqDose = mpi_request_null
#endif
#ifdef _DENATURATION_
      integer, dimension(12) :: ireqphi = mpi_request_null
#endif
#endif /* _USE_MPI_ */
!
      integer :: i, j, k, l, m, n
      integer :: dimsp(0:1, 3)
#ifdef _TEMPERATURE_
      integer :: dimsT(0:1, 3)
#endif
!
      integer :: nstep = 0
#ifdef _IMPLICIT_VIS_
      integer :: nloop = 0
#endif
      character*256 :: fname, fnamein
      real*8, dimension(4) :: delta = _PITCH_
      real*8, dimension(3) :: orig = 0.d0
!
      real*8 :: rho_inf, cs_inf
      real*8 :: K_L
!
      real*8 :: s0, s1, s2, s3, s4, s5, s6, s7, s8, s00, s01, s10, s11
      real*8 :: pi
!
      real*8, parameter :: amp_0 = 1.d-1         ! Amplitude [?]
      real*8, parameter :: frequency = 1.d+6   ! [Hz]
      real*8, parameter :: r3 = 1.d0/3.d0
      real*8 :: tstep, omega
      real*8 :: eps, dist
!
      real*8 :: cpu, cpu0, cpu1, cpu0core
#ifdef _USE_MPI_
      real*8 :: comm_time = 0.d0
#endif /* _USE_MPI_ */
!
      integer :: ID
!
#ifdef _ARGUMENT_
      integer :: iargc
      character*256 :: argv
      if ( iargc() /= 6 ) then
         write(*,*) "Argument is out of order.", iargc()
         write(*,*) "Usage : a.out nx ny nz idim jdim kdim"
         stop
      endif
      call getarg(1,argv) ; read(argv, *) i
      call getarg(2,argv) ; read(argv, *) j
      call getarg(3,argv) ; read(argv, *) k
      call getarg(4,argv) ; read(argv, *) l
      call getarg(5,argv) ; read(argv, *) m
      call getarg(6,argv) ; read(argv, *) n
#endif /* _ARGUMENT_ */
!
#ifdef _ARGUMENT_
      call Array_Global_Index_Init(i, j, k)
#endif
!
      call FDM_Init
!
! +++ Set Physical Properties
      call set_physical_properties
! ---
! +++ Initializing communicator
#ifdef _ARGUMENT_
      call MyMPI_init_comm_cart(l, m, n)
#else
      call MyMPI_init_comm_cart
#endif /* _ARGUMENT_ */
! ---
!
      call Array_Index_Init(ista, iend, jsta, jend, ksta, kend)
!
      call HIFU3D_Allocate
!
      call HIFU3D_Initialize_Array
!
      call PML_Init
!
#ifdef _USE_MPI_
!
#ifdef _BUFFER_SENDRECEIVE_
      call MyMPI_Allocate_Buffer_Array
#endif /* _BUFFER_SENDRECEIVE_ */
!
! +++ U
      isize = SIZE(u, 1) ; jsize = SIZE(u, 2) ; ksize = SIZE(u, 3)
      ibandu = istan - ista2n + 1 
      jbandu = jstam - jsta2m
      kbandu = kstam - ksta2m
      call MyMPI_init_type_vector
     $( isize, jsize, ksize, ibandu, jbandu, kbandu, mpi_vecu)
! --- U
! +++ V
      isize = SIZE(v, 1) ; jsize = SIZE(v, 2) ; ksize = SIZE(v, 3)
      ibandv = istam - ista2m
      jbandv = jstan - jsta2n + 1
      kbandv = kstam - ksta2m
      call MyMPI_init_type_vector
     $( isize, jsize, ksize, ibandv, jbandv, kbandv, mpi_vecv)
! --- V
! +++ W
      isize = SIZE(w, 1) ; jsize = SIZE(w, 2) ; ksize = SIZE(w, 3)
      ibandw = istam - ista2m
      jbandw = jstam - jsta2m
      kbandw = kstan - ksta2n + 1
      call MyMPI_init_type_vector
     $( isize, jsize, ksize, ibandw, jbandw, kbandw, mpi_vecw)
! --- W
! +++ P
      isize = SIZE(p0, 1) ; jsize = SIZE(p0, 2) ; ksize = SIZE(p0, 3)
      ibandp = istam - ista1m
      jbandp = jstam - jsta1m
      kbandp = kstam - ksta1m
      call MyMPI_init_type_vector
     $( isize, jsize, ksize, ibandp, jbandp, kbandp, mpi_vecp)
!
      dimsp(0,1)= LBOUND(p0,1) ; dimsp(1,1)= dimsp(0,1) + SIZE(p0,1) - 1
      dimsp(0,2)= LBOUND(p0,2) ; dimsp(1,2)= dimsp(0,2) + SIZE(p0,2) - 1
      dimsp(0,3)= LBOUND(p0,3) ; dimsp(1,3)= dimsp(0,3) + SIZE(p0,3) - 1
! --- P
#ifdef _TEMPERATURE_
! +++ T
      isize = SIZE(T, 1) ; jsize = SIZE(T, 2) ; ksize = SIZE(T, 3)
      ibandT = istam - ista2m
      jbandT = jstam - jsta2m
      kbandT = kstam - ksta2m
      call MyMPI_init_type_vector
     $( isize, jsize, ksize, ibandT, jbandT, kbandT, mpi_vecT)
!
      dimsT(0,1)= LBOUND(T,1) ; dimsT(1,1)= dimsT(0,1) + SIZE(T,1) - 1
      dimsT(0,2)= LBOUND(T,2) ; dimsT(1,2)= dimsT(0,2) + SIZE(T,2) - 1
      dimsT(0,3)= LBOUND(T,3) ; dimsT(1,3)= dimsT(0,3) + SIZE(T,3) - 1
! --- T
#endif
#endif /* _USE_MPI_ */
! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
!
      pi= DACOS(-1.d0) ! PI
!
      omega = 2.d0*pi*frequency ! Angular frequency [rad/s]
!
      ID = 0 ! Water
      rho_inf = coef_rho(ID) * ( p_inf + A1(ID) )**rgamma(ID) ! Density [kg/m^3]
      cs_inf = DSQRT ( gamma(ID) * ( p_inf + A1(ID)) / rho_inf ) ! Sound speed [m/s]
!
      if ( myrank == 0 )
     $     write(*,*) 'rho(WATER) =', rho_inf, ': cs(WATER) =', cs_inf
!
      dt = ( _CFL_ ) * ( _PITCH_ ) / cs_inf ! Time increment [s]
      delta(4) = dt
!
      dtau = ( _DTAU_ ) * dt
!
! ----------------------------------------------------------------------
! Generate Initial Data
#ifdef _TEST_CHECK_DATA_
! ----------------------------------------------------------------------
!
      call Input_TestData ( omega, rho_inf, cs_inf )
!
#ifdef _USE_MPI_
#ifdef _BUFFER_SENDRECEIVE_
      call MyMPI_SendReceive_p0( ibandp, jbandp, kbandp, p0 , mpi_vecp )
      call MyMPI_SendReceive_u ( ibandu, jbandu, kbandu, u  , mpi_vecu )
      call MyMPI_SendReceive_v ( ibandv, jbandv, kbandv, v  , mpi_vecv )
      call MyMPI_SendReceive_w ( ibandw, jbandw, kbandw, w  , mpi_vecw )
      call MyMPI_SendReceive_T ( ibandT, jbandT, kbandT, phi, mpi_vecT )
#else  /* _BUFFER_SENDRECEIVE_ */
      call MyMPI_sendp(ibandp, jbandp, kbandp, p0, mpi_vecp, ireqp0)
      call MyMPI_sendp(ibandu, jbandu, kbandu, u, mpi_vecu, irequ)
      call MyMPI_sendp(ibandv, jbandv, kbandv, v, mpi_vecv, ireqv)
      call MyMPI_sendp(ibandw, jbandw, kbandw, w, mpi_vecw, ireqw)
      call MyMPI_sendp(ibandT, jbandT, kbandT, phi, mpi_vecT, ireqphi)
#endif /* _BUFFER_SENDRECEIVE_ */
#endif /* _USE_MPI_ */
!
      call Check_TestResults
! ----------------------------------------------------------------------
#endif /* _TEST_CHECK_DATA_ */
! ----------------------------------------------------------------------
!
! ----------------------------------------------------------------------
! Load Piezo Data
#if defined( _PIEZO_DATA_LIST_ )
! ----------------------------------------------------------------------
#ifdef _USE_MPI_
      cpu0 = MPI_WTIME()
#else
      STIME(cpu0)
#endif
!
      write(fname,_PIEZO_DATA_LIST_) myrank
      call Parts_Load( fname, nfp, fp )
!
#ifdef _USE_MPI_
      cpu1 = MPI_WTIME() - cpu0
      call MPI_ALLREDUCE(cpu1,cpu,1,mpi_real8,mpi_max
     $     ,mpi_comm_world,ierr)
#else
      ETIME(cpu, cpu0)
#endif
      if ( myrank == 0 ) write(*,*) 'Load Piezo Data TIME=',cpu
!
      call Parts_SDFtoVF( nfp, fp )
!
! ----------------------------------------------------------------------
#endif /* _PIEZO_DATA_LIST_ */
! ----------------------------------------------------------------------
! Load Extra Parts Data
#ifdef _PARTS_DATA_LIST_
! ----------------------------------------------------------------------
!
#ifdef _USE_MPI_
      cpu0 = MPI_WTIME()
#else
      STIME(cpu0)
#endif
!
      write(fname,_PARTS_DATA_LIST_) myrank
      call Parts_Load( fname, nep, ep )
!
#ifdef _USE_MPI_
      cpu1 = MPI_WTIME() - cpu0
      call MPI_ALLREDUCE(cpu1,cpu,1,mpi_real8,mpi_max
     $     ,mpi_comm_world,ierr)
#else
      ETIME(cpu, cpu0)
#endif
      if ( myrank == 0 ) write(*,*) 'Load Parts Data TIME=',cpu
!
      call Parts_SDFtoVF( nep, ep )
!
! ----------------------------------------------------------------------
#endif /* _PARTS_DATA_LIST_ */
! ----------------------------------------------------------------------
! Load Object Data
#ifdef _OBJECT_DATA_
! ----------------------------------------------------------------------
#ifdef _USE_MPI_
      cpu0 = MPI_WTIME()
#else
      STIME(cpu0)
#endif
!
      write(fname,_OBJECT_DATA_) myrank
      write(*,*) "Data load from ", fname
!
! Load Body
      call dtloadS(nstep, fname, delta, orig, dimsp, w1)
!
! Check pitch
      if ( real(_PITCH_) /= real(delta(1)) .or.
     $     real(_PITCH_) /= real(delta(2)) .or.
     $     real(_PITCH_) /= real(delta(3)) ) then
         write(*,'("Object Pitch is not matched !!!")')
         write(*,*) fname, _PITCH_, delta(1)
      endif
!
#ifdef _USE_MPI_
      cpu1 = MPI_WTIME() - cpu0
      call MPI_ALLREDUCE(cpu1,cpu,1,mpi_real8,mpi_max
     $     ,mpi_comm_world,ierr)
#else
      ETIME(cpu, cpu0)
#endif
      if ( myrank == 0 ) write(*,*) 'Load Object Data TIME=',cpu
!
! +++ +++ B.C.
!
#ifdef _USE_MPI_
#ifdef _BUFFER_SENDRECEIVE_
      call MyMPI_SendReceive_p0( ibandp, jbandp, kbandp, w1, mpi_vecp )
#else  /* _BUFFER_SENDRECEIVE_ */
      call MyMPI_sendp(ibandp, jbandp, kbandp, w1, mpi_vecp, ireqp0)
#endif /* _BUFFER_SENDRECEIVE_ */
#endif /* _USE_MPI_ */
! 
      call HIFU3D_BC_Mirror( dimsp, w1 )
!
! --- --- B.C.
! +++
      do k = ksta1m, kend1m
      do j = jsta1m, jend1m
      do i = ista1m, iend1m
! Mask : Double => Integer
         mask(i,j,k) = IDNINT(w1(i,j,k))
!
         w1(i,j,k) = 1.d0 ! Fraction
!
         w2(i,j,k) = 0.d0
!
#ifdef _VOLUME_OBJECT_
         fb(i,j,k) = 1.d0 ! Fraction
#endif
      enddo
      enddo
      enddo
! ---
!
#ifdef _PIEZO_DATA_LIST_
      if ( nfp /= 0 ) then
!
      do n = 1, nfp
      do k = fp(n)% ksta, fp(n)% kend
      do j = fp(n)% jsta, fp(n)% jend
      do i = fp(n)% ista, fp(n)% iend
         w2(i,j,k) = w2(i,j,k) + fp(n)% x(i,j,k)
      enddo
      enddo
      enddo
      enddo
!
      endif
#endif /* _PIEZO_DATA_LIST_ */
!
#ifdef _PARTS_DATA_LIST_
      if ( nep /= 0 ) then
!
      do n = 1, nep
      do k = ep(n)% ksta, ep(n)% kend
      do j = ep(n)% jsta, ep(n)% jend
      do i = ep(n)% ista, ep(n)% iend
         w2(i,j,k) = w2(i,j,k) + ep(n)% x(i,j,k)
      enddo
      enddo
      enddo
      enddo
!
      endif
#endif /* _PARTS_DATA_LIST_ */
!
#ifdef _PIEZO_DATA_LIST_
      if ( nfp /= 0 ) then
!
      do n = 1, nfp
      do k = fp(n)% ksta, fp(n)% kend
      do j = fp(n)% jsta, fp(n)% jend
      do i = fp(n)% ista, fp(n)% iend
         if ( w2(i,j,k) >= 1.d0 ) then
            fp(n)% x(i,j,k) = fp(n)% x(i,j,k)/w2(i,j,k)
#ifdef _VOLUME_OBJECT_
            fb(i,j,k) = 0.d0
         else
            if ( w1(i,j,k) + w2(i,j,k) >= 1.d0 ) then
               fb(i,j,k) = 1.d0 - w2(i,j,k)
            endif
#endif
         endif
      enddo
      enddo
      enddo
      enddo
!
      endif
#endif /* _PIEZO_DATA_LIST_ */
!
#ifdef _PARTS_DATA_LIST_
!
      if ( nep /= 0 ) then
!
      do n = 1, nep
      do k = ep(n)% ksta, ep(n)% kend
      do j = ep(n)% jsta, ep(n)% jend
      do i = ep(n)% ista, ep(n)% iend
         if ( w2(i,j,k) >= 1.d0 ) then
            ep(n)% x(i,j,k) = ep(n)% x(i,j,k) / w2(i,j,k)
#ifdef _VOLUME_OBJECT_
            fb(i,j,k) = 0.d0
         else
            if ( w1(i,j,k) + w2(i,j,k) >= 1.d0 ) then
               fb(i,j,k) = 1.d0 - w2(i,j,k)
            endif
#endif
         endif
      enddo
      enddo
      enddo
      enddo
!
      endif
!
#endif /* _PARTS_DATA_LIST_ */
! ----------------------------------------------------------------------
#endif /* _OBJECT_DATA_ */
! ----------------------------------------------------------------------
#ifdef _PhaseDelayTable_
! ----------------------------------------------------------------------
      if ( nfp /= 0 ) then
!
      allocate( phase(nfp) )
      allocate( amp(nfp) )
      allocate( source(nfp) )
!
      fname = _PhaseDelayTable_
      j = 0
      s2 = 0.d0
      open(2, file=fname, status='old')
      do
         read(2,*, iostat=istat) i, s0, s1 ! ID, phase, amp
         if ( istat /= 0 ) exit
         j = j + 1
         s2 = s2 + s1*s1
         do n = 1, nfp
            if ( fp(n)% id == i ) then
              phase(n) = s0
              amp(n) = s1
            endif
         enddo
      enddo
      close(2)
      
      s2 = dsqrt( dble(j) / s2 )

      do n = 1, nfp
         amp(n) = amp(n) * s2
      enddo
!
      endif
!
#else /* _PhaseDelayTable_ */
!
#if defined( _PIEZO_DATA_LIST_ )
      if ( nfp /= 0 ) then
!
      allocate( phase(nfp) )
      allocate( amp(nfp) )
      allocate( source(nfp) )
!
      do n = 1, nfp
         phase(n) = 0.d0
         amp(n) = 1.d0
      enddo
!
      endif
#endif /* _PIEZO_DATA_LIST_ */
! ----------------------------------------------------------------------
#endif /* _PhaseDelayTable_ */
! ----------------------------------------------------------------------
#ifdef _RecordPatPiezo_
! ----------------------------------------------------------------------
!
      fname = _RecordPointsTalbe_
      open(10, file=fname, status='old', iostat=istat)
      nrec = 0
      do
         read(10,'(1x,a4,3(1x,e16.8e3),3(1x,i4))', iostat=istat)
     $ n, s1, s2, s3, i, j, k
         if ( istat /= 0 ) exit
         if (  kstan <= k .and. k <= kendn
     $   .and. jstan <= j .and. j <= jendn
     $   .and. istan <= i .and. i <= iendn ) then
            nrec = nrec + 1
         endif
      enddo
      close(10)
!
      write(*,*) "Myrank ", myrank, " : nrec = ", nrec
!
      allocate( idofpc(nrec) )
      allocate( iofpc(nrec) )
      allocate( jofpc(nrec) )
      allocate( kofpc(nrec) )
      allocate( xofpc(nrec) )
      allocate( yofpc(nrec) )
      allocate( zofpc(nrec) )
!
      open(10, file=fname, status='old', iostat=istat)
      nrec = 0
      do
         read(10,*, iostat=istat) n, s1, s2, s3, i, j, k
         if ( istat /= 0 ) exit
         if (  kstan <= k .and. k <= kendn
     $   .and. jstan <= j .and. j <= jendn
     $   .and. istan <= i .and. i <= iendn ) then
            nrec = nrec + 1
            idofpc(nrec) = n
            xofpc(nrec) = s1
            yofpc(nrec) = s2
            zofpc(nrec) = s3
            iofpc(nrec) = i
            jofpc(nrec) = j
            kofpc(nrec) = k
         endif
      enddo
      close(10)
!
      if ( nrec /= 0 ) then
         do n = 1, nrec
            write(fname,'("patpiezo-",i2.2,".dat")') idofpc(n)
            open((fileid + n), file=fname, status='unknown') 
         enddo
      endif
! ----------------------------------------------------------------------
#endif /* _RecordPatPiezo_ */
! ----------------------------------------------------------------------
!
#if defined(_HearingMode_) || defined(_RecordPatFocalPoint_)
! +++ Focal point
      ifp = IDNINT( ( xfp - orig(1) ) / delta(1) ) + ista
      jfp = IDNINT( ( yfp - orig(2) ) / delta(2) ) + jsta
      kfp = IDNINT( ( zfp - orig(3) ) / delta(3) ) + ksta
!
      if (  istan <= ifp .and. ifp <= iendn
     $.and. jstan <= jfp .and. jfp <= jendn
     $.and. kstan <= kfp .and. kfp <= kendn ) then
!         
         write(*,*) "Focal point is located at", myrank
         write(*,*) ifp, jfp, kfp
!
         fp_flag = 1
!
#ifdef _RecordPatFocalPoint_
         write(fname,'("patfp-",i2.2,".dat")') myrank
         open(fileid, file=fname, status='unknown') 
#endif /* _RecordPatFocalPoint_ */
!
      endif
! ---
#endif /* defined(_HearingMode_) || defined(_RecordPatFocalPoint_) */
!
! ---
!
      call HIFU3D_Constant_Mixture_Properties
!
!
      nstep = 0
!
#if defined( _RESTART_FILE_NAME_ ) && defined( _RESTART_ )
#ifdef _USE_MPI_
      cpu0 = MPI_WTIME()
#else
      STIME(cpu0)
#endif
!
      write(fname, _RESTART_FILE_NAME_ ) myrank
      call HIFU3D_Restart(fname, 1, nstep)
!
#ifdef _USE_MPI_
      cpu1 = MPI_WTIME() - cpu0
      call MPI_ALLREDUCE(cpu1,cpu,1,mpi_real8,mpi_max
     $     ,mpi_comm_world,ierr)
#else
      ETIME(cpu, cpu0)
#endif
      if ( myrank == 0 ) write(*,*) 'Load Restart Data TIME=',cpu
!
#endif /* _RESTART_ */
!
!TUNE09**************************************************************>>>
!ocl temp(ID, s0, s1, s2, s3)
      do k = kstam , kendm
      do j = jstam , jendm
      do i = istam , iendm
         mask1(1,i,j,k) = 0.0d0
         mask1(2,i,j,k) = 0.0d0
         mask1(3,i,j,k) = 0.0d0
         if ( mask(i-1,j,k) == mask(i  ,j,k) .and.
     $        mask(i  ,j,k) == mask(i+1,j,k) ) then
           mask1(1,i,j,k) = 1.0d0
         endif
         if ( mask(i,j-1,k) == mask(i,j  ,k) .and.
     $        mask(i,j  ,k) == mask(i,j+1,k) ) then
           mask1(2,i,j,k) = 1.0d0
         endif
         if ( mask(i,j,k-1) == mask(i,j,k  ) .and.
     $        mask(i,j,k  ) == mask(i,j,k+1) ) then
           mask1(3,i,j,k) = 1.0d0
         endif
      enddo
      enddo
      enddo
!TUNE09END<<<***********************************************************
      do k = ksta1n , kend1n
      do j = jsta1n , jend1n
      do i = ista1n , iend1n
         mask2(1,i,j,k) = 0.d0
         mask2(2,i,j,k) = 0.d0
         mask2(3,i,j,k) = 0.d0
         if ( mask(i  ,j,k) == mask(i+1,j,k) ) mask2(1,i,j,k) = 1.d0
         if ( mask(i,j  ,k) == mask(i,j+1,k) ) mask2(2,i,j,k) = 1.d0
         if ( mask(i,j,k  ) == mask(i,j,k+1) ) mask2(3,i,j,k) = 1.d0
      enddo
      enddo
      enddo
#ifdef _USE_MPI_
      cpu0core = MPI_WTIME()
#else
      STIME(cpu0core)
#endif
!
#ifndef _INTEL_
!>>> PA
      call START_COLLECTION("main_loop")
      call fpcoll_start()
!<<< PA
#endif /* _INTEL_ */
!
! ----------------------------------------------------------------------
 2007 continue
!
      nstep = nstep + 1
! ----------------------------------------------------------------------
!
      if ( myrank == 0 ) write(*,*) nstep
!
! +++ Density & sound speed of liquid
!
      call HIFU3D_Update_Mixture_Properties
!
! --- Density & sound speed of liquid
!
#ifdef _IMPLICIT_VIS_
!
      do k = ksta2m , kend2m
      do j = jsta2m , jend2m
      do i = ista2n , iend2n
         un(i,j,k) = u(i,j,k)
      enddo
      enddo
      enddo
      do k = ksta2m , kend2m
      do j = jsta2n , jend2n
      do i = ista2m , iend2m
         vn(i,j,k) = v(i,j,k)
      enddo
      enddo
      enddo
      do k = ksta2n , kend2n
      do j = jsta2m , jend2m
      do i = ista2m , iend2m
         wn(i,j,k) = w(i,j,k)
      enddo
      enddo
      enddo
!
      nloop = 0
!
 2009 continue
!
      nloop = nloop + 1
!
      if ( myrank == 0 ) write(*,*) nstep, nloop
!
#endif /* _IMPLICIT_VIS_ */
!
! +++ Viscous & Viscous dissipation => w2
!
      call HIFU3D_Cal_Viscous
!
! --- Viscous & Viscous dissipation => w2
!
! +++ U
!
      call HIFU3D_Cal_U
!
#ifdef _USE_MPI_
      cpu0 = MPI_WTIME()
!
#ifdef _BUFFER_SENDRECEIVE_
      call MyMPI_SendReceive_u ( ibandu, jbandu, kbandu, u , mpi_vecu )
#else  /* _BUFFER_SENDRECEIVE_ */
      call MyMPI_sendp(ibandu, jbandu, kbandu, u, mpi_vecu, irequ)
#endif /* _BUFFER_SENDRECEIVE_ */
!
      comm_time = comm_time + ( MPI_WTIME() - cpu0 )
#endif /* _USE_MPI_ */
!
      call HIFU3D_Update_Boundary_U
!
! --- U
!
! +++ V
!
      call HIFU3D_Cal_V
! 
#ifdef _USE_MPI_
      cpu0 = MPI_WTIME()
!
#ifdef _BUFFER_SENDRECEIVE_
      call MyMPI_SendReceive_v ( ibandv, jbandv, kbandv, v , mpi_vecv )
#else  /* _BUFFER_SENDRECEIVE_ */
      call MyMPI_sendp(ibandv, jbandv, kbandv, v, mpi_vecv, ireqv)
#endif /* _BUFFER_SENDRECEIVE_ */
!
      comm_time = comm_time + ( MPI_WTIME() - cpu0 )
#endif /* _USE_MPI_ */
!
      call HIFU3D_Update_Boundary_V
!
! --- V
!
! +++ W
!
      call HIFU3D_Cal_W
! 
#ifdef _USE_MPI_
      cpu0 = MPI_WTIME()
!
#ifdef _BUFFER_SENDRECEIVE_
      call MyMPI_SendReceive_w ( ibandw, jbandw, kbandw, w , mpi_vecw )
#else  /* _BUFFER_SENDRECEIVE_ */
      call MyMPI_sendp(ibandw, jbandw, kbandw, w, mpi_vecw, ireqw)
#endif /* _BUFFER_SENDRECEIVE_ */
!
      comm_time = comm_time + ( MPI_WTIME() - cpu0 )
#endif /* _USE_MPI_ */
!
      call HIFU3D_Update_Boundary_W
!
! --- W
!
#ifdef _IMPLICIT_VIS_
!
      if ( nloop < _IMPLICIT_VIS_ ) go to 2009
!
#endif /* _IMPLICIT_VIS_ */
!
#ifdef _TEMPERATURE_
! +++ T
!
      call HIFU3D_Cal_Grad_T
!
      call HIFU3D_Cal_T
!
#ifdef _USE_MPI_
      cpu0 = MPI_WTIME()
!
#ifdef _BUFFER_SENDRECEIVE_
      call MyMPI_SendReceive_T ( ibandT, jbandT, kbandT, T , mpi_vecT )
#else  /* _BUFFER_SENDRECEIVE_ */
      call MyMPI_sendp(ibandT, jbandT, kbandT, T, mpi_vecT, ireqT)
#endif /* _BUFFER_SENDRECEIVE_ */
!
      comm_time = comm_time + ( MPI_WTIME() - cpu0 )
#endif /* _USE_MPI_ */
!
      call HIFU3D_Update_Boundary_T
!
! --- T
#endif /* _TEMPERATURE_ */
!
#ifdef _THERMAL_DOSE_
! +++ Dose
!
      call HIFU3D_Cal_Dose
!
#ifdef _USE_MPI_
      cpu0 = MPI_WTIME()
!
#ifdef _BUFFER_SENDRECEIVE_
      call MyMPI_SendReceive_T( ibandT, jbandT, kbandT, Dose, mpi_vecT )
#else  /* _BUFFER_SENDRECEIVE_ */
      call MyMPI_sendp(ibandT, jbandT, kbandT, Dose, mpi_vecT, ireqDose)
#endif /* _BUFFER_SENDRECEIVE_ */
!
      comm_time = comm_time + ( MPI_WTIME() - cpu0 )
#endif /* _USE_MPI_ */
!
! --- Dose
#endif /* _THERMAL_DOSE_ */
!
#ifdef _DENATURATION_
! +++ Phi
!
      call HIFU3D_Cal_Grad_Phi
!
      call HIFU3D_Cal_Phi
!
#ifdef _USE_MPI_
      cpu0 = MPI_WTIME()
!
#ifdef _BUFFER_SENDRECEIVE_
      call MyMPI_SendReceive_T ( ibandT, jbandT, kbandT, phi, mpi_vecT )
#else  /* _BUFFER_SENDRECEIVE_ */
      call MyMPI_sendp(ibandT, jbandT, kbandT, phi, mpi_vecT, ireqphi)
#endif /* _BUFFER_SENDRECEIVE_ */
!
      comm_time = comm_time + ( MPI_WTIME() - cpu0 )
#endif /* _USE_MPI_ */
!
      call HIFU3D_Update_Boundary_Phi
!
! --- phi
#endif /* _DENATURATION_ */
!
! +++ Forcing
!ocl xfill
      w1 = 0.d0
!
      tstep = dble( nstep ) * dt
!
#ifdef _HearingMode_
      if ( fp_flag == 1 ) then
         eps = delta(1)
         s0 = amp_0 * omega * DSIN( omega * ( tstep ) ) / rho_inf * r3
         do k = kfp-1, kfp+1
            s3 = dble(k-1)*delta(3) + orig(3)
         do j = jfp-1, jfp+1
            s2 = dble(j-1)*delta(2) + orig(2)
         do i = ifp-1, ifp+1
            s1 = dble(i-1)*delta(1) + orig(1)
            dist = delta(1)
     $           - dsqrt( (s1-xfp)**2 + (s2-yfp)**2 + (s3-zfp)**2 )
!
            if (eps <= dist) then
               w1(i,j,k) = w1(i,j,k) + s0
            else if ((-eps < dist).and.(dist < eps)) then
               w1(i,j,k) = w1(i,j,k) 
     $          +s0*((dist+eps)/(2.d0*eps)+dsin(pi*dist/eps)/(2.d0*pi))
            endif
         enddo
         enddo
         enddo
!         w1(ifp,jfp,kfp) = w1(ifp,jfp,kfp) + s0
      endif
#else
#ifdef _PIEZO_DATA_LIST_
      if ( nfp /= 0 ) then
!
      ID = _PROPERTY_ID_PIEZO_
      do n = 1, nfp
         if ( tstep + phase(n) > 0.d0 ) then
           source(n) = amp_0 * amp(n) * omega / rho0( ID )
     $                * DCOS( omega*( tstep + phase(n) ) ) * r3
         else
           source(n) = 0.d0
         endif
      enddo
!
      do n = 1, nfp
         do k = fp(n)% ksta  , fp(n)% kend
         do j = fp(n)% jsta  , fp(n)% jend
         do i = fp(n)% ista  , fp(n)% iend
            w1(i,j,k) = w1(i,j,k) + fp(n)% x(i,j,k) * source(n)
         enddo
         enddo
         enddo
      enddo
!
      endif
#endif /* _PIEZO_DATA_LIST_ */
#endif /* _HearingMode_ */
! --- Forcing
!
! +++ P
!
      call HIFU3D_Cal_Boundary_P
!
      call HIFU3D_Cal_P
!
!TUNE21**************************************************************>>>
!#ifdef _CONST_PRESSURE_AIR_
!!TUNE01**************************************************************>>>
!!ocl nosimd
!!TUNE01END<<<***********************************************************
!      do k = kstam, kendm
!      do j = jstam, jendm
!      do i = istam, iendm
!         if ( AirID(mask(i,j,k)) == 1 ) then
!            px(i,j,k) = 0.d0
!            py(i,j,k) = 0.d0
!            pz(i,j,k) = 0.d0
!         endif
!      enddo
!      enddo
!      enddo
!#endif /* _CONST_PRESSURE_AIR_ */
!TUNE21END<<<***********************************************************
!
! +++ +++ Update P
      if ( nstep >= _CHECK_IN_ ) then
         do k = kstam , kendm
         do j = jstam , jendm
         do i = istam , iendm
!TUNE21**************************************************************>>>
!            p0(i,j,k) = px(i,j,k) + py(i,j,k) + pz(i,j,k)
!TUNE21END<<<***********************************************************
!
            pmax(i,j,k) = MAX( pmax(i,j,k), p0(i,j,k) )
         enddo
         enddo
         enddo
!TUNE21**************************************************************>>>
!      else
!         do k = kstam , kendm
!         do j = jstam , jendm
!         do i = istam , iendm
!            p0(i,j,k) = px(i,j,k) + py(i,j,k) + pz(i,j,k)
!         enddo
!         enddo
!         enddo
!TUNE21END<<<***********************************************************
      endif
! --- --- Update P
!
#ifdef _USE_MPI_
      cpu0 = MPI_WTIME()
!
#ifdef _BUFFER_SENDRECEIVE_
      call MyMPI_SendReceive_p0( ibandp, jbandp, kbandp, p0, mpi_vecp )
#else  /* _BUFFER_SENDRECEIVE_ */
      call MyMPI_sendp(ibandp, jbandp, kbandp, p0, mpi_vecp, ireqp0)
#endif /* _BUFFER_SENDRECEIVE_ */
!
      comm_time = comm_time + ( MPI_WTIME() - cpu0 )
#endif /* _USE_MPI_ */
!
      call HIFU3D_Update_Boundary_P
!
! --- P
!
! Data save
      if (mod(nstep,_DATA3D_SAVE_)==0) then
#ifdef _USE_MPI_
         cpu0 = MPI_WTIME()
#else
         STIME(cpu0)
#endif /* _USE_MPI_ */
!
         write(fname,'("p",i6.6,"-",i3.3,".sph")') nstep, myrank
         call dtsaveS(nstep, fname, delta, orig, dimsp, p0)
!
#ifdef _TEMPERATURE_
         write(fname,'("T",i6.6,"-",i3.3,".sph")') nstep, myrank
         call dtsaveS(nstep, fname, delta, orig, dimsT, T)
#endif
!
#ifdef _THERMAL_DOSE_
         write(fname,'("D",i6.6,"-",i3.3,".sph")') nstep, myrank
         call dtsaveS(nstep, fname, delta, orig, dimsT, Dose)
#endif
!
#ifdef _DENATURATION_
         write(fname,'("phi",i6.6,"-",i3.3,".sph")') nstep, myrank
         call dtsaveS(nstep, fname, delta, orig, dimsT, phi)
#endif
!
#ifdef _USE_MPI_
         cpu1 = MPI_WTIME() - cpu0
         call MPI_ALLREDUCE(cpu1,cpu,1,mpi_real8,mpi_max
     $     ,mpi_comm_world,ierr)
#else
         ETIME(cpu, cpu0)
#endif /* _USE_MPI_ */
         if ( myrank == 0 ) write(*,*) 'Save Scalar Data TIME=',cpu
      endif
!
      if (mod(nstep,_DATA2D_SAVE_)==0) then
         dir = 1
#ifdef _HearingMode_
         ics = ifp
#else
         ics = nx/2
#endif
         write(fname,'("p",i1.1,"cs-",i6.6,"-",i3.3,".sph")')
     $        dir, nstep, cart_dims(2)*mycoords(3)+mycoords(2)
         call dtsaveS2D(nstep, fname, delta, orig, dimsp, p0, dir, ics)
!
         dir = 3
#ifdef _HearingMode_
         ics = kfp
#else
         ics = nz/2
#endif
         write(fname,'("p",i1.1,"cs-",i6.6,"-",i3.3,".sph")')
     $        dir, nstep, cart_dims(1)*mycoords(2)+mycoords(1)
         call dtsaveS2D(nstep, fname, delta, orig, dimsp, p0, dir, ics)
      endif
!
#ifdef _RecordPatFocalPoint_
      if ( fp_flag == 1 ) then
#ifdef _TEMPERATURE_
         write(fileid,*) tstep, p0(ifp, jfp, kfp), T(ifp, jfp, kfp)
#else
         write(fileid,*) tstep, p0(ifp, jfp, kfp)
#endif
      endif
#endif /* _RecordPatFocalPoint_ */
!
#ifdef _RecordPatPiezo_
      if ( nrec /= 0 ) then
         do n = 1, nrec
            i = iofpc(n)
            j = jofpc(n)
            k = kofpc(n)
            x0 = xofpc(n) - ( dble(i-ista)*delta(1) + orig(1) )
            y0 = yofpc(n) - ( dble(j-jsta)*delta(2) + orig(2) )
            z0 = zofpc(n) - ( dble(k-ksta)*delta(3) + orig(3) )
            x0 = x0 / delta(1)
            y0 = y0 / delta(2)
            z0 = z0 / delta(3)
            x1 = 1.d0 - x0
            y1 = 1.d0 - y0
            z1 = 1.d0 - z0
            
            s1 = 
     $           x1*y1*z1 * p0(i  ,j  ,k  )
     $         + x0*y1*z1 * p0(i+1,j  ,k  )
     $         + x1*y0*z1 * p0(i  ,j+1,k  )
     $         + x0*y0*z1 * p0(i+1,j+1,k  )
     $         + x1*y1*z0 * p0(i  ,j  ,k+1)
     $         + x0*y1*z0 * p0(i+1,j  ,k+1)
     $         + x1*y0*z0 * p0(i  ,j+1,k+1)
     $         + x0*y0*z0 * p0(i+1,j+1,k+1)

            write((fileid+n),*) tstep, p0(i,j,k), s1
!            write((fileid+n),*) tstep, p0(iofpc(n), jofpc(n), kofpc(n))
         enddo
      endif
#endif /* _RecordPatPiezo_ */
!
! ----------------------------------------------------------------------
      if ( nstep < _NSTEPMAX_ ) go to 2007
! ----------------------------------------------------------------------
!
#ifndef _INTEL_
!>>> PA
      call STOP_COLLECTION("main_loop")
      call fpcoll_stop()
!<<< PA
#endif /* _INTEL_ */
!
#ifdef _USE_MPI_
      cpu1 = MPI_WTIME() - cpu0core
      call MPI_ALLREDUCE(cpu1,cpu,1,mpi_real8,mpi_max
     $     ,mpi_comm_world,ierr)
      if ( myrank == 0 ) write(*,*) 'Rank=0: Calculation TIME=',cpu1
#else
      ETIME(cpu, cpu0core)
#endif /* _USE_MPI_ */
      if ( myrank == 0 ) write(*,*) 'Max Calculation TIME=',cpu
!
#ifdef _USE_MPI_
      call MPI_ALLREDUCE(comm_time,cpu,1,mpi_real8,mpi_max
     $     ,mpi_comm_world,ierr)
      if ( myrank == 0 ) then
         write(*,*) 'Rank=0: Communication TIME=',comm_time
         write(*,*) 'Max Communication TIME=',cpu
      endif
#endif /* _USE_MPI_ */
!
#ifdef _RecordPatFocalPoint_
      if ( fp_flag == 1 ) then
         close(fileid)
      endif
#endif /* _RecordPatFocalPoint_ */
!
#ifdef _RecordPatPiezo_
      if ( nrec /= 0 ) then
         do n = 1, nrec
            close((fileid+n))
         enddo
      endif
#endif /* _RecordPatPiezo_ */
!
! ----------------------------------------------------------------------
! Check Results
#ifdef _TEST_CHECK_DATA_
      call Check_TestResults
#endif /* _TEST_CHECK_DATA_ */
! ----------------------------------------------------------------------
!
#if defined( _RESTART_FILE_NAME_ )
#ifdef _USE_MPI_
      cpu0 = MPI_WTIME()
#else
      STIME(cpu0)
#endif /* _USE_MPI_ */
!
      write(fname, _RESTART_FILE_NAME_ ) myrank
      call HIFU3D_Restart(fname, 0, nstep)
!
#ifdef _USE_MPI_
      cpu1 = MPI_WTIME() - cpu0
      call MPI_ALLREDUCE(cpu1,cpu,1,mpi_real8,mpi_max
     $     ,mpi_comm_world,ierr)
#else
      ETIME(cpu, cpu0)
#endif /* _USE_MPI_ */
      if ( myrank == 0 ) write(*,*) 'Save Restart Data TIME=',cpu
!
#endif /* _RESTART_FILE_NAME_ */
!
      if ( nstep >= _CHECK_IN_ ) then
!
#ifdef _USE_MPI_
      call MPI_BARRIER(MPI_COMM_WORLD, ierr)
#endif /* _USE_MPI_ */
!
#ifdef _USE_MPI_
      cpu0 = MPI_WTIME()
#else
      STIME(cpu0)
#endif /* _USE_MPI_ */
!
      write(fname,'("pmax",i6.6,"-",i3.3,".sph")') nstep, myrank
      call dtsaveS(nstep, fname, delta, orig, dimsp, pmax)
!
#ifdef _USE_MPI_
      cpu1 = MPI_WTIME() - cpu0
      call MPI_ALLREDUCE(cpu1,cpu,1,mpi_real8,mpi_max
     $     ,mpi_comm_world,ierr)
#else
      ETIME(cpu, cpu0)
#endif /* _USE_MPI_ */
      if ( myrank == 0 ) write(*,*) 'Save Scalar Data TIME=',cpu
!
      endif
!
#ifdef _USE_MPI_
      call MPI_FINALIZE(ierr)
#endif /* _USE_MPI_ */
!
      stop
      end program ZZ_HIFU_K

! **********************************************************************
      subroutine dtloadS(nstep, fname, delta, orig, dims, sp)
!     scalar data for V-Isio
! **********************************************************************
      use MyMPI , only : ista, iend, jsta, jend, ksta, kend
!
      implicit none
!
      integer, intent(in) :: nstep
      character*256, intent(in) :: fname
      real*8, intent(out) :: delta(4)
      real*8, intent(out) :: orig(3)
      integer, intent(in) :: dims(0:1, 3)
      real*8, dimension(  dims(0,1):dims(1,1)
     $                  , dims(0,2):dims(1,2)
     $                  , dims(0,3):dims(1,3) ), intent(out) :: sp
!
      integer :: i, j, k
! +++ Sph Data File Format
      integer*4 :: svType, dType
      integer*4 :: IMAX, JMAX, KMAX
      real*4 :: XORG, YORG, ZORG
      real*4 :: XPITCH, YPITCH, ZPITCH
      integer*4 :: STEP ! time step
      real*4 :: TIME ! time
      real*4, dimension( :, :, : ), allocatable :: SDATA
!
      allocate( SDATA( ista:iend, jsta:jend, ksta:kend ) )
! Scalar
      open (10, file=fname,status='old',form='unformatted')
      read(10) svType, dType
      read(10) IMAX, JMAX, KMAX
      read(10) XORG, YORG, ZORG
      read(10) XPITCH, YPITCH, ZPITCH
      read(10) STEP, TIME
      read(10) (((SDATA(i,j,k), i=ista,iend)
     $                        , j=jsta,jend)
     $                        , k=ksta,kend)
      close(10)
!
      do k=ksta,kend ; do j=jsta,jend ; do i=ista,iend
         sp(i,j,k)=dble(SDATA(i,j,k))
      enddo ; enddo ; enddo
      delta(1)=dble(XPITCH)
      delta(2)=dble(YPITCH)
      delta(3)=dble(ZPITCH)
      orig(1)=dble(XORG)
      orig(2)=dble(YORG)
      orig(3)=dble(ZORG)
!
      deallocate( SDATA )
!
      return
      end subroutine dtloadS
      
! **********************************************************************
      subroutine dtsaveS(nstep, fname, delta, orig, dims, sp)
!     scalar data for V-Isio
! **********************************************************************
      use Array_Index , only : nx, ny, nz
      use mympi , only : ista, iend, jsta, jend, ksta, kend
!
      implicit none
!
      integer, intent(in) :: nstep
      character*256, intent(in) :: fname
      real*8, intent(in) :: delta(4)
      real*8, intent(in) :: orig(3)
      integer, intent(in) :: dims(0:1, 3)
      real*8, dimension( dims(0,1):dims(1,1)
     $                 , dims(0,2):dims(1,2)
     $                 , dims(0,3):dims(1,3) ), intent(in) :: sp
!
      integer :: i, j, k
! +++ Sph Data File Format
      integer*4 :: svType, dType
      integer*4 :: IMAX, JMAX, KMAX
      real*4 :: XORG, YORG, ZORG
      real*4 :: XPITCH, YPITCH, ZPITCH
      integer*4 :: STEP ! time step
      real*4 :: TIME ! time
      real*4, dimension( :, :, : ), allocatable :: SDATA
!
      allocate( SDATA( ista:iend, jsta:jend, ksta:kend ) )
! Scalar
      svType=1                  ! svType(1:scalar, 2:vector)
      dType=1                   ! dType(1:single, 2:double)
      IMAX=iend-ista+1
      JMAX=jend-jsta+1
      KMAX=kend-ksta+1
      XORG=sngl(orig(1))
      YORG=sngl(orig(2))
      ZORG=sngl(orig(3))
      XPITCH=sngl(delta(1))
      YPITCH=sngl(delta(2))
      ZPITCH=sngl(delta(3))
      STEP=nstep ; TIME=real(nstep)*sngl(delta(4)) ;
      do k=ksta,kend ; do j=jsta,jend ; do i=ista,iend
         SDATA(i,j,k)=sngl(sp(i,j,k))
      enddo ; enddo ; enddo
!
      open (10, file=fname,status='unknown',form='unformatted')
      write(10) svType, dType
      write(10) IMAX, JMAX, KMAX
      write(10) XORG, YORG, ZORG
      write(10) XPITCH, YPITCH, ZPITCH
      write(10) STEP, TIME
      write(10) (((SDATA(i,j,k), i=ista,iend)
     $                         , j=jsta,jend)
     $                         , k=ksta,kend)
      write(10) NX, NY, NZ
      write(10) ISTA, JSTA, KSTA
      close(10)
!
      deallocate( SDATA )
!
      return
      end subroutine dtsaveS

! **********************************************************************
      subroutine dtsaveS2D(nstep, fname, delta, orig, dims, sp
     $     , dir, ics)
!     scalar data for V-Isio
! **********************************************************************
      use Array_Index , only : nx, ny, nz
      use mympi , only : ista, iend, jsta, jend, ksta, kend
!
      implicit none
!
      integer, intent(in) :: nstep
      character*256, intent(in) :: fname
      real*8, intent(in) :: delta(4)
      real*8, intent(in) :: orig(3)
      integer, intent(in) :: dims(0:1, 3)
      real*8, dimension( dims(0,1):dims(1,1)
     $                 , dims(0,2):dims(1,2)
     $                 , dims(0,3):dims(1,3) ), intent(in) :: sp
      integer, intent(in) :: dir, ics
!
      integer :: i, j, k
! +++ Sph Data File Format
      integer*4 :: svType, dType
      integer*4 :: IMAX, JMAX, KMAX
      integer*4 :: IMAX_G, JMAX_G, KMAX_G
      real*4 :: XORG, YORG, ZORG
      real*4 :: XPITCH, YPITCH, ZPITCH
      integer*4 :: STEP ! time step
      real*4 :: TIME ! time
      real*4, dimension( :, :, : ), allocatable :: SDATA
!
      integer :: istat, iendt, jstat, jendt, kstat, kendt
!
      XORG=sngl(orig(1))
      YORG=sngl(orig(2))
      ZORG=sngl(orig(3))
      XPITCH=sngl(delta(1))
      YPITCH=sngl(delta(2))
      ZPITCH=sngl(delta(3))
      STEP=nstep ; TIME=real(nstep)*delta(4) ;
!
      istat = ista ; iendt = iend
      jstat = jsta ; jendt = jend
      kstat = ksta ; kendt = kend
      IMAX_G = nx
      JMAX_G = ny
      KMAX_G = nz
      select case ( dir )
      case (1)                  ! +++ I-direction
         if ( ( istat <= ics ) .and. ( ics <= iendt ) ) then
            istat = ics
            iendt = ics
            IMAX_G = 1
            XORG=XORG+XPITCH*(ics-ista)
         else
            return
         endif
      case (2)                  ! +++ J-direction
         if ( ( jstat <= ics ) .and. ( ics <= jendt ) ) then
            jstat = ics
            jendt = ics
            JMAX_G = 1
            YORG=YORG+YPITCH*(ics-jsta)
         else
            return
         endif
      case(3)                   ! +++ K-direction
         if ( ( kstat <= ics ) .and. ( ics <= kendt ) ) then
            kstat = ics
            kendt = ics
            KMAX_G = 1
            ZORG=ZORG+ZPITCH*(ics-ksta)
         else
            return
         endif
      end select
! ---
      allocate( SDATA( istat:iendt, jstat:jendt, kstat:kendt ) )
! Scalar
      svType=1                  ! svType(1:scalar, 2:vector)
      dType=1                   ! dType(1:single, 2:double)
      IMAX=iendt-istat+1
      JMAX=jendt-jstat+1
      KMAX=kendt-kstat+1
      do k=kstat,kendt ; do j=jstat,jendt ; do i=istat,iendt
         SDATA(i,j,k)=sngl(sp(i,j,k))
      enddo ; enddo ; enddo
!
      open (10, file=fname,status='unknown',form='unformatted')
      write(10) svType, dType
      write(10) IMAX, JMAX, KMAX
      write(10) XORG, YORG, ZORG
      write(10) XPITCH, YPITCH, ZPITCH
      write(10) STEP, TIME
      write(10) (((SDATA(i,j,k), i=istat,iendt)
     $                         , j=jstat,jendt)
     $                         , k=kstat,kendt)
      write(10) IMAX_G, JMAX_G, KMAX_G
      write(10) ISTAT, JSTAT, KSTAT
      close(10)
!
      deallocate( SDATA )
!
      return
      end subroutine dtsaveS2D

#ifdef _TEST_CHECK_DATA_

! ----------------------------------------------------------------------
      subroutine Input_TestData ( omega, rho, cs )
! ----------------------------------------------------------------------
      use Array_Index
      use Grid
      use FDM
      use HIFU3D
      implicit none
      real*8, intent(in) :: omega, rho, cs
      real*8 :: kw
      real*8 :: xm, ym, zm
      real*8 :: sx, sy, sz
      integer*4 :: i, j, k
      integer*4 :: m
!
      kw = omega / cs
!
! P
      do k = ksta1m, kend1m
         zm = ( dble(k) - .5d0 )*dz
         sz = rho * cs * dsin( kw * zm )
      do j = jsta1m, jend1m
         ym = ( dble(j) - .5d0 )*dy
         sy = rho * cs * dsin( kw * ym )
      do i = ista1m, iend1m
         xm = ( dble(i) - .5d0 )*dx
         sx = rho * cs * dsin( kw * xm )
         px(i,j,k) = sx
         py(i,j,k) = sy
         pz(i,j,k) = sz
         p0(i,j,k) = sx + sy + sz
      enddo
      enddo
      enddo
!
! U
      do k = kstam , kendm
      do j = jstam , jendm
      do i = istan , iendn
#ifdef _UNROLL_FDM_
         sx =  d1n(ms  )*p0(i+ms  ,j,k)
     $        +d1n(ms+1)*p0(i+ms+1,j,k)
     $        +d1n(ms+2)*p0(i+ms+2,j,k)
     $        +d1n(ms+3)*p0(i+ms+3,j,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*p0(i+ms+4,j,k)
     $        +d1n(ms+5)*p0(i+ms+5,j,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*p0(i+ms+6,j,k)
     $        +d1n(ms+7)*p0(i+ms+7,j,k)
#endif
#else /* _UNROLL_FDM_ */
         sx = 0.d0
         do m = ms, me
            sx = sx + d1n(m)*p0  (i+m,j,k)
         enddo
#endif /* _UNROLL_FDM_ */
         u(i,j,k) = u(i,j,k) + .5d0*dt * sx / rho
      enddo
      enddo
      enddo
!
! V
      do k = kstam , kendm
      do j = jstan , jendn
      do i = istam , iendm
#ifdef _UNROLL_FDM_
         sy =  d1n(ms  )*p0(i,j+ms  ,k)
     $        +d1n(ms+1)*p0(i,j+ms+1,k)
     $        +d1n(ms+2)*p0(i,j+ms+2,k)
     $        +d1n(ms+3)*p0(i,j+ms+3,k)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*p0(i,j+ms+4,k)
     $        +d1n(ms+5)*p0(i,j+ms+5,k)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*p0(i,j+ms+6,k)
     $        +d1n(ms+7)*p0(i,j+ms+7,k)
#endif
#else /* _UNROLL_FDM_ */
         sy = 0.d0
         do m = ms, me
            sy = sy + d1n(m)*p0  (i,j+m,k)
         enddo
#endif /* _UNROLL_FDM_ */
         v(i,j,k) = v(i,j,k) + .5d0*dt * sy / rho
      enddo
      enddo
      enddo
!
! W
      do k = kstan , kendn
      do j = jstam , jendm
      do i = istam , iendm
#ifdef _UNROLL_FDM_
         sz =  d1n(ms  )*p0(i,j,k+ms  )
     $        +d1n(ms+1)*p0(i,j,k+ms+1)
     $        +d1n(ms+2)*p0(i,j,k+ms+2)
     $        +d1n(ms+3)*p0(i,j,k+ms+3)
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 6 )
     $        +d1n(ms+4)*p0(i,j,k+ms+4)
     $        +d1n(ms+5)*p0(i,j,k+ms+5)
#endif
#if defined(_FDM_ORDER_) && ( _FDM_ORDER_ >= 8 )
     $        +d1n(ms+6)*p0(i,j,k+ms+6)
     $        +d1n(ms+7)*p0(i,j,k+ms+7)
#endif
#else /* _UNROLL_FDM_ */
         sz = 0.d0
         do m = ms, me
            sz = sz + d1n(m)*p0  (i,j,k+m)
         enddo
#endif /* _UNROLL_FDM_ */
         w(i,j,k) = w(i,j,k) + .5d0*dt * sz / rho
      enddo
      enddo
      enddo
!
#ifdef _DENATURATION_
! Phi
      do k = ksta1m, kend1m
         zm = ( dble(k) - .5d0 )*dz
         sz = dsin( kw * zm )
      do j = jsta1m, jend1m
         ym = ( dble(j) - .5d0 )*dy
         sy = dsin( kw * ym )
      do i = ista1m, iend1m
         xm = ( dble(i) - .5d0 )*dx
         sx = dsin( kw * xm )
         phi(i,j,k) = ( sx + sy + sz )/3.d0
      enddo
      enddo
      enddo
#endif
!
      return
      end subroutine Input_TestData

!TUNE0 **************************************************************>>>
!ocl nofp_relaxed
!TUNE0 END<<<***********************************************************
! ----------------------------------------------------------------------
      subroutine Check_TestResults
! ----------------------------------------------------------------------
      use Array_Index
      use MyMPI
      use HIFU3D
      implicit none
!
#ifdef _USE_MPI_
      include 'mpif.h'
      integer :: ierr
#endif
!
      real*8 :: smaxl, sminl, srmsl
      real*8 :: smaxg, sming, srmsg
      integer :: i, j, k
!
#if defined( _USE_MPI_ )
#define _CHECK_DATA_(_SD_) \
      smaxl = _SD_(istam, jstam, kstam) ;\
      sminl = _SD_(istam, jstam, kstam) ;\
      srmsl = 0.d0 ;\
      do k = kstam , kendm ;\
      do j = jstam , jendm ;\
      do i = istam , iendm ;\
         smaxl = dmax1( smaxl, _SD_(i,j,k) ) ;\
         sminl = dmin1( sminl, _SD_(i,j,k) ) ;\
         srmsl = srmsl + _SD_(i,j,k)**2 ;\
      enddo ;\
      enddo ;\
      enddo ;\
      call MPI_ALLREDUCE(smaxl,smaxg,1,mpi_real8,mpi_max,mpi_comm_world,ierr) ;\
      call MPI_ALLREDUCE(sminl,sming,1,mpi_real8,mpi_min,mpi_comm_world,ierr) ;\
      call MPI_ALLREDUCE(srmsl,srmsg,1,mpi_real8,mpi_sum,mpi_comm_world,ierr) ;\
      srmsg = dsqrt( srmsg / dble( (_NX_)*(_NY_)*(_NZ_) ) )
! /* _CHECK_DATA_(_SD_) */
#else
#define _CHECK_DATA_(_SD_) \
      smaxl = _SD_(istam, jstam, kstam) ;\
      sminl = _SD_(istam, jstam, kstam) ;\
      srmsl = 0.d0 ;\
      do k = kstam , kendm ;\
      do j = jstam , jendm ;\
      do i = istam , iendm ;\
         smaxl = dmax1( smaxl, _SD_(i,j,k) ) ;\
         sminl = dmin1( sminl, _SD_(i,j,k) ) ;\
         srmsl = srmsl + _SD_(i,j,k)**2 ;\
      enddo ;\
      enddo ;\
      enddo ;\
      smaxg = smaxl ;\
      sming = sminl ;\
      srmsg = srmsl ;\
      srmsg = dsqrt( srmsg / dble( (_NX_)*(_NY_)*(_NZ_) ) )
#endif /* _USE_MPI_ */
!
      if ( myrank == 0 ) write(*,*) '<<<<< Check Results <<<<<'
! P
      _CHECK_DATA_(p0)
      if ( myrank == 0 ) write(*,*) 'P   :', smaxg, sming, srmsg
!
! U
      _CHECK_DATA_(u)
      if ( myrank == 0 ) write(*,*) 'U   :', smaxg, sming, srmsg
!
! V
      _CHECK_DATA_(v)
      if ( myrank == 0 ) write(*,*) 'V   :', smaxg, sming, srmsg
!
! W
      _CHECK_DATA_(w)
      if ( myrank == 0 ) write(*,*) 'W   :', smaxg, sming, srmsg
!
#ifdef _TEMPERATURE_
! T
      _CHECK_DATA_(T)
      if ( myrank == 0 ) write(*,*) 'T   :', smaxg, sming, srmsg
#endif
!
#ifdef _DENATURATION_
! Phi
      _CHECK_DATA_( phi )
      if ( myrank == 0 ) write(*,*) 'Phi :', smaxg, sming, srmsg
#endif
!
      if ( myrank == 0 ) write(*,*) '>>>>> Check Results >>>>>'
!
      return
      end subroutine Check_TestResults

#endif /* _TEST_CHECK_DATA_ */
